@@ -35,18 +35,11 @@ the :class:`.PulsarServerCommands` handler.
    :member-order: bysource
 
 '''
-try:
-    import pulsar
-except ImportError:  # pragma nocover
-    import sys
-    sys.path.append('../../')
-
 from random import normalvariate
 
-from pulsar import coroutine_return, task
+from pulsar import as_coroutine
 from pulsar.apps import rpc, wsgi
 from pulsar.utils.httpurl import JSON_CONTENT_TYPES
-from pulsar.utils.pep import range
 
 
 def divide(request, a, b):
@@ -77,11 +70,10 @@ def randompaths(request, num_paths=1, size=250, mu=0, sigma=1):
 
 class RequestCheck:
 
-    @task
     def __call__(self, request, name):
-        data = yield request.body_data()
+        data = yield from as_coroutine(request.body_data())
         assert(data['method'] == name)
-        coroutine_return(True)
+        return True
 
 
 class Root(rpc.PulsarServerCommands):
@@ -124,8 +116,10 @@ class Site(wsgi.LazyWsgi):
         middleware = wsgi.Router('/', post=json_handler,
                                  accept_content_types=JSON_CONTENT_TYPES)
         response = [wsgi.GZipMiddleware(200)]
-        return wsgi.WsgiHandler(middleware=[middleware],
-                                response_middleware=response)
+        return wsgi.WsgiHandler(middleware=[wsgi.wait_for_body_middleware,
+                                            middleware],
+                                response_middleware=response,
+                                async=True)
 
 
 def server(callable=None, **params):
