@@ -38,14 +38,9 @@ Implementation
    :member-order: bysource
 '''
 import os
-import sys
 import time
-try:
-    import pulsar
-except ImportError:  # pragma nocover
-    sys.path.append('../../')
 
-from pulsar import get_actor, Future
+from pulsar import Future, async
 from pulsar.apps.wsgi import (Router, WsgiHandler, LazyWsgi, WSGIServer,
                               GZipMiddleware)
 from pulsar.apps.ws import WS, WebSocket
@@ -113,7 +108,7 @@ class Chat(WS):
                     lines.append(l)
             msg = ' '.join(lines)
             if msg:
-                self.pubsub.publish(self.channel, msg)
+                return self.pubsub.publish(self.channel, msg)
 
 
 #    RPC MIDDLEWARE To publish messages
@@ -126,7 +121,7 @@ class Rpc(PulsarServerCommands):
 
     def rpc_message(self, request, message):
         '''Publish a message via JSON-RPC'''
-        self.pubsub.publish(self.channel, message)
+        yield from self.pubsub.publish(self.channel, message)
         return 'OK'
 
 
@@ -147,13 +142,14 @@ class WebChat(LazyWsgi):
         self.store = create_store(cfg.data_store, loop=loop)
         pubsub = self.store.pubsub(protocol=Protocol())
         channel = '%s_webchat' % self.name
-        pubsub.subscribe(channel)
+        async(pubsub.subscribe(channel), loop=loop)
         return WsgiHandler([Router('/', get=self.home_page),
                             WebSocket('/message', Chat(pubsub, channel)),
                             Router('/rpc', post=Rpc(pubsub, channel),
                                    response_content_types=JSON_CONTENT_TYPES)],
                            [AsyncResponseMiddleware,
-                            GZipMiddleware(min_length=20)])
+                            GZipMiddleware(min_length=20)],
+                           async=True)
 
     def home_page(self, request):
         data = open(os.path.join(CHAT_DIR, 'chat.html')).read()
@@ -172,11 +168,7 @@ def AsyncResponseMiddleware(environ, resp):
 
 def server(callable=None, name=None, data_store=None, **params):
     name = name or 'wsgi'
-    if not data_store:
-        actor = get_actor()
-        if actor:
-            data_store = actor.cfg.data_store
-        data_store = pulsards_url(data_store)
+    data_store = pulsards_url(data_store)
     return WSGIServer(callable=WebChat(name), name=name,
                       data_store=data_store, **params)
 
