@@ -23,14 +23,11 @@ import os
 import sys
 import string
 import mimetypes
+from functools import partial
 from itertools import repeat, chain
 from random import random
-
-try:
-    from pulsar.utils.pep import ispy3k, range
-except ImportError:     # pragma    nocover
-    sys.path.append('../../')
-    from pulsar.utils.pep import ispy3k, range
+from base64 import b64encode
+from asyncio import async
 
 from pulsar import HttpRedirect, HttpException, version, JAPANESE, CHINESE
 from pulsar.utils.httpurl import (Headers, ENCODE_URL_METHODS,
@@ -38,7 +35,7 @@ from pulsar.utils.httpurl import (Headers, ENCODE_URL_METHODS,
 from pulsar.utils.html import escape
 from pulsar.apps import wsgi, ws
 from pulsar.apps.wsgi import (route, Html, Json, HtmlDocument, GZipMiddleware,
-                              AsyncString)
+                              String)
 from pulsar.utils.structures import MultiValueDict
 from pulsar.utils.system import json
 
@@ -47,11 +44,7 @@ METHODS = frozenset(chain((m.lower() for m in ENCODE_URL_METHODS),
 pyversion = '.'.join(map(str, sys.version_info[:3]))
 ASSET_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'assets')
 FAVICON = os.path.join(ASSET_DIR, 'favicon.ico')
-
-if ispy3k:  # pragma nocover
-    characters = string.ascii_letters + string.digits
-else:   # pragma nocover
-    characters = string.letters + string.digits
+characters = string.ascii_letters + string.digits
 
 
 def asset(name, mode='r', chunk_size=None):
@@ -94,14 +87,18 @@ class BaseRouter(wsgi.Router):
         else:
             args, files = request.data_and_files()
             jfiles = MultiValueDict()
-            for name, parts in files.lists():
-                for part in parts:
-                    try:
-                        part = part.string()
-                    except UnicodeError:
-                        part = part.base64()
-                    jfiles[name] = part
-            data.update((('args', dict(args)),
+            if files:
+                for name, parts in files.lists():
+                    for part in parts:
+                        try:
+                            part = part.string()
+                        except UnicodeError:
+                            part = part.base64()
+                        jfiles[name] = part
+            if isinstance(args, MultiValueDict):
+                args = dict(args)
+
+            data.update((('args', args),
                          ('files', dict(jfiles))))
         data.update(params)
         return data
@@ -136,7 +133,6 @@ class HttpBin(BaseRouter):
         html.head.title = title
         html.head.links.append('httpbin.css')
         html.head.links.append('favicon.ico', rel="icon", type='image/x-icon')
-        html.head.scripts.append('jquery')
         html.head.scripts.append('httpbin.js')
         ul = ul.render(request)
         templ, _, _ = asset('template.html')
@@ -152,6 +148,14 @@ class HttpBin(BaseRouter):
     def post_post(self, request):
         return self.info_data_response(request)
 
+    @route(title='Returns Post bytes data')
+    def post_post_chunks(self, request):
+        data, _ = request.data_and_files()
+        content_type = request.get('CONTENT_TYPE')
+        request.response.content_type = content_type
+        request.response.content = data
+        return request.response
+
     @route(title='Returns PATCH data')
     def patch_patch(self, request):
         return self.info_data_response(request)
@@ -206,7 +210,8 @@ class HttpBin(BaseRouter):
            defaults={'status': 418})
     def status(self, request):
         request.response.content_type = 'text/html'
-        raise HttpException(status=request.urlargs['status'])
+        msg = request.url_data.get('message', 'test error')
+        raise HttpException(msg, status=request.urlargs['status'])
 
     @route(title='Returns response headers')
     def response_headers(self, request):
@@ -295,6 +300,12 @@ class HttpBin(BaseRouter):
         response.content = data
         return response
 
+    @route('servername',
+           title='display the server name')
+    def servername(self, request):
+        name = request.get('SERVER_NAME')
+        return String(name, '\n').http_response(request)
+
     ########################################################################
     #    BENCHMARK ROUTES
     @route()
@@ -303,17 +314,53 @@ class HttpBin(BaseRouter):
 
     @route()
     def plaintext(self, request):
-        return AsyncString('Hello, World!').http_response(request)
+        return String('Hello, World!').http_response(request)
+
+
+class Upload(BaseRouter):
+    response_content_types = ['multipart/form-data']
+
+    def put(self, request):
+        return async(self._async_put(request))
+
+    def _async_put(self, request):
+        headers = self.getheaders(request)
+        data = {'method': request.method,
+                'headers': headers,
+                'pulsar': self.pulsar_info(request),
+                'args': MultiValueDict(),
+                'files': MultiValueDict()}
+        request.cache.response_data = data
+        yield from request.data_and_files(stream=partial(self.stream, request))
+        data['args'] = dict(data['args'])
+        data['files'] = dict(data['files'])
+        return Json(data).http_response(request)
+
+    def stream(self, request, part):
+        if request.cache.current_data is not part:
+            request.cache.current_data = part
+            request.cache.current_data_buffer = []
+
+        request.cache.current_data_buffer.append(part.recv())
+
+        if part.complete():
+            data_store = request.cache.response_data
+            data = b''.join(request.cache.current_data_buffer)
+            store = data_store['args']
+            if part.is_file():
+                store = data_store['files']
+                try:
+                    data.decode('utf-8')
+                except UnicodeError:
+                    data = b64encode(data)
+
+            store[part.name] = data.decode('utf-8')
 
 
 class ExpectFail(BaseRouter):
 
     def post(self, request):
-        chunk = request.get('wsgi.input')
-        if not chunk.done():
-            chunk.fail()
-        else:
-            return self.info_data_response(request)
+        request.get('wsgi.input').fail()
 
 
 class Graph(ws.WS):
@@ -327,13 +374,15 @@ class Site(wsgi.LazyWsgi):
     def setup(self, environ):
         router = HttpBin('/')
         return wsgi.WsgiHandler([ExpectFail('expect'),
+                                 Upload('upload'),
                                  wsgi.wait_for_body_middleware,
                                  wsgi.clean_path_middleware,
                                  wsgi.authorization_middleware,
                                  wsgi.MediaRouter('media', ASSET_DIR,
                                                   show_indexes=True),
                                  ws.WebSocket('/graph-data', Graph()),
-                                 router])
+                                 router],
+                                async=True)
 
 
 def server(description=None, **kwargs):
