@@ -1,9 +1,11 @@
+from config import *    # noqa
+
 twitter_api_key = 's8KXEZziHgqxTQy962p4Q'
 twitter_api_secret = 'FL2A3pzdKLfznC7o3HAYaKqHMdpuGmUGlJYnVTwpsA'
 twitter_access_token = '19934307-zaUhqhvq79PjL62eTsOSQeTXAn9Omv2Tg6AxxHBCI'
 twitter_access_secret = 'bR4hzVCyfmmuH6wAibCadJ7sedNQUuxkliwSFxCLd4'
 
-twitter_stream_filter = {'track': 'wine'}
+twitter_stream_filter = {'track': 'python'}
 
 
 def test_twitter():
@@ -15,11 +17,11 @@ def test_twitter():
         print(body)
 
     http = HttpClient(encode_multipart=False)
-    oauth2 = OAuth1(twitter_api_key,
+    oauth1 = OAuth1(twitter_api_key,
                     client_secret=twitter_api_secret,
                     resource_owner_key=twitter_access_token,
                     resource_owner_secret=twitter_access_secret)
-    http.bind_event('pre_request', oauth2)
+    http.bind_event('pre_request', oauth1)
     response = http.post(url, data={'track': 'twitter'},
                          data_processed=new_data)
     print(response.status_code)
