@@ -1,29 +1,27 @@
 # -*- coding: utf-8 -
 '''Event driven concurrent framework for Python'''
-VERSION = (0, 9, 2, 'final', 0)
-
 import os
 
 from .utils.version import get_version
 
+
+VERSION = (1, 1, 1, 'final', 0)
+
 __version__ = version = get_version(VERSION)
 __license__ = "BSD"
 __author__ = "Luca Sbardella"
 __contact__ = "luca.sbardella@gmail.com"
 __homepage__ = "https://github.com/quantmind/pulsar"
 __docformat__ = "restructuredtext"
-CLASSIFIERS = ['Development Status :: 4 - Beta',
+CLASSIFIERS = ['Development Status :: 5 - Production/Stable',
                'Environment :: Web Environment',
                'Intended Audience :: Developers',
                'License :: OSI Approved :: BSD License',
                'Operating System :: OS Independent',
                'Programming Language :: Python',
-               'Programming Language :: Python :: 2',
-               'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 3',
-               'Programming Language :: Python :: 3.3',
                'Programming Language :: Python :: 3.4',
-               'Programming Language :: Python :: Implementation :: PyPy',
+               'Programming Language :: Python :: 3.5',
                'Topic :: Internet',
                'Topic :: Utilities',
                'Topic :: System :: Distributed Computing',
@@ -47,19 +45,17 @@ if os.environ.get('pulsar_setup_running') != 'yes':
     else:
         HAS_C_EXTENSIONS = True
         try:
-            from .utils import lib
+            from .utils import lib      # noqa
         except ImportError:
             HAS_C_EXTENSIONS = False
 
-    from .utils.exceptions import *
-    from .utils import system
-    platform = system.platform
-    from .utils.config import *
-    from .async import *
-    from .apps import *
+    from .utils.exceptions import *     # noqa
+    from .utils import system           # noqa
+    platform = system.platform          # noqa
+    from .utils.config import *         # noqa
+    from .async import *                # noqa
+    from .apps import *                 # noqa
 
     del get_version
     # Import data stores
-    from .apps.data import stores
-    del stores
-    from .apps.data import data_stores
+    from .apps.data import data_stores  # noqa
