@@ -62,12 +62,11 @@ import os
 import sys
 from inspect import getfile
 from functools import partial
-from collections import namedtuple
+from collections import namedtuple, OrderedDict
 
 import pulsar
-from pulsar import (get_actor, coroutine_return, Config, task, From,
+from pulsar import (get_actor, Config, task,
                     multi_async, Future, ImproperlyConfigured)
-from pulsar.utils.structures import OrderedDict
 
 __all__ = ['Application', 'MultiApp', 'get_application', 'when_monitor_start']
 
@@ -93,18 +92,18 @@ def get_application(name):
 
 
 def _get_remote_app(actor, name):
-    cfg = yield From(actor.send('arbiter', 'run', _get_app, name))
-    coroutine_return(cfg.app() if cfg else None)
+    cfg = yield from actor.send('arbiter', 'run', _get_app, name)
+    return cfg.app() if cfg else None
 
 
 def _get_app(arbiter, name, safe=True):
     monitor = arbiter.get_actor(name)
     if monitor:
-        cfg = yield From(monitor.start_event)
+        cfg = yield from monitor.start_event
         if safe:
-            coroutine_return(cfg)
+            return cfg
         else:
-            coroutine_return(monitor.app)
+            return monitor.app
 
 
 @task
@@ -119,14 +118,22 @@ def monitor_start(self, exc=None):
         self.bind_event('on_info', monitor_info)
         self.bind_event('stopping', monitor_stopping)
         for callback in when_monitor_start:
-            yield callback(self)
+            coro = callback(self)
+            if coro:
+                yield from coro
         self.bind_event('periodic_task', app.monitor_task)
-        yield app.monitor_start(self)
+        coro = app.monitor_start(self)
+        if coro:
+            yield from coro
         if not self.cfg.workers:
-            yield app.worker_start(self)
+            coro = app.worker_start(self)
+            if coro:
+                yield from coro
         result = self.cfg
     except Exception as exc:
-        yield self.stop(exc)
+        coro = self.stop(exc)
+        if coro:
+            yield from coro
         start_event.set_result(None)
     else:
         start_event.set_result(result)
@@ -135,9 +142,13 @@ def monitor_start(self, exc=None):
 @task
 def monitor_stopping(self, exc=None):
     if not self.cfg.workers:
-        yield self.app.worker_stopping(self)
-    yield self.app.monitor_stopping(self)
-    coroutine_return(self)
+        coro = self.app.worker_stopping(self)
+        if coro:
+            yield from coro
+    coro = self.app.monitor_stopping(self)
+    if coro:
+        yield from coro
+    return self
 
 
 def monitor_info(self, info=None):
@@ -268,10 +279,9 @@ class Configurator(object):
         return self.__repr__()
 
     def python_path(self, script):
-        '''Called during initialisation to obtain the ``script`` name and
-        to add the :attr:`script` directory to the python path if not in the
-        path already.
-        If ``script`` does not evalueate to ``True`` it is evaluated from
+        '''Called during initialisation to obtain the ``script`` name.
+
+        If ``script`` does not evaluate to ``True`` it is evaluated from
         the ``__main__`` import. Returns the real path of the python
         script which runs the application.
         '''
@@ -282,9 +292,10 @@ class Configurator(object):
             except Exception:  # pragma    nocover
                 return
         script = os.path.realpath(script)
-        path = os.path.dirname(script)
-        if path not in sys.path:
-            sys.path.insert(0, path)
+        if self.cfg.get('python_path', True):
+            path = os.path.dirname(script)
+            if path not in sys.path:
+                sys.path.insert(0, path)
         return script
 
     def on_config(self, arbiter):
@@ -327,24 +338,9 @@ class Configurator(object):
                     self.cfg.set(name, value)
         # parse console args
         if self.console_parsed:
-            parser = self.cfg.parser()
-            opts = parser.parse_args(self.argv)
-            config = getattr(opts, 'config', None)
-            # set the config only if config is part of the settings
-            if config is not None and self.cfg.config:
-                self.cfg.set('config', config)
+            self.cfg.parse_command_line(self.argv)
         else:
-            parser, opts = None, None
-        #
-        # Load up the config file if found.
-        self.cfg.params.update(self.cfg.import_from_module())
-        #
-        # Update the configuration with any command line settings.
-        if opts:
-            for k, v in opts.__dict__.items():
-                if v is None:
-                    continue
-                self.cfg.set(k.lower(), v)
+            self.cfg.params.update(self.cfg.import_from_module())
 
     def start(self):
         '''Invoked the application callable method and start
@@ -378,7 +374,7 @@ class Configurator(object):
                 cfg[prefix] = prefix
             cfg = Config(**cfg)
         cfg.update_settings()
-        cfg.update(params)
+        cfg.update(params, True)
         return cfg
 
 
@@ -417,8 +413,9 @@ class Application(Configurator):
         Default ``None``
     """
     def __init__(self, callable=None, load_config=True, **params):
-        super(Application, self).__init__(load_config=load_config, **params)
+        super().__init__(load_config=load_config, **params)
         self.cfg.callable = callable
+        self.logger = None
         if load_config:
             self.load_config()
 
@@ -473,6 +470,17 @@ class Application(Configurator):
                 return
         raise ImproperlyConfigured('Already started or not in arbiter domain')
 
+    def stop(self, actor=None):
+        '''Stop the application
+        '''
+        if actor is None:
+            actor = get_actor()
+        if actor and actor.is_arbiter():
+            monitor = actor.get_actor(self.name)
+            if monitor:
+                return monitor.stop()
+        raise RuntimeError('Cannot stop application')
+
     # WORKERS CALLBACKS
     def worker_start(self, worker, exc=None):
         '''Added to the ``start`` :ref:`worker hook <actor-hooks>`.'''
@@ -535,9 +543,6 @@ class MultiApp(Configurator):
 
     It makes sure all :ref:`settings <settings>` for the
     applications created are available in the command line.
-    Check the :class:`~examples.taskqueue.manage.server` class in the
-    :ref:`taskqueue example <tutorials-taskqueue>` for an actual
-    implementation.
 
     :class:`MultiApp` derives from :class:`Configurator` and therefore
     supports all its configuration utilities,
