@@ -1,13 +1,25 @@
+'''
+Error classes and redis parser function
+'''
 import pulsar
 
 from .pyparser import Parser
 
 
-class ResponseError(pulsar.PulsarException):
+class RedisError(pulsar.PulsarException):
+    '''Redis Error Base class'''
     pass
 
 
-class InvalidResponse(pulsar.PulsarException):
+class CommandError(RedisError):
+    pass
+
+
+class ResponseError(RedisError):
+    pass
+
+
+class InvalidResponse(RedisError):
     pass
 
 
@@ -31,12 +43,16 @@ def response_error(response):
     return EXCEPTION_CLASSES[error_code](response)
 
 
-PyRedisParser = lambda: Parser(InvalidResponse, response_error)
+def PyRedisParser():
+    return Parser(InvalidResponse, response_error)
 
 
 if pulsar.HAS_C_EXTENSIONS:
     from pulsar.utils.lib import RedisParser as _RedisParser
-    RedisParser = lambda: _RedisParser(InvalidResponse, response_error)
+
+    def RedisParser():
+        return _RedisParser(InvalidResponse, response_error)
+
 else:    # pragma nocover
     RedisParser = PyRedisParser
 
