@@ -44,35 +44,36 @@ import os
 import re
 import time
 import math
+import pickle
 from random import choice
-from hashlib import sha1
 from itertools import islice, chain
 from functools import partial, reduce
 from collections import namedtuple
+from itertools import zip_longest
 
 import pulsar
 from pulsar.apps.socket import SocketServer
 from pulsar.utils.config import Global
 from pulsar.utils.structures import Dict, Zset, Deque
-from pulsar.utils.pep import map, range, zip, ispy3k, pickle
-try:
-    from pulsar.utils.lua import Lua
-except ImportError:     # pragma    nocover
-    Lua = None
-
 
 from .parser import redis_parser
 from .utils import sort_command, count_bytes, and_op, or_op, xor_op, save_data
-from .client import (command, PulsarStoreClient, LuaClient, Blocked,
+from .client import (command, PulsarStoreClient, Blocked,
                      COMMANDS_INFO, check_input, redis_to_py_pattern)
 
 
 DEFAULT_PULSAR_STORE_ADDRESS = '127.0.0.1:6410'
 
 
-def pulsards_url(address=None, db=3):
+def pulsards_url(address=None):
+    if not address:
+        actor = pulsar.get_actor()
+        if actor:
+            address = actor.cfg.data_store
     address = address or DEFAULT_PULSAR_STORE_ADDRESS
-    return 'pulsar://%s/%s' % (address, db)
+    if not address.startswith('pulsar://'):
+        address = 'pulsar://%s' % address
+    return address
 
 
 # Keyspace changes notification classes
@@ -80,13 +81,6 @@ STRING_LIMIT = 2**32
 
 nan = float('nan')
 
-if ispy3k:
-    from itertools import zip_longest
-    _ord = lambda x: x
-else:   # pragma    nocover
-    from itertools import izip_longest as zip_longest
-    _ord = lambda x: ord(x)
-
 
 class RedisParserSetting(Global):
     name = "redis_py_parser"
@@ -139,7 +133,8 @@ class KeyValuePassword(PulsarDsSetting):
 
 class KeyValueSave(PulsarDsSetting):
     name = "key_value_save"
-    default = [(900, 1), (300, 10), (60, 10000)]
+    # default = [(900, 1), (300, 10), (60, 10000)]
+    default = []
     validator = validate_list_of_pairs
     desc = '''\
         List of pairs controlling data store persistence.
@@ -166,13 +161,13 @@ class KeyValueFileName(PulsarDsSetting):
 class TcpServer(pulsar.TcpServer):
 
     def __init__(self, cfg, *args, **kwargs):
-        super(TcpServer, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.cfg = cfg
         self._parser_class = redis_parser(cfg.redis_py_parser)
         self._key_value_store = Storage(self, cfg)
 
     def info(self):
-        info = super(TcpServer, self).info()
+        info = super().info()
         info.update(self._key_value_store._info())
         return info
 
@@ -195,7 +190,7 @@ class PulsarDS(SocketServer):
         cfg = self.cfg
         workers = min(1, cfg.workers)
         cfg.set('workers', workers)
-        return super(PulsarDS, self).monitor_start(monitor)
+        return super().monitor_start(monitor)
 
 
 # #############################################################################
@@ -254,7 +249,7 @@ class Storage(object):
                                 self.NOTIFY_HASH: self._hash_event,
                                 self.NOTIFY_LIST: self._list_event,
                                 self.NOTIFY_ZSET: self._zset_event}
-        self._set_options = (b'ex', b'px', 'nx', b'xx')
+        self._set_options = (b'ex', b'px', b'nx', b'xx')
         self.OK = b'+OK\r\n'
         self.QUEUED = b'+QUEUED\r\n'
         self.ZERO = b':0\r\n'
@@ -292,17 +287,10 @@ class Storage(object):
         self.databases = dict(((num, Db(num, self))
                                for num in range(cfg.key_value_databases)))
         # Initialise lua
-        if Lua:
-            self.lua = Lua()
-            self.scripts = {}
-            self.lua.register('redis', LuaClient(self),
-                              'call', 'pcall', 'error_reply', 'status_reply')
-            self.version = '2.6.16'
-        else:   # pragma    nocover
-            self.lua = None
-            self.version = '2.4.10'
+        self.lua = None
+        self.version = '2.4.10'
         self._loaddb()
-        pulsar.call_repeatedly(self._loop, 1, self._cron)
+        self._cron()
 
     # #########################################################################
     # #    KEYS COMMANDS
@@ -387,7 +375,7 @@ class Storage(object):
             if db2 is None:
                 raise ValueError
         except Exception:
-            return client.reply_zero()
+            return client.reply_error('index out of range')
         db = client.db
         value = db.get(key)
         if db2.exists(key) or value is None:
@@ -865,13 +853,13 @@ class Storage(object):
         db = client.db
         string = db.get(key)
         if string is None:
-            string = bytearray(b'\x00')
+            string = bytearray(b'')
             db._data[key] = string
         elif not isinstance(string, bytearray):
             return client.reply_wrongtype()
         N = len(string)
         if N < T:
-            string.extend((T + 1 - N)*b'\x00')
+            string.extend((T - N)*b'\x00')
         string[offset:T] = value
         self._signal(self.NOTIFY_STRING, db, request[0], key, 1)
         client.reply_int(len(string))
@@ -1293,9 +1281,7 @@ class Storage(object):
             client.reply_wrongtype()
         else:
             assert value
-            start = len(value)
             value.trim(start, end)
-            client.reply_ok()
             self._signal(self.NOTIFY_LIST, db, request[0], key,
                          start-len(value))
             client.reply_ok()
@@ -1569,10 +1555,10 @@ class Storage(object):
         else:
             min_value, max_value = request[2], request[3]
             include_min = include_max = True
-            if min_value and _ord(min_value[0]) == 40:
+            if min_value and min_value[0] == 40:
                 include_min = False
                 min_value = min_value[1:]
-            if max_value and _ord(max_value[0]) == 40:
+            if max_value and max_value[0] == 40:
                 include_max = False
                 max_value = max_value[1:]
             try:
@@ -1622,7 +1608,7 @@ class Storage(object):
                 start, end = self._range_values(value, request[2], request[3])
             except Exception:
                 return client.reply_error(self.SYNTAX_ERROR)
-            reverse = (request[0] == b'zrevrange')
+            # reverse = (request[0] == b'zrevrange')
             if N == 4:
                 if request[4].lower() == b'withscores':
                     result = []
@@ -1946,49 +1932,17 @@ class Storage(object):
 
     # #########################################################################
     # #    SCRIPTING
-    @command('Scripting', script=0)
+    @command('Scripting', supported=False)
     def eval(self, client, request, N):
-        check_input(request, N < 2)
-        if not self.lua:
-            return client.reply_error(self.SYNTAX_ERROR)
-        script = request[1]
-        self._eval_script(client, script, request)
+        client.reply_error(self.NOT_SUPPORTED)
 
-    @command('Scripting', script=0)
+    @command('Scripting', supported=False)
     def evalsha(self, client, request, N):
-        check_input(request, N < 2)
-        if not self.lua:
-            return client.reply_error(self.SYNTAX_ERROR)
-        script = self.scripts.get(request[2])
-        if script is None:
-            client.reply_error('the script is not available', 'NOSCRIPT')
-        else:
-            self._eval_script(client, script, request)
+        client.reply_error(self.NOT_SUPPORTED)
 
-    @command('Scripting', script=0)
+    @command('Scripting', supported=False)
     def script(self, client, request, N):
-        check_input(request, not N)
-        if not self.lua:
-            return client.reply_error(self.SYNTAX_ERROR)
-        scripts = self.scripts
-        subcommand = request[1].upper()
-        if subcommand == b'EXISTS':
-            check_input(request, N < 2)
-            result = [int(sha in scripts) for sha in request[2:]]
-            client.reply_multi_bulk(result)
-        elif subcommand == b'FLUSH':
-            check_input(request, N != 1)
-            scripts.clear()
-            client.reply_ok()
-        elif subcommand == b'KILL':
-            check_input(request, N != 1)
-            client.reply_ok()
-        elif subcommand == b'LOAD':
-            check_input(request, N != 2)
-            script = request[2]
-            sha = sha1(script).hexdigest().encode('utf-8')
-            scripts[sha] = script
-            client.reply_bulk(sha)
+        client.reply_error(self.NOT_SUPPORTED)
 
     # #########################################################################
     # #    CONNECTION COMMANDS
@@ -2168,12 +2122,13 @@ class Storage(object):
                 if gap >= interval and dirty >= changes:
                     self._save()
                     break
+        self._loop.call_later(1, self._cron)
 
     def _set(self, client, key, value, seconds=0, milliseconds=0,
              nx=False, xx=False):
         try:
             seconds = int(seconds)
-            milliseconds = 0.000001*int(milliseconds)
+            milliseconds = 0.001*int(milliseconds)
             if seconds < 0 or milliseconds < 0:
                 raise ValueError
         except Exception:
@@ -2408,10 +2363,10 @@ class Storage(object):
 
     def _score_values(self, min_value, max_value):
         include_min = include_max = True
-        if min_value and _ord(min_value[0]) == 40:
+        if min_value and min_value[0] == 40:
             include_min = False
             min_value = min_value[1:]
-        if max_value and _ord(max_value[0]) == 40:
+        if max_value and max_value[0] == 40:
             include_max = False
             max_value = max_value[1:]
         return float(min_value), include_min, float(max_value), include_max
@@ -2472,7 +2427,7 @@ class Storage(object):
 
     def _loaddb(self):
         filename = self._filename
-        if os.path.isfile(filename):
+        if self.cfg.key_value_save and os.path.isfile(filename):
             self.logger.info('loading data from "%s"', filename)
             with open(filename, 'rb') as file:
                 data = pickle.load(file)
@@ -2554,57 +2509,9 @@ class Storage(object):
         if remove:
             self._monitors.difference_update(remove)
 
-    def _eval_script(self, client, script, request):
-        try:
-            numkeys = int(request[2])
-            if numkeys > 0:
-                keys = request[3:3+numkeys]
-                if len(keys) < numkeys:
-                    raise ValueError
-            elif numkeys < 0:
-                raise ValueError
-            else:
-                keys = []
-        except Exception:
-            return client.reply_error(self.SYNTAX_ERROR)
-        args = request[3+numkeys:]
-        self.lua.set_global('KEYS', keys)
-        self.lua.set_global('ARGV', args)
-        result = self.lua.execute(script)
-        if isinstance(result, dict):
-            if len(result) == 1:
-                key = tuple(result)[0]
-                keyu = key.upper()
-                if keyu == b'OK':
-                    return client.reply_ok()
-                elif keyu == b'ERR':
-                    return client.reply_error(result[key])
-                else:
-                    return client.reply_multi_bulk(())
-            else:
-                array = []
-                index = 0
-                while result:
-                    index += 1
-                    value = result.pop(index, None)
-                    if value is None:
-                        break
-                    array.append(value)
-                result = array
-        if isinstance(result, list):
-            client.reply_multi_bulk(result)
-        elif result is True:
-            client.reply_one()
-        elif result is False:
-            client.reply_bulk()
-        elif isinstance(result, (int, float)):
-            client.reply_int(int(result))
-        else:
-            client.reply_bulk(result)
-
 
 class Db(object):
-    '''The database.
+    '''A database.
     '''
     def __init__(self, num, store):
         self.store = store
@@ -2641,7 +2548,7 @@ class Db(object):
             return self._data[key]
         elif key in self._expires:
             self.store._hit_keys += 1
-            return self._expires[key]
+            return self._expires[key][1]
         else:
             self.store._missed_keys += 1
             return default
