@@ -1,4 +1,4 @@
-'''Pulsar ships with a thread safe, fully featured, :class:`.HttpClient`
+'''Pulsar ships with a fully featured, :class:`.HttpClient`
 class for multiple asynchronous HTTP requests.
 
 To get started, one builds a client::
@@ -8,10 +8,6 @@ To get started, one builds a client::
 
 and than makes requests, in a coroutine::
 
-    response = yield http.get('http://www.bbc.co.uk')
-
-or (only in python 3)::
-
     response = yield from http.get('http://www.bbc.co.uk')
 
 Making requests
@@ -20,7 +16,7 @@ Pulsar HTTP client has no dependencies and an API similar to requests_::
 
     from pulsar.apps import http
     client = http.HttpClient()
-    response = yield client.get('https://github.com/timeline.json')
+    response = yield from client.get('https://github.com/timeline.json')
 
 ``response`` is a :class:`HttpResponse` object which contains all the
 information about the request and the result:
@@ -39,6 +35,17 @@ information about the request and the result:
 The :attr:`~.ProtocolConsumer.request` attribute of :class:`HttpResponse`
 is an instance of :class:`.HttpRequest`.
 
+Posting data and Parameters
+=============================
+
+You can attach parameters to the ``url`` by passing the
+``urlparams`` dictionary:
+
+    response = yield from http.get('http://bla.com',
+                                   urlparams={'page': 2, 'key': 'foo'})
+    response.request.full_url == 'http://bla.com?page=2&key=foo'
+
+
 .. _http-cookie:
 
 Cookie support
@@ -50,13 +57,13 @@ To disable cookie one can pass ``store_cookies=False`` during
 
 If a response contains some Cookies, you can get quick access to them::
 
-    >>> response = yield client.get(...)
+    >>> response = yield from client.get(...)
     >>> type(response.cookies)
     <type 'dict'>
 
 To send your own cookies to the server, you can use the cookies parameter::
 
-    response = client.get(..., cookies={'sessionid': 'test'})
+    response = yield from client.get(..., cookies={'sessionid': 'test'})
 
 
 .. _http-authentication:
@@ -77,7 +84,6 @@ TLS/SSL
 =================
 Supported out of the box::
 
-    client = HttpClient()
     client.get('https://github.com/timeline.json')
 
 you can include certificate file and key too, either
@@ -95,7 +101,17 @@ Streaming
 
 This is an event-driven client, therefore streaming support is native.
 
-To stream data received from the client one uses the
+The easyiest way to use streaming is to pass the ``stream=True`` parameter
+during a request and access the :attr:`HttpResponse.raw` attribute.
+For example::
+
+    response = yield from http.get(..., stream=True)
+    for data in response.raw:
+        # data can be a future or bytes
+
+The ``raw`` attribute is an iterable over bytes or Futures resulting in bytes.
+
+Another approach to streaming is to use the
 :ref:`data_processed <http-many-time-events>` event handler.
 For example::
 
@@ -128,7 +144,7 @@ websocket handler::
 
 The websocket response is obtained by::
 
-    ws = yield http.get('ws://...', websocket_handler=Echo())
+    ws = yield from http.get('ws://...', websocket_handler=Echo())
 
 .. _http-redirects:
 
@@ -262,45 +278,55 @@ OAuth2
 '''
 import os
 import platform
+import logging
+import asyncio
 from functools import partial
 from collections import namedtuple
-from base64 import b64encode
 from io import StringIO, BytesIO
+from urllib.parse import urlparse, parse_qsl, urlencode, urlunparse
+from http.client import responses
+try:
+    import ssl
+except ImportError:
+    ssl = None
 
 import pulsar
-from pulsar import (AbstractClient, Pool, coroutine_return, task, Connection,
-                    ProtocolConsumer, From)
+from pulsar import (AbortRequest, AbstractClient, Pool, Connection, is_async,
+                    ProtocolConsumer)
+from pulsar.utils import websocket
 from pulsar.utils.system import json
-from pulsar.utils.pep import native_str, is_string, to_bytes, ispy3k
+from pulsar.utils.pep import native_str, to_bytes
 from pulsar.utils.structures import mapping_iterator
-from pulsar.utils.websocket import SUPPORTED_VERSIONS
-from pulsar.utils.internet import CERT_NONE, SSLContext
-from pulsar.utils.multipart import parse_options_header
-from pulsar.utils.httpurl import (urlparse, parse_qsl, responses,
-                                  http_parser, ENCODE_URL_METHODS,
-                                  encode_multipart_formdata, urlencode,
+from pulsar.utils.httpurl import (http_parser, ENCODE_URL_METHODS,
+                                  encode_multipart_formdata,
                                   Headers, get_environ_proxies,
-                                  choose_boundary, urlunparse, request_host,
+                                  choose_boundary, request_host,
                                   is_succesful, HTTPError, URLError,
                                   get_hostport, cookiejar_from_dict,
-                                  host_no_default_port, DEFAULT_CHARSET,
-                                  JSON_CONTENT_TYPES)
+                                  host_no_default_port, http_chunks,
+                                  parse_options_header, JSON_CONTENT_TYPES)
 
-try:
-    from pulsar.apps import greenio
-except ImportError:
-    greenio = None
-
-from .plugins import (handle_cookies, handle_100, handle_101, handle_redirect,
-                      Tunneling, TooManyRedirects)
+from .plugins import (handle_cookies, handle_100, WebSocket, Redirect,
+                      Tunneling, TooManyRedirects, start_request,
+                      response_content)
 
 from .auth import Auth, HTTPBasicAuth, HTTPDigestAuth
 from .oauth import OAuth1, OAuth2
+from .stream import HttpStream, StreamConsumedError
+
+
+__all__ = ['HttpRequest', 'HttpResponse', 'HttpClient', 'HTTPDigestAuth',
+           'TooManyRedirects', 'Auth', 'OAuth1', 'OAuth2',
+           'HttpStream', 'StreamConsumedError']
 
 
 scheme_host = namedtuple('scheme_host', 'scheme netloc')
 tls_schemes = ('https', 'wss')
 
+LOGGER = logging.getLogger('pulsar.http')
+FORM_URL_ENCODED = 'application/x-www-form-urlencoded'
+MULTIPART_FORM_DATA = 'multipart/form-data'
+
 
 def guess_filename(obj):
     """Tries to guess the filename of the given object."""
@@ -309,6 +335,14 @@ def guess_filename(obj):
         return os.path.basename(name)
 
 
+def is_streamed(data):
+    try:
+        len(data)
+    except TypeError:
+        return True
+    return False
+
+
 class RequestBase(object):
     inp_params = None
     release_connection = True
@@ -338,26 +372,16 @@ class RequestBase(object):
     def get_full_url(self):
         return self.full_url
 
-
-if not ispy3k:  # pragma     nocover
-    _RequestBase = RequestBase
-
-    class RequestBase(_RequestBase):
-
-        def is_unverifiable(self):
-            return self.unverifiable
-
-        def get_origin_req_host(self):
-            return self.origin_req_host
-
-        def get_type(self):
-            return self.scheme
+    def write_body(self, transport):
+        pass
 
 
 class HttpTunnel(RequestBase):
     first_line = None
+    data = None
 
     def __init__(self, request, scheme, host):
+        self.status = 0
         self.request = request
         self.scheme = scheme
         self.host, self.port = get_hostport(scheme, host)
@@ -372,7 +396,6 @@ class HttpTunnel(RequestBase):
 
     @property
     def key(self):
-
         return self.request.key
 
     @property
@@ -399,6 +422,19 @@ class HttpTunnel(RequestBase):
     def remove_header(self, header_name):
         self.headers.pop(header_name, None)
 
+    def apply(self, response, handler):
+        '''Tunnel the connection if needed
+        '''
+        connection = response.connection
+        tunnel_status = getattr(connection, '_tunnel_status', 0)
+        if not tunnel_status:
+            connection._tunnel_status = 1
+            response.bind_event('pre_request', handler)
+        elif tunnel_status == 1:
+            connection._tunnel_status = 2
+            response._request = self
+            response.bind_event('on_headers', handler.on_headers)
+
 
 class HttpRequest(RequestBase):
     '''An :class:`HttpClient` request for an HTTP resource.
@@ -416,6 +452,13 @@ class HttpRequest(RequestBase):
 
         HTTP version for this request, usually ``HTTP/1.1``
 
+    .. attribute:: encode_multipart
+
+        If ``True`` (default), defaults POST data as ``multipart/form-data``.
+        Pass ``encode_multipart=False`` to default to
+        ``application/x-www-form-urlencoded``. In any case, this parameter is
+        overwritten by passing the correct content-type header.
+
     .. attribute:: history
 
         List of past :class:`.HttpResponse` (collected during redirects).
@@ -425,27 +468,34 @@ class HttpRequest(RequestBase):
         if ``True``, the :class:`HttpRequest` includes the
         ``Expect: 100-Continue`` header.
 
+    .. attribute:: stream
+
+        Allow for streaming body
+
     '''
     CONNECT = 'CONNECT'
     _proxy = None
     _ssl = None
     _tunnel = None
+    _write_done = False
 
-    def __init__(self, client, url, method, inp_params, headers=None,
-                 data=None, files=None, timeout=None, history=None,
+    def __init__(self, client, url, method, inp_params=None, headers=None,
+                 data=None, files=None, history=None, auth=None,
                  charset=None, encode_multipart=True, multipart_boundary=None,
                  source_address=None, allow_redirects=False, max_redirects=10,
                  decompress=True, version=None, wait_continue=False,
-                 websocket_handler=None, cookies=None, **ignored):
+                 websocket_handler=None, cookies=None, urlparams=None,
+                 stream=False, proxies=None, **ignored):
         self.client = client
         self._data = None
         self.files = files
-        self.inp_params = inp_params
+        self.urlparams = urlparams
+        self.inp_params = inp_params or {}
         self.unredirected_headers = Headers(kind='client')
-        self.timeout = timeout
         self.method = method.upper()
         self.full_url = url
-        self.set_proxy(None)
+        if urlparams:
+            self._encode_url(urlparams)
         self.history = history
         self.wait_continue = wait_continue
         self.max_redirects = max_redirects
@@ -457,17 +507,21 @@ class HttpRequest(RequestBase):
         self.multipart_boundary = multipart_boundary
         self.websocket_handler = websocket_handler
         self.source_address = source_address
+        self.stream = stream
         self.new_parser()
         if self._scheme in tls_schemes:
             self._ssl = client.ssl_context(**ignored)
+        if auth and not isinstance(auth, Auth):
+            auth = HTTPBasicAuth(*auth)
+        self.auth = auth
         self.headers = client.get_headers(self, headers)
         cookies = cookiejar_from_dict(client.cookies, cookies)
         if cookies:
             cookies.add_cookie_header(self)
         self.unredirected_headers['host'] = host_no_default_port(self._scheme,
                                                                  self._netloc)
-        client.set_proxy(self)
         self.data = data
+        self._set_proxy(proxies)
 
     @property
     def address(self):
@@ -490,13 +544,19 @@ class HttpRequest(RequestBase):
 
     @property
     def key(self):
-        return (self.scheme, self.host, self.port, self.timeout)
+        tunnel = self._tunnel.full_url if self._tunnel else None
+        return (self.scheme, self.host, self.port, tunnel)
 
     @property
     def proxy(self):
         '''Proxy server for this request.'''
         return self._proxy
 
+    @property
+    def tunnel(self):
+        '''Tunnel for this request.'''
+        return self._tunnel
+
     @property
     def netloc(self):
         if self._proxy:
@@ -532,11 +592,14 @@ class HttpRequest(RequestBase):
         self._data = self._encode_data(data)
 
     def first_line(self):
-        if not self._proxy and self.method != self.CONNECT:
+        if self._proxy:
+            if self.method == self.CONNECT:
+                url = self._netloc
+            else:
+                url = self.full_url
+        else:
             url = urlunparse(('', '', self.path or '/', self.params,
                               self.query, self.fragment))
-        else:
-            url = self.full_url
         return '%s %s %s' % (self.method, url, self.version)
 
     def new_parser(self):
@@ -544,27 +607,8 @@ class HttpRequest(RequestBase):
                                               decompress=self.decompress,
                                               method=self.method)
 
-    def set_proxy(self, scheme, *host):
-        if not host and scheme is None:
-            self.scheme = self._scheme
-            self._set_hostport(self._scheme, self._netloc)
-        else:
-            le = 2 + len(host)
-            if not le == 3:
-                raise TypeError(
-                    'set_proxy() takes exactly three arguments (%s given)'
-                    % le)
-            if not self._ssl:
-                self.scheme = scheme
-                self._set_hostport(scheme, host[0])
-                self._proxy = scheme_host(scheme, host[0])
-            else:
-                self._tunnel = HttpTunnel(self, scheme, host[0])
-
-    def _set_hostport(self, scheme, host):
-        self._tunnel = None
-        self._proxy = None
-        self.host, self.port = get_hostport(scheme, host)
+    def is_chunked(self):
+        return self.data and 'content-length' not in self.headers
 
     def encode(self):
         '''The bytes representation of this :class:`HttpRequest`.
@@ -574,17 +618,13 @@ class HttpRequest(RequestBase):
         '''
         # Call body before fist_line in case the query is changes.
         first_line = self.first_line()
-        body = self.data
-        if body and self.wait_continue:
+        if self.data and self.wait_continue:
             self.headers['expect'] = '100-continue'
-            body = None
         headers = self.headers
         if self.unredirected_headers:
             headers = self.unredirected_headers.copy()
             headers.update(self.headers)
         buffer = [first_line.encode('ascii'), b'\r\n',  bytes(headers)]
-        if body:
-            buffer.append(body)
         return b''.join(buffer)
 
     def add_header(self, key, value):
@@ -605,12 +645,24 @@ class HttpRequest(RequestBase):
     def remove_header(self, header_name):
         '''Remove ``header_name`` from this request.
         '''
-        self.headers.pop(header_name, None)
-        self.unredirected_headers.pop(header_name, None)
+        val1 = self.headers.pop(header_name, None)
+        val2 = self.unredirected_headers.pop(header_name, None)
+        return val1 or val2
 
     def add_unredirected_header(self, header_name, header_value):
         self.unredirected_headers[header_name] = header_value
 
+    def write_body(self, transport):
+        assert not self._write_done, 'Body already sent'
+        self._write_done = True
+        if not self.data:
+            return
+        if is_streamed(self.data):
+            asyncio.async(self._write_streamed_data(transport),
+                          loop=transport._loop)
+        else:
+            self._write_body_data(transport, self.data, True)
+
     # INTERNAL ENCODING METHODS
     def _encode_data(self, data):
         body = None
@@ -621,10 +673,18 @@ class HttpRequest(RequestBase):
             assert self.files is None, ('data cannot be bytes when files are '
                                         'present')
             body = data
-        elif is_string(data):
+        elif isinstance(data, str):
             assert self.files is None, ('data cannot be string when files are '
                                         'present')
             body = to_bytes(data, self.charset)
+        elif data and is_streamed(data):
+            assert self.files is None, ('data cannot be an iterator when '
+                                        'files are present')
+            if 'content-type' not in self.headers:
+                self.headers['content-type'] = 'application/octet-stream'
+            if 'content-length' not in self.headers:
+                self.headers['transfer-encoding'] = 'chunked'
+            return data
         elif data or self.files:
             if self.files:
                 body, content_type = self._encode_files(data)
@@ -656,7 +716,7 @@ class HttpRequest(RequestBase):
     def _encode_files(self, data):
         fields = []
         for field, val in mapping_iterator(data or ()):
-            if (is_string(val) or isinstance(val, bytes) or
+            if (isinstance(val, str) or isinstance(val, bytes) or
                     not hasattr(val, '__iter__')):
                 val = [val]
             for v in val:
@@ -680,7 +740,7 @@ class HttpRequest(RequestBase):
                 fp = v
             if isinstance(fp, bytes):
                 fp = BytesIO(fp)
-            elif is_string(fp):
+            elif isinstance(fp, str):
                 fp = StringIO(fp)
             if ft:
                 new_v = (fn, fp.read(), ft)
@@ -692,22 +752,74 @@ class HttpRequest(RequestBase):
 
     def _encode_params(self, data):
         content_type = self.headers.get('content-type')
-        # No content type given
+        # No content type given, chose one
         if not content_type:
             if self.encode_multipart:
-                return encode_multipart_formdata(
-                    data, boundary=self.multipart_boundary,
-                    charset=self.charset)
+                content_type = MULTIPART_FORM_DATA
             else:
-                content_type = 'application/x-www-form-urlencoded'
-                body = urlencode(data).encode(self.charset)
-        elif content_type in JSON_CONTENT_TYPES:
+                content_type = FORM_URL_ENCODED
+
+        if content_type in JSON_CONTENT_TYPES:
             body = json.dumps(data).encode(self.charset)
+        elif content_type == FORM_URL_ENCODED:
+            body = urlencode(data).encode(self.charset)
+        elif content_type == MULTIPART_FORM_DATA:
+            body, content_type = encode_multipart_formdata(
+                data, boundary=self.multipart_boundary, charset=self.charset)
         else:
             raise ValueError("Don't know how to encode body for %s" %
                              content_type)
         return body, content_type
 
+    def _write_body_data(self, transport, data, finish=False):
+        if self.is_chunked():
+            data = http_chunks(data, finish)
+        elif data:
+            data = (data,)
+        else:
+            return
+        for chunk in data:
+            transport.write(chunk)
+
+    def _write_streamed_data(self, transport):
+        for data in self.data:
+            if is_async(data):
+                data = yield from data
+            self._write_body_data(transport, data)
+        self._write_body_data(transport, b'', True)
+
+    # PROXY INTERNALS
+    def _set_proxy(self, proxies):
+        request_proxies = self.client.proxies.copy()
+        if proxies:
+            request_proxies.update(proxies)
+        self.proxies = request_proxies
+        self.scheme = self._scheme
+        self._set_hostport(self._scheme, self._netloc)
+        #
+        if self.scheme in request_proxies:
+            hostonly = self.host
+            no_proxy = [n for n in request_proxies.get('no', '').split(',')
+                        if n]
+            if not any(map(hostonly.endswith, no_proxy)):
+                url = request_proxies[self.scheme]
+                p = urlparse(url)
+                if not p.scheme:
+                    raise ValueError('Could not understand proxy %s' % url)
+                scheme = p.scheme
+                host = p.netloc
+                if not self._ssl:
+                    self.scheme = scheme
+                    self._set_hostport(scheme, host)
+                    self._proxy = scheme_host(scheme, host)
+                else:
+                    self._tunnel = HttpTunnel(self, scheme, host)
+
+    def _set_hostport(self, scheme, host):
+        self._tunnel = None
+        self._proxy = None
+        self.host, self.port = get_hostport(scheme, host)
+
 
 class HttpResponse(ProtocolConsumer):
     '''A :class:`.ProtocolConsumer` for the HTTP client protocol.
@@ -732,8 +844,9 @@ class HttpResponse(ProtocolConsumer):
     _data_sent = None
     _status_code = None
     _cookies = None
+    _raw = None
     request_again = None
-    ONE_TIME_EVENTS = ProtocolConsumer.ONE_TIME_EVENTS + ('on_headers',)
+    ONE_TIME_EVENTS = ('pre_request', 'on_headers', 'post_request')
 
     @property
     def parser(self):
@@ -741,11 +854,9 @@ class HttpResponse(ProtocolConsumer):
         if request:
             return request.parser
 
-    def __str__(self):
-        return '%s' % (self.status_code or '<None>')
-
     def __repr__(self):
-        return '%s(%s)' % (self.__class__.__name__, self)
+        return '<Response [%s]>' % (self.status_code or 'None')
+    __str__ = __repr__
 
     @property
     def status_code(self):
@@ -781,12 +892,28 @@ class HttpResponse(ProtocolConsumer):
         else:
             return False
 
+    @property
+    def ok(self):
+        return not self.is_error
+
     @property
     def cookies(self):
         '''Dictionary of cookies set by the server or ``None``.
         '''
         return self._cookies
 
+    @property
+    def content(self):
+        '''Content of the response, in bytes'''
+        return response_content(self)
+
+    @property
+    def raw(self):
+        '''A raw asynchronous Http response'''
+        if self._raw is None:
+            self._raw = HttpStream(self)
+        return self._raw
+
     def recv_body(self):
         '''Flush the response body and return it.'''
         return self.parser.recv_body()
@@ -796,22 +923,16 @@ class HttpResponse(ProtocolConsumer):
         if code:
             return '%d %s' % (code, responses.get(code, 'Unknown'))
 
-    def get_content(self):
-        '''Retrieve the body without flushing'''
-        b = self.parser.recv_body()
-        if b or self._content is None:
-            self._content = self._content + b if self._content else b
-        return self._content
-
-    def content_string(self, charset=None, errors=None):
+    def text(self, charset=None, errors=None):
         '''Decode content as a string.'''
-        data = self.get_content()
+        data = self.content
         if data is not None:
             return data.decode(charset or 'utf-8', errors or 'strict')
+    content_string = text
 
     def json(self, charset=None):
         '''Decode content as a JSON object.'''
-        return json.loads(self.content_string(charset))
+        return json.loads(self.text(charset))
 
     def decode_content(self):
         '''Return the best possible representation of the response body.
@@ -823,8 +944,8 @@ class HttpResponse(ProtocolConsumer):
             if ct in JSON_CONTENT_TYPES:
                 return self.json(charset)
             elif ct.startswith('text/'):
-                return self.content_string(charset)
-        return self.get_content()
+                return self.text(charset)
+        return self.content
 
     def raise_for_status(self):
         '''Raises stored :class:`HTTPError` or :class:`URLError`, if occured.
@@ -832,7 +953,7 @@ class HttpResponse(ProtocolConsumer):
         if self.is_error:
             if self.status_code:
                 raise HTTPError(self.url, self.status_code,
-                                self.content_string(), self.headers, None)
+                                self.text(), self.headers, None)
             else:
                 raise URLError(self.on_finished.result.error)
 
@@ -845,22 +966,31 @@ class HttpResponse(ProtocolConsumer):
     # #####################################################################
     # #    PROTOCOL IMPLEMENTATION
     def start_request(self):
-        self.transport.write(self._request.encode())
+        request = self._request
+        self.transport.write(request.encode())
+        if request.headers.get('expect') != '100-continue':
+            self.write_body()
 
     def data_received(self, data):
-        request = self._request
+        request = self.request
         # request.parser my change (100-continue)
         # Always invoke it via request
-        if request.parser.execute(data, len(data)) == len(data):
-            if request.parser.is_headers_complete():
-                self._status_code = request.parser.get_status_code()
-                if not self.event('on_headers').fired():
-                    self.fire_event('on_headers')
-                if (not self.event('post_request').fired() and
-                        request.parser.is_message_complete()):
-                    self.finished()
-        else:
-            raise pulsar.ProtocolError('%s\n%s' % (self, self.headers))
+        try:
+            if request.parser.execute(data, len(data)) == len(data):
+                if request.parser.is_headers_complete():
+                    self._status_code = request.parser.get_status_code()
+                    if not self.event('on_headers').fired():
+                        self.fire_event('on_headers')
+                    if (not self.event('post_request').fired() and
+                            request.parser.is_message_complete()):
+                        self.finished()
+            else:
+                raise pulsar.ProtocolError('%s\n%s' % (self, self.headers))
+        except Exception as exc:
+            self.finished(exc=exc)
+
+    def write_body(self):
+        self.request.write_body(self.transport)
 
 
 class HttpClient(AbstractClient):
@@ -892,8 +1022,7 @@ class HttpClient(AbstractClient):
 
     .. attribute:: timeout
 
-        Default timeout for the connecting sockets. If 0 it is an asynchronous
-        client.
+        Default timeout for requests. If None or 0, no timeout on requests
 
     .. attribute:: encode_multipart
 
@@ -905,7 +1034,7 @@ class HttpClient(AbstractClient):
 
         Default: ``True``
 
-    .. attribute:: proxy_info
+    .. attribute:: proxies
 
         Dictionary of proxy servers for this client.
 
@@ -917,13 +1046,6 @@ class HttpClient(AbstractClient):
 
         Dictionary of connection pools for different hosts
 
-    .. attribute:: green
-
-        If ``True`` the requests are processed using the
-        :mod:`~pulsar.apps.greenio` application. When operating in this mode,
-        the :class:`.HttpClient` must be used from a greenlet ather than the
-        main one.
-
     .. attribute:: DEFAULT_HTTP_HEADERS
 
         Default headers for this :class:`HttpClient`
@@ -932,7 +1054,6 @@ class HttpClient(AbstractClient):
     MANY_TIMES_EVENTS = ('connection_made', 'pre_request', 'on_headers',
                          'post_request', 'connection_lost')
     protocol_factory = partial(Connection, HttpResponse)
-    allow_redirects = False
     max_redirects = 10
     '''Maximum number of redirects.
 
@@ -957,21 +1078,22 @@ class HttpClient(AbstractClient):
         ('Proxy-Connection', 'Keep-Alive')],
         kind='client')
     request_parameters = ('encode_multipart', 'max_redirects', 'decompress',
-                          'allow_redirects', 'multipart_boundary', 'version',
-                          'timeout', 'websocket_handler')
+                          'websocket_handler', 'multipart_boundary', 'version',
+                          'verify', 'stream')
     # Default hosts not affected by proxy settings. This can be overwritten
-    # by specifying the "no" key in the proxy_info dictionary
+    # by specifying the "no" key in the proxies dictionary
     no_proxy = set(('localhost', platform.node()))
 
-    def __init__(self, proxy_info=None, cache=None, headers=None,
+    def __init__(self, proxies=None, cache=None, headers=None,
                  encode_multipart=True, multipart_boundary=None,
-                 keyfile=None, certfile=None, cert_reqs=CERT_NONE,
+                 keyfile=None, certfile=None, verify=True,
                  ca_certs=None, cookies=None, store_cookies=True,
                  max_redirects=10, decompress=True, version=None,
                  websocket_handler=None, parser=None, trust_env=True,
-                 loop=None, client_version=None, timeout=None,
-                 pool_size=10, green=False):
-        super(HttpClient, self).__init__(loop)
+                 loop=None, client_version=None, timeout=None, stream=False,
+                 pool_size=10, frame_parser=None, logger=None):
+        super().__init__(loop)
+        self._logger = logger or LOGGER
         self.client_version = client_version or self.client_version
         self.connection_pools = {}
         self.pool_size = pool_size
@@ -982,42 +1104,31 @@ class HttpClient(AbstractClient):
         self.cookies = cookiejar_from_dict(cookies)
         self.decompress = decompress
         self.version = version or self.version
-        self.green = green and greenio
+        self.verify = verify
+        self.stream = stream
         dheaders = self.DEFAULT_HTTP_HEADERS.copy()
         dheaders['user-agent'] = self.client_version
         if headers:
             dheaders.override(headers)
         self.headers = dheaders
         self.tunnel_headers = self.DEFAULT_TUNNEL_HEADERS.copy()
-        self.proxy_info = dict(proxy_info or ())
-        if not self.proxy_info and self.trust_env:
-            self.proxy_info = get_environ_proxies()
-            if 'no' not in self.proxy_info:
-                self.proxy_info['no'] = ','.join(self.no_proxy)
+        self.proxies = dict(proxies or ())
+        if not self.proxies and self.trust_env:
+            self.proxies = get_environ_proxies()
+            if 'no' not in self.proxies:
+                self.proxies['no'] = ','.join(self.no_proxy)
         self.encode_multipart = encode_multipart
         self.multipart_boundary = multipart_boundary or choose_boundary()
         self.websocket_handler = websocket_handler
-        self.https_defaults = {'keyfile': keyfile,
-                               'certfile': certfile,
-                               'cert_reqs': cert_reqs,
-                               'ca_certs': ca_certs}
         self.http_parser = parser or http_parser
+        self.frame_parser = frame_parser or websocket.frame_parser
         # Add hooks
+        self.bind_event('finish', self._close)
         self.bind_event('pre_request', Tunneling(self._loop))
-        self.bind_event('on_headers', handle_101)
+        self.bind_event('pre_request', WebSocket())
         self.bind_event('on_headers', handle_100)
         self.bind_event('on_headers', handle_cookies)
-        self.bind_event('post_request', handle_redirect)
-        # greenlet
-        if self.green:
-            self.request = partial(green_request, self.request)
-
-    @property
-    def websocket_key(self):
-        if not hasattr(self, '_websocket_key'):
-            self._websocket_key = native_str(b64encode(os.urandom(16)),
-                                             DEFAULT_CHARSET)
-        return self._websocket_key
+        self.bind_event('post_request', Redirect())
 
     def connect(self, address):
         if isinstance(address, tuple):
@@ -1082,8 +1193,7 @@ class HttpClient(AbstractClient):
         '''
         return self.request('DELETE', url, **kwargs)
 
-    @task
-    def request(self, method, url, **params):
+    def request(self, method, url, timeout=None, **params):
         '''Constructs and sends a request to a remote server.
 
         It returns a :class:`.Future` which results in a
@@ -1099,6 +1209,17 @@ class HttpClient(AbstractClient):
 
         :rtype: a :class:`.Future`
         '''
+        response = self._request(method, url, **params)
+        if timeout is None:
+            timeout = self.timeout
+        if timeout:
+            response = asyncio.wait_for(response, timeout, loop=self._loop)
+        if not self._loop.is_running():
+            return self._loop.run_until_complete(response)
+        else:
+            return response
+
+    def _request(self, method, url, **params):
         nparams = params.copy()
         nparams.update(((name, getattr(self, name)) for name in
                         self.request_parameters if name not in params))
@@ -1110,39 +1231,28 @@ class HttpClient(AbstractClient):
                 partial(self._connect, host, port, request.ssl),
                 pool_size=self.pool_size, loop=self._loop)
             self.connection_pools[request.key] = pool
-        conn = yield pool.connect()
+        conn = yield from pool.connect()
         with conn:
-            consumer = conn.current_consumer()
-            # bind request-specific events
-            consumer.bind_events(**request.inp_params)
-            consumer.start(request)
-            response = yield consumer.on_finished
-            if response is not None:
-                consumer = response
-            if consumer.request_again:
-                if isinstance(consumer.request_again, Exception):
-                    raise consumer.request_again
-                elif isinstance(consumer.request_again, ProtocolConsumer):
-                    consumer = consumer.request_again
-            headers = consumer.headers
+            try:
+                response = yield from start_request(request, conn)
+                headers = response.headers
+            except AbortRequest:
+                response = None
+                headers = None
+
             if (not headers or
                     not headers.has('connection', 'keep-alive') or
-                    consumer.status_code == 101):
+                    response.status_code == 101 or
+                    # Streaming responses return before the response
+                    # is finished - therefore we detach the connection
+                    response.request.stream):
                 conn.detach()
-        if isinstance(consumer.request_again, tuple):
-            method, url, params = consumer.request_again
-            consumer = yield self.request(method, url, **params)
-        coroutine_return(consumer)
 
-    def close(self, async=True, timeout=5):
-        '''Close all connections.
-
-        Fire the ``finish`` :ref:`one time event <one-time-event>` once done.
-        Return the :class:`.Future` fired by the ``finish`` event.
-        '''
-        for p in self.connection_pools.values():
-            p.close()
-        self.connection_pools.clear()
+        # Handle a possible redirect
+        if response and isinstance(response.request_again, tuple):
+            method, url, params = response.request_again
+            response = yield from self._request(method, url, **params)
+        return response
 
     def add_basic_authentication(self, username, password):
         '''Add a :class:`HTTPBasicAuth` handler to the ``pre_requests`` hook.
@@ -1158,46 +1268,35 @@ class HttpClient(AbstractClient):
     def get_headers(self, request, headers=None):
         # Returns a :class:`Header` obtained from combining
         # :attr:`headers` with *headers*. Can handle websocket requests.
-        if request.scheme in ('ws', 'wss'):
-            d = Headers((
-                ('Connection', 'Upgrade'),
-                ('Upgrade', 'websocket'),
-                ('Sec-WebSocket-Version', str(max(SUPPORTED_VERSIONS))),
-                ('Sec-WebSocket-Key', self.websocket_key),
-                ('user-agent', self.client_version)
-                ), kind='client')
-        else:
-            d = self.headers.copy()
+        d = self.headers.copy()
         if headers:
             d.override(headers)
         return d
 
-    def ssl_context(self, **kwargs):
-        params = self.https_defaults.copy()
-        for name in kwargs:
-            if name in params:
-                params[name] = kwargs[name]
-        return SSLContext(**params)
-
-    def set_proxy(self, request):
-        if request.scheme in self.proxy_info:
-            hostonly = request.host
-            no_proxy = [n for n in self.proxy_info.get('no', '').split(',')
-                        if n]
-            if not any(map(hostonly.endswith, no_proxy)):
-                url = self.proxy_info[request.scheme]
-                p = urlparse(url)
-                if not p.scheme:
-                    raise ValueError('Could not understand proxy %s' % url)
-                request.set_proxy(p.scheme, p.netloc)
+    def ssl_context(self, verify=True, cert_reqs=None,
+                    check_hostname=False, certfile=None, keyfile=None,
+                    cafile=None, capath=None, cadata=None, **kw):
+        assert ssl, 'SSL not supported'
+        if verify:
+            cert_reqs = ssl.CERT_REQUIRED
+            check_hostname = True
+        return ssl._create_unverified_context(cert_reqs=cert_reqs,
+                                              check_hostname=check_hostname,
+                                              certfile=certfile,
+                                              keyfile=keyfile,
+                                              cafile=cafile, capath=capath,
+                                              cadata=cadata)
 
     def _connect(self, host, port, ssl):
-        _, connection = yield From(self._loop.create_connection(
-            self.create_protocol, host, port, ssl=ssl))
+        _, connection = yield from self._loop.create_connection(
+            self.create_protocol, host, port, ssl=ssl)
         # Wait for the connection made event
-        yield From(connection.event('connection_made'))
-        coroutine_return(connection)
+        yield from connection.event('connection_made')
+        return connection
 
-
-def green_request(request, method, url, **params):
-    return greenio.wait(request(method, url, **params))
+    def _close(self, _, exc=None, async=True):
+        '''Close all connections.
+        '''
+        for p in self.connection_pools.values():
+            p.close(async=async)
+        self.connection_pools.clear()
