@@ -2,11 +2,12 @@ import os
 import time
 from hashlib import sha1
 from base64 import b64encode
+from urllib.parse import urlparse
 
 from pulsar.utils.httpurl import (parse_dict_header, hexmd5, hexsha1,
-                                  urlparse, native_str, DEFAULT_CHARSET)
+                                  to_string, DEFAULT_CHARSET)
 
-from .plugins import request_again
+from .plugins import request_again, noerror
 
 
 __all__ = ['Auth',
@@ -45,7 +46,7 @@ class HTTPBasicAuth(Auth):
     def header(self):
         b64 = b64encode(('%s:%s' % (
             self.username, self.password)).encode(DEFAULT_CHARSET))
-        return 'Basic %s' % native_str(b64.strip(), DEFAULT_CHARSET)
+        return 'Basic %s' % to_string(b64.strip(), DEFAULT_CHARSET)
 
     def __repr__(self):
         return 'Basic: %s' % self.username
@@ -92,7 +93,6 @@ class HTTPDigestAuth(Auth):
         path = p_parsed.path
         if p_parsed.query:
             path += '?' + p_parsed.query
-        KD = lambda s, d: self.hex("%s:%s" % (s, d))
         ha1 = self.ha1(realm, self.password)
         ha2 = self.ha2(qop, method, path)
         if qop == 'auth':
@@ -107,9 +107,9 @@ class HTTPDigestAuth(Auth):
             s += os.urandom(8)
             cnonce = sha1(s).hexdigest()[:16]
             noncebit = "%s:%s:%s:%s:%s" % (nonce, ncvalue, cnonce, qop, ha2)
-            respdig = KD(ha1, noncebit)
+            respdig = self.KD(ha1, noncebit)
         elif qop is None:
-            respdig = KD(ha1, "%s:%s" % (nonce, ha2))
+            respdig = self.KD(ha1, "%s:%s" % (nonce, ha2))
         else:
             # XXX handle auth-int.
             return
@@ -125,9 +125,10 @@ class HTTPDigestAuth(Auth):
             base += ', qop=%s, nc=%s, cnonce="%s"' % (qop, ncvalue, cnonce)
         return 'Digest %s' % (base)
 
+    @noerror
     def handle_401(self, response, exc=None):
         """Takes the given response and tries digest-auth, if needed."""
-        if not exc and response.status_code == 401:
+        if response.status_code == 401:
             request = response.request
             response._handle_401 = getattr(response, '_handle_401', 0) + 1
             s_auth = response.headers.get('www-authenticate', '')
@@ -142,6 +143,9 @@ class HTTPDigestAuth(Auth):
                                                        request.full_url,
                                                        params)
 
+    def KD(self, s, d):
+        return self.hex("%s:%s" % (s, d))
+
     def hex(self, x):
         if self.algorithm == 'MD5':
             return hexmd5(x)
