@@ -60,12 +60,6 @@ from pulsar.apps.wsgi import (LazyWsgi, WsgiHandler,
                               wait_for_body_middleware,
                               middleware_in_executor)
 from pulsar.utils.importer import module_attribute
-try:
-    from pulsar.apps import greenio
-    from pulsar.apps.greenio import pg, local
-except ImportError:
-    greenio = None
-    pg = None
 
 
 class Wsgi(LazyWsgi):
@@ -88,16 +82,5 @@ class Wsgi(LazyWsgi):
             app = module_attribute(dotted)
         else:
             app = get_wsgi_application()
-        green_workers = self.cfg.greenlet if self.cfg else 0
-        if greenio and green_workers:
-            if pg:
-                pg.make_asynchronous()
-            app = greenio.RunInPool(app, green_workers)
-            self.green_safe_connections()
-        else:
-            app = middleware_in_executor(app)
-        return WsgiHandler((wait_for_body_middleware, app))
-
-    def green_safe_connections(self):
-        from django.db import connections
-        connections._connections = local()
+        app = middleware_in_executor(app)
+        return WsgiHandler((wait_for_body_middleware, app), async=True)
