@@ -159,7 +159,7 @@ Options
 
 All standard :ref:`settings <settings>` can be applied to the test application.
 In addition, the following options are
-:ref:`test-suite specific <setting-section-test>`:
+:ref:`test suite specific <setting-section-test>`:
 
 .. _apps-test-sequential:
 
@@ -191,11 +191,10 @@ is executed by the :class:`.TestSuite` at a given time. Indeed, several
 :class:`~unittest.TestCase` are executed at the same time and therefore
 each one of the may have one test function running.
 
-In order to run only one function at any time, the ``sequential``
-option should be used in conjunction with
-:ref:`--concurrent-tasks <setting-concurrent_tasks>` option::
+In order to run only one test function at any time, the ``sequential``
+option should be used::
 
-    python runtests.py --sequential --concurrent-tasks 1
+    python runtests.py --sequential
 
 list labels
 ~~~~~~~~~~~~~~~~~~~
@@ -315,26 +314,21 @@ Utilities
 
 '''
 import sys
-import unittest
-from functools import partial
 
 import pulsar
-from pulsar import multi_async, task
-from pulsar.apps import tasks
-from pulsar.apps.data import create_store
-from pulsar.apps.ds import PulsarDS
 from pulsar.utils.log import lazyproperty
 from pulsar.utils.config import section_docs, TestOption
-from pulsar.utils.pep import default_timer, to_string
 
-from .case import mock
-from .populate import populate
-from .result import *
-from .plugins.base import *
-from .loader import *
-from .utils import *
-from .wsgi import *
-from .pep import pep8_run
+from .populate import populate, random_string
+from .result import *           # noqa
+from .plugins.base import *     # noqa
+from .loader import *           # noqa
+from .utils import *            # noqa
+from .wsgi import *             # noqa
+from .runner import Runner
+
+
+__all__ = ['populate', 'random_string']
 
 
 pyver = '%s.%s' % (sys.version_info[:2])
@@ -361,7 +355,7 @@ class TestTimeout(TestOption):
     flags = ['--test-timeout']
     validator = pulsar.validate_pos_int
     type = int
-    default = 30
+    default = 5
     desc = '''\
         Tests which take longer than this many seconds are timed-out
         and failed.'''
@@ -427,15 +421,7 @@ class TestShowLeaks(TestOption):
     """
 
 
-class TestPep8(TestOption):
-    name = "pep8"
-    flags = ['--pep8']
-    nargs = '*'
-    validator = pulsar.validate_list
-    desc = """Run pep8"""
-
-
-class TestSuite(tasks.TaskQueue):
+class TestSuite(pulsar.Application):
     '''An asynchronous test suite which works like a task queue.
 
     Each task is a group of test methods in a python TestCase class.
@@ -462,17 +448,13 @@ class TestSuite(tasks.TaskQueue):
     :parameter plugins: Optional list of :class:`.TestPlugin` instances.
     '''
     name = 'test'
-    cfg = pulsar.Config(apps=('tasks', 'test'),
+    cfg = pulsar.Config(apps=['test'],
                         loglevel=['none'],
-                        task_paths=['pulsar.apps.test.case'],
                         plugins=())
 
     def new_runner(self):
         '''The :class:`.TestRunner` driving test cases.
         '''
-        if mock is None:    # pragma    nocover
-            raise ExitTest('python %s requires mock library for pulsar '
-                           'test suite application' % pyver)
         result_class = getattr(self, 'result_class', None)
         stream = pulsar.get_stream(self.cfg)
         runner = TestRunner(self.cfg.plugins, stream, result_class)
@@ -524,29 +506,17 @@ class TestSuite(tasks.TaskQueue):
                 stream.writeln(tag)
             stream.writeln('')
             return False
-        elif self.cfg.pep8:
-            msg, code = pep8_run(self.cfg.pep8)
-            stream.writeln(msg)
-            if code:
-                sys.exit(code)
-            return False
 
-    @task
     def monitor_start(self, monitor):
         '''When the monitor starts load all test classes into the queue'''
-        # Create a datastore for this test suite
-        if not self.cfg.task_backend:
-            server = PulsarDS(bind='127.0.0.1:0', workers=0,
-                              key_value_save=[],
-                              name='%s_store' % self.name)
-            yield server()
-            address = 'pulsar://%s:%s' % server.cfg.addresses[0]
-        else:
-            address = self.cfg.task_backend
-
-        store = create_store(address, pool_size=2, loop=monitor._loop)
-        self.get_backend(store)
+        cfg = self.cfg
+        workers = min(0, cfg.workers)
+        cfg.set('workers', workers)
         loader = self.loader
+
+        if self.cfg.callable:
+            self.cfg.callable()
+
         tags = self.cfg.labels
         exclude_tags = self.cfg.exclude_labels
         if self.cfg.show_leaks:
@@ -564,23 +534,7 @@ class TestSuite(tasks.TaskQueue):
             self._time_start = None
             if tests:
                 self.logger.info('loading %s test classes', len(tests))
-                monitor.cfg.set('workers', min(self.cfg.workers, len(tests)))
-                self._time_start = default_timer()
-                queued = []
-                self._tests_done = set()
-                self._tests_queued = None
-                #
-                # Bind to the task_done event
-                self.backend.bind_event('task_done',
-                                        partial(self._test_done, monitor))
-                for tag, testcls in tests:
-                    r = self.backend.queue_task('test', testcls=testcls,
-                                                tag=tag)
-                    queued.append(r)
-                queued = yield multi_async(queued)
-                self.logger.debug('loaded %s test classes', len(tests))
-                self._tests_queued = set(queued)
-                yield self._test_done(monitor)
+                monitor._loop.call_soon(Runner, monitor, loader.runner, tests)
             else:   # pragma    nocover
                 raise ExitTest('Could not find any tests.')
         except ExitTest as e:   # pragma    nocover
@@ -601,31 +555,6 @@ class TestSuite(tasks.TaskQueue):
         return cfg
 
     def arbiter_params(self):
-        params = super(TestSuite, self).arbiter_params()
+        params = super().arbiter_params()
         params['concurrency'] = self.cfg.concurrency
         return params
-
-    @task
-    def _test_done(self, monitor, task_id=None, exc=None):
-        runner = self.runner
-        if task_id:
-            self._tests_done.add(to_string(task_id))
-        if self._tests_queued is not None:
-            left = self._tests_queued.difference(self._tests_done)
-            if not left:
-                tests = yield self.backend.get_tasks(self._tests_done)
-                self.logger.info('All tests have finished.')
-                time_taken = default_timer() - self._time_start
-                for task in tests:
-                    runner.add(task.get('result'))
-                runner.on_end()
-                runner.printSummary(time_taken)
-                # Shut down the arbiter
-                if runner.result.errors or runner.result.failures:
-                    exit_code = 2
-                else:
-                    exit_code = 0
-                monitor._loop.call_soon(self._exit, exit_code)
-
-    def _exit(self, exit_code):
-        raise pulsar.HaltServer(exit_code=exit_code)
