@@ -1,5 +1,4 @@
 import pulsar
-from pulsar.utils.httpurl import iteritems
 from pulsar.apps.test.result import Plugin
 
 
@@ -48,7 +47,7 @@ class TestPluginMeta(type):
         for base in bases:
             if isinstance(base, TestPluginMeta):
                 settings.update(base.config.settings)
-        for key, setting in list(iteritems(attrs)):
+        for key, setting in list(attrs.items()):
             if isinstance(setting, pulsar.Setting):
                 attrs.pop(key)
                 setting.name = setting.name or key.lower()
@@ -63,8 +62,8 @@ class TestPluginMeta(type):
                 default = attrs.pop('default', None)
                 validator = attrs.pop('validator', None)
                 nargs = attrs.pop('nargs', None)
-                if (validator is None and default is None and type is None
-                        and nargs is None):
+                if (validator is None and default is None and type is None and
+                        nargs is None):
                     if action is None or action == 'store_true':
                         action = 'store_true'
                         default = False
@@ -85,8 +84,7 @@ class TestPluginMeta(type):
         return super(TestPluginMeta, cls).__new__(cls, name, bases, attrs)
 
 
-class TestPlugin(TestPluginMeta('TestPluginBase',
-                                (Plugin,), {'virtual': True})):
+class TestPlugin(Plugin, metaclass=TestPluginMeta):
     '''Base class for :class:`.Plugin` which can be added to a
     :class:`.TestSuite` to extend its functionalities.
 
