@@ -1,16 +1,11 @@
 from datetime import date, timedelta
 from random import uniform, randint, choice
-import string
 
-from pulsar.utils.pep import ispy3k
+from pulsar.utils.string import random_string
 
-if ispy3k:
-    characters = string.ascii_letters + string.digits
-else:   # pragma nocover
-    characters = string.letters + string.digits
-    range = xrange
 
-def_converter = lambda x: x
+def def_converter(x):
+    return x
 
 
 def populate(datatype='string', size=10, start=None, end=None,
@@ -88,10 +83,5 @@ def populate(datatype='string', size=10, start=None, end=None,
     return data
 
 
-def random_string(min_len=3, max_len=20, **kwargs):
-    len = randint(min_len, max_len) if max_len > min_len else min_len
-    return ''.join((choice(characters) for s in range(len)))
-
-
 def random_date(date_start, delta):
     return date_start + timedelta(days=randint(0, delta))
