@@ -17,6 +17,10 @@ STDOUT_LINE = '\nStdout:\n%s'
 STDERR_LINE = '\nStderr:\n%s'
 
 
+def def_return_val(c):
+    return None
+
+
 class Plugin(object):
     '''Interface for all classes which are part of the :class:`.TestRunner`.
 
@@ -311,14 +315,14 @@ class TestResult(Plugin):
         '''
         self._testsRun += 1
 
-    def addError(self, test, err):
+    def addError(self, test, err):    # pragma    nocover
         '''Called when an unexpected error has occurred.
 
         ``err`` is a tuple of values as returned by ``sys.exc_info()``
         '''
         self._add_error(test, err, self.errors)
 
-    def addFailure(self, test, err):
+    def addFailure(self, test, err):    # pragma    nocover
         '''Called when an test failure has occurred.
 
         ``err`` is a tuple of values as returned by ``sys.exc_info()``
@@ -343,15 +347,6 @@ class TestResult(Plugin):
         test = self.getDescription(test)
         container.append((test, str(exc)))
 
-    def add(self, result):
-        self._count += 1
-        self._testsRun += result.testsRun
-        self.failures.extend(result.failures)
-        self.errors.extend(result.errors)
-        self.skipped.extend(result.skipped)
-        self.expectedFailures.extend(result.expectedFailures)
-        self.unexpectedSuccesses.extend(result.unexpectedSuccesses)
-
     def wasSuccessful(self):
         "Tells whether or not this result was a success"
         return len(self.failures) == len(self.errors) == 0
@@ -359,7 +354,7 @@ class TestResult(Plugin):
 
 def testsafe(name, return_val=None):
     if not return_val:
-        return_val = lambda c: None
+        return_val = def_return_val
 
     def _(self, *args):
         for p in self.plugins:
@@ -367,7 +362,7 @@ def testsafe(name, return_val=None):
                 c = getattr(p, name)(*args)
                 if c is not None:
                     return return_val(c)
-            except Exception:
+            except Exception:       # pragma    nocover
                 LOGGER.exception('Unhadled error in %s.%s' % (p, name))
     return _
 
@@ -418,11 +413,6 @@ class TestRunner(Plugin):
         else:
             return c
 
-    def add(self, result):
-        '''Add a new result to the :attr:`~.Plugin.result` container
-        '''
-        self.result.add(result)
-
     def import_module(self, mod, parent=None):
         for p in self.plugins:
             mod = p.import_module(mod, parent)
@@ -431,8 +421,7 @@ class TestRunner(Plugin):
         return mod
 
     def before_test_function_run(self, test):
-        '''Called just before the test is run,
-        in the test process domain.'''
+        '''Called just before the test is run'''
         test.plugins = plugins = {}
         for p in self.plugins:
             local = AttributeDictionary()
