@@ -60,17 +60,18 @@ from functools import partial
 import pulsar
 from pulsar.apps.socket import SocketServer, Connection
 
-from .html import *
-from .content import *
-from .utils import *
-from .middleware import *
-from .response import *
-from .wrappers import *
-from .server import *
-from .route import *
-from .handlers import *
-from .routers import *
-from .auth import *
+from .html import *         # noqa
+from .content import *      # noqa
+from .utils import *        # noqa
+from .middleware import *   # noqa
+from .response import *     # noqa
+from .wrappers import *     # noqa
+from .server import *       # noqa
+from .route import *        # noqa
+from .handlers import *     # noqa
+from .routers import *      # noqa
+from .auth import *         # noqa
+from .formdata import *     # noqa
 
 
 class WSGIServer(SocketServer):
