@@ -22,7 +22,7 @@ from base64 import b64decode
 from pulsar import HttpException
 from pulsar.utils.httpurl import (parse_dict_header, hexmd5, hexsha1,
                                   quote_header_value, DEFAULT_CHARSET)
-from pulsar.utils.pep import iteritems, to_bytes
+from pulsar.utils.pep import to_bytes
 
 
 __all__ = ['HttpAuthenticate', 'parse_authorization_header']
@@ -47,7 +47,7 @@ class HttpAuthenticate(HttpException):
             value = None
         if value:
             value = [('WWW-Authenticate', value)]
-        super(HttpAuthenticate, self).__init__(status=401, headers=value)
+        super().__init__(status=401, headers=value)
 
     def digest_auth_header(self, realm=None, nonce=None, qop=None, opaque=None,
                            algorithm=None, stale=None):
@@ -72,7 +72,7 @@ class HttpAuthenticate(HttpException):
         return '%s %s' % (type.title(), ', '.join((
             '%s=%s' % (key, quote_header_value(
                 value, allow_token=key not in _require_quoting))
-            for key, value in iteritems(options)
+            for key, value in options.items()
         )))
 
 
