@@ -1,5 +1,5 @@
 '''The :mod:`pulsar.apps.wsgi.content` introduces several utility classes for
-handling asynchronous content within a :ref:`WSGI handler <wsgi-async> or
+handling asynchronous content within a :ref:`WSGI handler <wsgi-async>` or
 :ref:`middleware <wsgi-middleware>`.
 
 These classes can operate instead or in conjunction with a template engine,
@@ -10,22 +10,22 @@ response to an :ref:`HTTP client request <app-wsgi-request>`.
 A string can be ``html``, ``json``, ``plain text`` or any other valid HTTP
 content type.
 
-The main class of this module is the :class:`AsyncString`, which can be
+The main class of this module is the :class:`String`, which can be
 considered as the atomic component of an asynchronous web framework::
 
-    >>> from pulsar.apps.wsgi import AsyncString
-    >>> string = AsyncString('Hello')
+    >>> from pulsar.apps.wsgi import String
+    >>> string = String('Hello')
     >>> string.render()
     'Hello'
     >>> string.render()
     ...
-    RuntimeError: AsyncString already streamed
+    RuntimeError: String already streamed
 
-An :class:`AsyncString` can only be rendered once, and it accepts
+An :class:`String` can only be rendered once, and it accepts
 :ref:`asynchronous components  <tutorials-coroutine>`::
 
     >>> a = Future()
-    >>> string = AsyncString('Hello, ', a)
+    >>> string = String('Hello, ', a)
     >>> value = string.render()
     >>> value
     MultiFuture (pending)
@@ -44,20 +44,20 @@ Once the future is done, we have the concatenated string::
 Design
 ===============
 
-The :meth:`~AsyncString.do_stream` method is responsible for the streaming
+The :meth:`~String.do_stream` method is responsible for the streaming
 of ``strings`` or :ref:`asynchronous components  <tutorials-coroutine>`.
 It can be overwritten by subclasses to customise the way an
-:class:`AsyncString` streams its :attr:`~AsyncString.children`.
+:class:`String` streams its :attr:`~String.children`.
 
-On the other hand, the :meth:`~AsyncString.to_string` method is responsible
-for the concatenation of ``strings`` and, like :meth:`~AsyncString.do_stream`,
+On the other hand, the :meth:`~String.to_string` method is responsible
+for the concatenation of ``strings`` and, like :meth:`~String.do_stream`,
 it can be customised by subclasses.
 
 
 Asynchronous String
 =====================
 
-.. autoclass:: AsyncString
+.. autoclass:: String
    :members:
    :member-order: bysource
 
@@ -141,24 +141,21 @@ Html Factory
 
 .. _`HTML5 document`: http://www.w3schools.com/html/html5_intro.asp
 '''
-import os
 import re
-import json as pyjson
 
-from collections import Mapping, OrderedDict
+from collections import Mapping
 from functools import partial
-from inspect import isgenerator
 
 from pulsar import HttpException
-from pulsar import multi_async, async, coroutine_return, chain_future
-from pulsar.utils.pep import iteritems, to_string, ispy3k
+from pulsar import multi_async, chain_future, is_async
 from pulsar.utils.slugify import slugify
 from pulsar.utils.html import INLINE_TAGS, escape, dump_data_value, child_tag
+from pulsar.utils.pep import to_string
 from pulsar.utils.system import json
 
 from .html import html_visitor, newline
 
-__all__ = ['AsyncString', 'Html',
+__all__ = ['AsyncString', 'String', 'Html',
            'Json', 'HtmlDocument', 'Links', 'Scripts', 'Media',
            'html_factory']
 
@@ -166,38 +163,28 @@ __all__ = ['AsyncString', 'Html',
 DATARE = re.compile('data[-_]')
 
 
-if ispy3k:
-
-    def stream_to_string(stream):
-        for value in stream:
-            if value is None:
-                continue
-            elif isinstance(value, bytes):
-                yield value.decode('utf-8')
-            elif isinstance(value, str):
-                yield value
-            else:
-                yield str(value)
-
-else:  # pragma nocover
-
-    def stream_to_string(stream):
-        for value in stream:
-            if value is None:
-                continue
-            elif isinstance(value, unicode):
-                yield value
-            else:
-                yield str(value)
+def stream_to_string(stream):
+    for value in stream:
+        if value is None:
+            continue
+        elif isinstance(value, bytes):
+            yield value.decode('utf-8')
+        elif isinstance(value, str):
+            yield value
+        else:
+            yield str(value)
 
 
-def stream_mapping(value, request=None):
+def stream_mapping(value, request):
     result = {}
-    for key, value in iteritems(value):
-        if isinstance(value, AsyncString):
+    async = False
+    for key, value in value.items():
+        if isinstance(value, String):
             value = value.render(request)
+        if is_async(value):
+            async = True
         result[key] = value
-    return multi_async(result)
+    return multi_async(result) if async else result
 
 
 def attr_iter(attrs):
@@ -207,12 +194,12 @@ def attr_iter(attrs):
             yield " %s='%s'" % (k, escape(v, force=True))
 
 
-class AsyncString(object):
+class String(object):
     '''An asynchronous string which can be used with pulsar WSGI servers.
     '''
     _default_content_type = 'text/plain'
     _content_type = None
-    '''Content type for this :class:`AsyncString`'''
+    '''Content type for this :class:`String`'''
     _streamed = False
     _children = None
     _parent = None
@@ -234,15 +221,15 @@ class AsyncString(object):
 
     @property
     def parent(self):
-        '''The :class:`AsyncString` element which contains this
-        :class:`AsyncString`.'''
+        '''The :class:`String` element which contains this
+        :class:`String`.'''
         return self._parent
 
     @property
     def children(self):
-        '''A copy of all children of this :class:`AsyncString`.
+        '''A copy of all children of this :class:`String`.
 
-        Children can be other :class:`AsyncString` or string or bytes,
+        Children can be other :class:`String` or string or bytes,
         depending on implementation.
         :attr:`children` are added and removed via the :meth:`append` and
         :meth:`remove` methods.
@@ -266,8 +253,8 @@ class AsyncString(object):
     def append(self, child):
         '''Append ``child`` to the list of :attr:`children`.
 
-        :param child: String, bytes or another :class:`.AsyncString`.
-            If it is an :class:`.AsyncString`, this instance will be
+        :param child: String, bytes or another :class:`.String`.
+            If it is an :class:`.String`, this instance will be
             set as its :attr:`parent`.
             If ``child`` is ``None``, this method does nothing.
 
@@ -279,8 +266,8 @@ class AsyncString(object):
 
         This is a shortcut for the :meth:`insert` method at index 0.
 
-        :param child: String, bytes or another :class:`AsyncString`.
-            If it is an :class:`.AsyncString`, this instance will be set
+        :param child: String, bytes or another :class:`String`.
+            If it is an :class:`.String`, this instance will be set
             as its :attr:`parent`.
             If ``child`` is ``None``, this method does nothing.
         '''
@@ -290,14 +277,14 @@ class AsyncString(object):
         '''Insert ``child`` into the list of :attr:`children` at ``index``.
 
         :param index: The index (positive integer) where to insert ``child``.
-        :param child: String, bytes or another :class:`AsyncString`.
-            If it is an :class:`.AsyncString`, this instance will be set as
+        :param child: String, bytes or another :class:`String`.
+            If it is an :class:`.String`, this instance will be set as
             its :attr:`parent`.
             If ``child`` is ``None``, this method does nothing.
         '''
         # make sure that child is not in child
         if child not in (None, self):
-            if isinstance(child, AsyncString):
+            if isinstance(child, String):
                 child_parent = child._parent
                 if self._parent is child:
                     # the parent is the child we are appending.
@@ -319,7 +306,7 @@ class AsyncString(object):
         '''Remove a ``child`` from the list of :attr:`children`.'''
         try:
             self.children.remove(child)
-            if isinstance(child, AsyncString):
+            if isinstance(child, String):
                 child._parent = None
         except ValueError:
             pass
@@ -328,7 +315,7 @@ class AsyncString(object):
         '''Remove all :attr:`children`.'''
         if self._children:
             for child in self._children:
-                if isinstance(child, AsyncString):
+                if isinstance(child, String):
                     child._parent = None
             self._children = []
 
@@ -340,9 +327,9 @@ class AsyncString(object):
     def stream(self, request):
         '''An iterable over strings or asynchronous elements.
 
-        This is the most important method of an :class:`AsyncString`.
+        This is the most important method of an :class:`String`.
         It is called by :meth:`http_response` or by the :attr:`parent`
-        of this :class:`AsyncString`.
+        of this :class:`String`.
         It returns an iterable (list, tuple or a generator) over
         strings (``unicode/str`` for python 2, ``str`` only for python 3) or
         :ref:`asynchronous elements <tutorials-coroutine>` which result in
@@ -371,7 +358,7 @@ class AsyncString(object):
         '''
         if self._children:
             for child in self._children:
-                if isinstance(child, AsyncString):
+                if isinstance(child, String):
                     for bit in child.stream(request):
                         yield bit
                 else:
@@ -383,20 +370,15 @@ class AsyncString(object):
         This method asynchronously wait for :meth:`stream` and subsequently
         returns a :class:`.WsgiResponse`.
         '''
+        if not stream:
+            return self.render(request,
+                               partial(self.http_response, request))
+        stream = stream[0]
         content_types = request.content_types
         if not content_types or self._content_type in content_types:
             response = request.response
             response.content_type = self._content_type
             response.encoding = self.charset
-            if stream:
-                stream = stream[0]
-            else:
-                stream = multi_async(self.stream(request))
-                if stream.done():
-                    stream = stream.result()
-                else:
-                    return chain_future(
-                        stream, callback=partial(self.http_response, request))
             response.content = self.to_string(stream)
             return response
         else:
@@ -423,28 +405,39 @@ class AsyncString(object):
             self._before_stream = []
         self._before_stream.append(callback)
 
-    def render(self, request=None):
+    def render(self, request=None, callback=None):
         '''Render this string.
 
         This method returns a string or a :class:`~asyncio.Future` which
         results in a string. On the other hand, the callable method of
-        a :class:`.AsyncString` **always** returns a :class:`~asyncio.Future`.
+        a :class:`.String` **always** returns a :class:`~asyncio.Future`.
         '''
-        stream = multi_async(self.stream(request))
-        if stream.done():
-            return self.to_string(stream.result())
+        stream = []
+        async = False
+        for data in self.stream(request):
+            if is_async(data):
+                async = True
+            stream.append(data)
+
+        if not callback:
+            callback = self.to_string
+
+        if async:
+            return chain_future(multi_async(stream), callback=callback)
         else:
-            return chain_future(stream, callback=self.to_string)
+            return callback(stream)
 
     def __call__(self, request):
         stream = multi_async(self.stream(request))
         return chain_future(stream, callback=self.to_string)
 
+AsyncString = String
 
-class Json(AsyncString):
-    '''An :class:`AsyncString` which renders into a json string.
 
-    The :attr:`AsyncString.content_type` attribute is set to
+class Json(String):
+    '''An :class:`String` which renders into a json string.
+
+    The :attr:`String.content_type` attribute is set to
     ``application/json``.
 
     .. attribute:: as_list
@@ -460,12 +453,12 @@ class Json(AsyncString):
 
     def _setup(self, as_list=False, **params):
         self.as_list = as_list
-        super(Json, self)._setup(**params)
+        super()._setup(**params)
 
     def do_stream(self, request):
         if self._children:
             for child in self._children:
-                if isinstance(child, AsyncString):
+                if isinstance(child, String):
                     for bit in child.stream(request):
                         yield bit
                 elif isinstance(child, Mapping):
@@ -495,15 +488,15 @@ def html_factory(tag, **defaults):
     return html_input
 
 
-class Html(AsyncString):
-    '''An :class:`AsyncString` for ``html`` content.
+class Html(String):
+    '''An :class:`String` for ``html`` content.
 
-    The :attr:`~AsyncString.content_type` attribute is set to ``text/html``.
+    The :attr:`~String.content_type` attribute is set to ``text/html``.
 
     :param tag: Set the :attr:`tag` attribute. Must be given and can be
         ``None``.
     :param children: Optional children which will be added via the
-        :meth:`~AsyncString.append` method.
+        :meth:`~String.append` method.
     :param params: Optional keyed-value parameters
         including:
 
@@ -587,7 +580,7 @@ class Html(AsyncString):
                         child = Html(tag, child)
                 elif not child.startswith('<%s' % tag):
                     child = Html(tag, child)
-            super(Html, self).append(child)
+            super().append(child)
 
     def _setup(self, cn=None, attr=None, css=None, data=None,
                content_type=None, **params):
@@ -628,7 +621,7 @@ class Html(AsyncString):
             if data is None:
                 self._extra['data'] = {}
             add = self._visitor.add_data
-            for key, value in iteritems(result):
+            for key, value in result.items():
                 add(self, key, value)
             return self
         else:
@@ -741,11 +734,9 @@ class Html(AsyncString):
                     yield '<%s%s>%s' % (tag, self.flatatt(), n)
             if self._children:
                 for child in self._children:
-                    if isinstance(child, AsyncString):
+                    if isinstance(child, String):
                         for bit in child.stream(request):
                             yield bit
-                    elif isgenerator(child):
-                        yield async(child, getattr(request, '_loop', None))
                     else:
                         yield child
                 if tag:
@@ -769,7 +760,7 @@ class Html(AsyncString):
                 return cont.get(name) if cont else None, False
 
 
-class Media(AsyncString):
+class Media(String):
     '''A container for both :class:`.Links` and :class:`.Scripts`.
 
     .. attribute:: media_path
@@ -788,7 +779,7 @@ class Media(AsyncString):
     mediatype = None
 
     def __init__(self, media_path, minified=False, asset_protocol=None):
-        super(Media, self).__init__()
+        super().__init__()
         self.media_path = media_path
         self.asset_protocol = asset_protocol
         if self.media_path and not self.media_path.endswith('/'):
@@ -801,8 +792,8 @@ class Media(AsyncString):
         A path is local relative when it does not start with a slash
         ``/`` nor ``http://`` nor ``https://``.
         '''
-        return not (path.startswith('http://') or path.startswith('https://')
-                    or path.startswith('/'))
+        return not (path.startswith('http://') or
+                    path.startswith('https://') or path.startswith('/'))
 
     def absolute_path(self, path, minify=True):
         '''Return a suitable absolute url for ``path``.
@@ -891,7 +882,7 @@ class Scripts(Media):
         self.wait = kwargs.pop('wait', 200)
         self.require = []
         self.paths = {}
-        super(Scripts, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def script(self, src, type=None, **kwargs):
         type = type or 'application/javascript'
@@ -930,7 +921,7 @@ class Scripts(Media):
 class Embedded(Html):
 
     def __init__(self, tag, **kwargs):
-        super(Embedded, self).__init__(None, **kwargs)
+        super().__init__(None, **kwargs)
         self._child_tag = tag
         self._child_kwargs = kwargs
 
@@ -943,7 +934,7 @@ class Embedded(Html):
             if media:
                 kwargs['media'] = media
             child = Html(self._child_tag, child, **kwargs)
-        super(Embedded, self).insert(index, child)
+        super().insert(index, child)
 
 
 class Head(Html):
@@ -1000,7 +991,7 @@ class Head(Html):
     '''
     def __init__(self, media_path=None, title=None, meta=None, minified=False,
                  asset_protocol=None, **params):
-        super(Head, self).__init__('head', **params)
+        super().__init__('head', **params)
         self.title = title
         self.append(Html(None, meta))
         self.append(Links(media_path, minified=minified,
@@ -1054,7 +1045,7 @@ class Head(Html):
     def do_stream(self, request):
         if self.title:
             self._children.insert(0, '<title>%s</title>\n' % self.title)
-        return super(Head, self).do_stream(request)
+        return super().do_stream(request)
 
     def add_meta(self, **kwargs):
         '''Add a new :class:`Html` meta tag to the :attr:`meta` collection.'''
@@ -1130,35 +1121,35 @@ class HtmlDocument(Html):
 
     def __init__(self, title=None, media_path='/media/', charset=None,
                  minified=False, loop=None, asset_protocol=None, **params):
-        super(HtmlDocument, self).__init__(None, **params)
+        super().__init__(None, **params)
         self.head = Head(title=title, media_path=media_path, minified=minified,
                          charset=charset, asset_protocol=asset_protocol)
         self.body = Html('body')
 
     def do_stream(self, request):
         # stream the body
-        body = multi_async(self.body.stream(request))
+        body = self.body.render(request)
         # the body has asynchronous components
         # delay the header untl later
-        if not body.done():
+        if is_async(body):
             yield self._html(request, body)
+
+        head = self.head.render(request)
+        #
+        # header not ready (this should never occur really)
+        if is_async(head):
+            yield self._html(request, body, head)
         else:
-            head = multi_async(self.head.stream(request))
-            #
-            # header not ready (this should never occur really)
-            if not head.done():
-                yield self._html(request, body, head)
-            else:
-                yield self._template % (self.flatatt(),
-                                        self.head.to_string(head.result()),
-                                        self.body.to_string(body.result()))
+            yield self._template % (self.flatatt(), head, body)
 
     def _html(self, request, body, head=None):
+        '''Asynchronous rendering
+        '''
         if head is None:
-            body = yield body
-            head = multi_async(self.head.stream(request))
-        head = yield head
-        result = self._template % (self.flatatt(),
-                                   self.head.to_string(head),
-                                   self.body.to_string(body))
-        coroutine_return(result)
+            body = yield from body
+            head = self.head.render(request)
+
+        if is_async(head):
+            head = yield from head
+
+        return self._template % (self.flatatt(), head, body)
