@@ -98,8 +98,7 @@ via the ``_loop`` attribute::
 .. _WSGI: http://www.wsgi.org
 .. _`WSGI 1.0.1`: http://www.python.org/dev/peps/pep-3333/
 '''
-from pulsar import (From, Http404, coroutine_return, isfuture,
-                    get_event_loop, task, async)
+from pulsar import Http404, async, isfuture
 from pulsar.utils.log import LocalMixin, local_method
 
 from .utils import handle_wsgi_error
@@ -110,7 +109,7 @@ __all__ = ['WsgiHandler', 'LazyWsgi']
 
 
 class WsgiHandler(object):
-    '''An handler for application conforming to python WSGI_.
+    '''An handler for applications conforming to python WSGI_.
 
     .. attribute:: middleware
 
@@ -131,97 +130,58 @@ class WsgiHandler(object):
         :ref:`response middlewares <wsgi-response-middleware>`.
 
     '''
-    def __init__(self, middleware=None, response_middleware=None, **kwargs):
+    def __init__(self, middleware=None, response_middleware=None,
+                 async=False, **kwargs):
         if middleware:
             middleware = list(middleware)
         self.middleware = middleware or []
-        self._response_done = False
-        if response_middleware is False:
-            self._response_done = True
         self.response_middleware = response_middleware or []
+        self.async = async
 
     def __call__(self, environ, start_response):
         '''The WSGI callable'''
-        response = AsyncResponse(environ, start_response,
-                                 iter(self.middleware),
-                                 iter(self.response_middleware),
-                                 self._response_done)
-        return response()
-
-
-class AsyncResponse(object):
-    __slots__ = ('environ', 'start_response',
-                 'middleware', 'response_middleware',
-                 '_response_done')
-
-    def __init__(self, environ, start_response, middleware,
-                 response_middleware, response_done):
-        self.environ = environ
-        self.start_response = start_response
-        self.middleware = middleware
-        self.response_middleware = response_middleware
-        self._response_done = response_done
-
-    def __call__(self, resp=None, exc=None):
+        if self.async:
+            return async(self._async(environ, start_response))
+        response = None
         try:
-            while not exc and resp is None:
-                try:
-                    handler = next(self.middleware)
-                except StopIteration:
+            for middleware in self.middleware:
+                response = middleware(environ, start_response)
+                if response is not None:
                     break
-                else:
-                    resp = handler(self.environ, self.start_response)
-                    if resp is not None:
-                        try:
-                            resp = async(resp)
-                        except TypeError:
-                            pass
-                        else:
-                            return self._async(self, resp, True)
-            if not exc and resp is None:
-                if not self._response_done:
-                    raise Http404
+            if response is None:
+                raise Http404
+
         except Exception as exc:
-            if self._response_done:
-                raise
-            resp = handle_wsgi_error(self.environ, exc)
-        else:
-            if exc:
-                resp = handle_wsgi_error(self.environ, exc)
-        #
-        if not self._response_done:
-            self._response_done = True
-            return self._response(resp)
-        return resp
+            response = handle_wsgi_error(environ, exc)
 
-    def _response(self, resp=None, exc=None):
-        while not exc:
-            try:
-                handler = next(self.response_middleware)
-                resp = handler(self.environ, resp)
-                try:
-                    return self._async(self._response, async(resp))
-                except TypeError:
-                    pass
-            except StopIteration:
-                break
-        if isinstance(resp, WsgiResponse):
-            self.start_response(resp.status, resp.get_headers())
-        return resp
+        if isinstance(response, WsgiResponse) and not response.started:
+            for middleware in self.response_middleware:
+                response = middleware(environ, response) or response
+            response.start(start_response)
+        return response
 
-    @task
-    def _async(self, callable, future, safe=False):
-        while isfuture(future):
-            kw = {}
-            try:
-                resp = yield future
-            except Exception as exc:
-                if not safe:
-                    raise
-                future = callable(exc=exc)
-            else:
-                future = callable(resp)
-        coroutine_return(future)
+    def _async(self, environ, start_response):
+        response = None
+        try:
+            for middleware in self.middleware:
+                response = middleware(environ, start_response)
+                if isfuture(response):
+                    response = yield from response
+                if response is not None:
+                    break
+            if response is None:
+                raise Http404
+
+        except Exception as exc:
+            response = handle_wsgi_error(environ, exc)
+
+        if isinstance(response, WsgiResponse) and not response.started:
+            for middleware in self.response_middleware:
+                response = middleware(environ, response)
+                if isfuture(response):
+                    response = yield from response
+            response.start(start_response)
+        return response
 
 
 class LazyWsgi(LocalMixin):
