@@ -49,9 +49,10 @@ Middleware in Executor
 
 '''
 import re
+from functools import wraps
 
 import pulsar
-from pulsar import isfuture, chain_future, get_event_loop
+from pulsar import is_async, task, get_event_loop
 from pulsar.utils.httpurl import BytesIO
 
 from .auth import parse_authorization_header
@@ -90,6 +91,7 @@ def authorization_middleware(environ, start_response=None):
             environ[key] = parse_authorization_header(environ[code])
 
 
+@task
 def wait_for_body_middleware(environ, start_response=None):
     '''Use this middleware to wait for the full body.
 
@@ -98,16 +100,12 @@ def wait_for_body_middleware(environ, start_response=None):
 
     Useful when using synchronous web-frameworks such as :django:`django <>`.
     '''
-    def _wsgi_input(value):
-        environ['wsgi.input'] = BytesIO(value)
-
     if environ['wsgi.input']:
         stream = environ['wsgi.input']
         chunk = stream.read()
-        if isfuture(chunk):
-            return chain_future(chunk, callback=_wsgi_input)
-        else:
-            _wsgi_input(chunk)
+        if is_async(chunk):
+            chunk = yield from chunk
+        environ['wsgi.input'] = BytesIO(chunk)
 
 
 def middleware_in_executor(middleware):
@@ -116,6 +114,7 @@ def middleware_in_executor(middleware):
 
     Useful when using synchronous web-frameworks such as :django:`django <>`.
     '''
+    @wraps(middleware)
     def _(environ, start_response):
         loop = get_event_loop()
         return loop.run_in_executor(None, middleware, environ, start_response)
