@@ -53,7 +53,7 @@ import re
 from collections import namedtuple
 
 from pulsar import Http404
-from pulsar.utils.httpurl import (iteritems, iri_to_uri, remove_double_slash,
+from pulsar.utils.httpurl import (iri_to_uri, remove_double_slash,
                                   ENCODE_URL_METHODS, ENCODE_BODY_METHODS)
 from pulsar.utils.pep import to_string
 from pulsar.utils.slugify import slugify
@@ -111,7 +111,7 @@ def _pythonize(value):
             pass
     if value[:1] == value[-1:] and value[0] in '"\'':
         value = value[1:-1]
-    return unicode(value)
+    return str(value)
 
 
 def parse_rule(rule):
@@ -194,11 +194,12 @@ class route(object):
     creation_count = 0
 
     def __init__(self, rule=None, method=None, defaults=None,
-                 position=None, **parameters):
+                 position=None, re=False, **parameters):
         self.__class__.creation_count += 1
         self.position = position
         self.creation_count = self.__class__.creation_count
         self.rule = rule
+        self.re = re
         self.defaults = defaults
         self.method = method
         self.parameters = parameters
@@ -217,8 +218,12 @@ class route(object):
                 bits = bits[1:]
         name = self.parameters.get('name', '_'.join(bits))
         self.parameters['name'] = name
-        method = (self.method or method or 'get').lower()
-        rule = Route(self.rule or name, defaults=self.defaults)
+        method = self.method or method or 'get'
+        if isinstance(method, (list, tuple)):
+            method = tuple((m.lower() for m in method))
+        else:
+            method = method.lower()
+        rule = Route(self.rule or name, defaults=self.defaults, is_re=self.re)
         callable.rule_method = rule_info(rule, method, self.parameters,
                                          self.position, self.order)
         return callable
@@ -255,7 +260,7 @@ class Route(object):
 
     .. _werkzeug: https://github.com/mitsuhiko/werkzeug
     '''
-    def __init__(self, rule, defaults=None):
+    def __init__(self, rule, defaults=None, is_re=False):
         rule = remove_double_slash('/%s' % rule)
         self.defaults = defaults if defaults is not None else {}
         self.is_leaf = not rule.endswith('/')
@@ -286,9 +291,9 @@ class Route(object):
                     self._converters[variable] = convobj
                     self.variables.add(str(variable))
                 else:
-                    variable = bit
-                    regex_parts.append(re.escape(variable))
-                    breadcrumbs.append((False, variable))
+                    variable = bit if is_re else re.escape(bit)
+                    regex_parts.append(variable)
+                    breadcrumbs.append((False, bit))
 
         self.breadcrumbs = tuple(breadcrumbs)
         self._regex_string = '/'.join(regex_parts)
@@ -386,7 +391,7 @@ class Route(object):
             remaining = path[match.end():]
             groups = match.groupdict()
             result = {}
-            for name, value in iteritems(groups):
+            for name, value in groups.items():
                 try:
                     value = self._converters[name].to_python(value)
                 except Http404:
@@ -415,12 +420,14 @@ class Route(object):
     def __add__(self, other):
         cls = self.__class__
         defaults = self.defaults.copy()
+        is_re = False
         if isinstance(other, cls):
             rule = other.rule
             defaults.update(other.defaults)
+            is_re = True
         else:
             rule = str(other)
-        return cls('%s/%s' % (self.rule, rule), defaults)
+        return cls('%s/%s' % (self.rule, rule), defaults, is_re=is_re)
 
 
 class BaseConverter(object):
@@ -553,7 +560,7 @@ class FloatConverter(NumberConverter):
     num_convert = float
 
     def __init__(self, min=None, max=None):
-        super(FloatConverter, self).__init__(0, min, max)
+        super().__init__(0, min, max)
 
 
 def parse_converter_args(argstr):
