@@ -11,13 +11,20 @@ sub-routers for handling additional urls::
                                               'application/json'))
 
         def get(self, request):
-            "This method handle request with get-method"
+            "This method handle requests with get-method"
             ...
 
         def post(self, request):
-            "This method handle request with post-method"
+            "This method handle requests with post-method"
             ...
 
+        def delete(self, request):
+            "This method handle requests with delete-method"
+            ...
+
+        ...
+
+
     middleware = Page('/bla')
 
 
@@ -76,7 +83,6 @@ from pulsar import Http404, HttpException
 from .route import Route
 from .utils import wsgi_request
 from .content import Html
-from .structures import ContentAccept
 
 
 __all__ = ['Router', 'MediaRouter', 'FileRouter', 'MediaMixin',
@@ -101,6 +107,15 @@ def update_args(urlargs, args):
     return args
 
 
+def _get_default(parent, name):
+    if name in parent.defaults:
+        return getattr(parent, name)
+    elif parent._parent:
+        return _get_default(parent._parent, name)
+    else:
+        raise AttributeError
+
+
 class RouterParam(object):
     '''A :class:`RouterParam` is a way to flag a :class:`Router` parameter
     so that children can inherit the value if they don't define their own.
@@ -127,6 +142,7 @@ class RouterType(type):
         for key, value in list(attrs.items()):
             if isinstance(value, RouterParam):
                 defaults[key] = attrs.pop(key).value
+
         no_rule = set(attrs) - set((x[0] for x in rule_methods))
         base_rules = []
         for base in reversed(bases):
@@ -156,7 +172,7 @@ class RouterType(type):
         return super(RouterType, cls).__new__(cls, name, bases, attrs)
 
 
-class Router(RouterType('RouterBase', (object,), {})):
+class Router(metaclass=RouterType):
     '''A :ref:`WSGI middleware <wsgi-middleware>` to handle client requests
     on multiple :ref:`routes <apps-wsgi-route>`.
 
@@ -218,7 +234,7 @@ class Router(RouterType('RouterBase', (object,), {})):
         if not isinstance(rule, Route):
             rule = Route(rule)
         self._route = rule
-        parameters.setdefault('name', rule.name)
+        parameters.setdefault('name', rule.name or self.name or '')
         self._set_params(parameters)
         self.routes = []
         # add routes specified via the initialiser first
@@ -291,33 +307,42 @@ class Router(RouterType('RouterBase', (object,), {})):
         '''
         return self.full_route.url(**urlargs)
 
-    def getparam(self, name, default=None):
+    def getparam(self, name, default=None, parents=False):
         '''A parameter in this :class:`.Router`
         '''
-        try:
-            return getattr(self, name)
-        except AttributeError:
-            return default
+        value = getattr(self, name, None)
+        if value is None:
+            if parents and self._parent:
+                return self._parent.getparam(name, default, parents)
+            else:
+                return default
+        else:
+            return value
 
     def __getattr__(self, name):
-        '''Check the value of a :attr:`parameters` ``name``.
+        '''Get the value of the ``name`` attribute.
 
-        If the parameter is not available, retrieve the parameter from the
+        If the ``name`` is not available, retrieve it from the
         :attr:`parent` :class:`Router` if it exists.
         '''
-        if name in self.defaults:
-            if self._parent:
-                try:
-                    return getattr(self._parent, name)
-                except AttributeError:
-                    pass
-            return self.defaults[name]
+        available = False
+        value = None
 
-        if self._parent:
-            return getattr(self._parent, name)
-        else:
-            raise AttributeError("'%s' object has no attribute '%s'" %
-                                 (self.__class__.__name__, name))
+        if name in self.defaults:
+            available = True
+            value = self.defaults[name]
+
+        if self._parent and value is None:
+            try:
+                return _get_default(self._parent, name)
+            except AttributeError:
+                pass
+
+        if available:
+            return value
+
+        raise AttributeError("'%s' object has no attribute '%s'" %
+                             (self.__class__.__name__, name))
 
     def content_type(self, request):
         '''Evaluate the content type for the response to a client ``request``.
@@ -337,7 +362,7 @@ class Router(RouterType('RouterBase', (object,), {})):
             return ct
 
     def __repr__(self):
-        return self.route.__repr__()
+        return self.full_route.__repr__()
 
     def __call__(self, environ, start_response=None):
         path = environ.get('PATH_INFO') or '/'
@@ -383,7 +408,7 @@ class Router(RouterType('RouterBase', (object,), {})):
             return response_wrapper(callable, request)
         return callable(request)
 
-    def add_child(self, router):
+    def add_child(self, router, index=None):
         '''Add a new :class:`Router` to the :attr:`routes` list.
         '''
         assert isinstance(router, Router), 'Not a valid Router'
@@ -400,7 +425,10 @@ class Router(RouterType('RouterBase', (object,), {})):
         if router.parent:
             router.parent.remove_child(router)
         router._parent = self
-        self.routes.append(router)
+        if index is None:
+            self.routes.append(router)
+        else:
+            self.routes.insert(index, router)
         return router
 
     def remove_child(self, router):
@@ -458,16 +486,13 @@ class Router(RouterType('RouterBase', (object,), {})):
                     router._set_params(params)
                     break
         if method and handler:
-            setattr(router, method, handler)
+            if isinstance(method, tuple):
+                for m in method:
+                    setattr(router, m, handler)
+            else:
+                setattr(router, method, handler)
         return router
 
-    @classmethod
-    def copy(cls):
-        '''Create a copy of this router without routes'''
-        newcls = RouterType(cls.__name__ + 'Copy', (cls,), {})
-        newcls.rule_methods.clear()
-        return newcls
-
     # INTERNALS
     def _set_params(self, parameters):
         for name, value in parameters.items():
@@ -509,20 +534,25 @@ class MediaMixin(object):
         :param mtime: the modification time of the item in question.
         :param size: the size of the item.
         '''
+        header_mtime = self.modified_since(header, size)
+        if header_mtime and header_mtime <= mtime:
+            return False
+        return True
+
+    def modified_since(self, header, size=0):
         try:
             if header is None:
                 raise ValueError
-            matches = re.match(r"^([^;]+)(; length=([0-9]+))?$", header,
+            matches = re.match(r"^([^;]+)(; length=([0-9]+))?$",
+                               header,
                                re.IGNORECASE)
             header_mtime = mktime_tz(parsedate_tz(matches.group(1)))
             header_len = matches.group(3)
             if header_len and int(header_len) != size:
-                raise ValueError()
-            if mtime > header_mtime:
-                raise ValueError()
+                raise ValueError
+            return header_mtime
         except (AttributeError, ValueError, OverflowError):
-            return True
-        return False
+            pass
 
     def directory_index(self, request, fullpath):
         names = [Html('a', '../', href='../', cn='folder')]
@@ -572,7 +602,7 @@ class MediaRouter(Router, MediaMixin):
     def __init__(self, rule, path, show_indexes=False,
                  default_suffix=None, default_file='index.html',
                  raise_404=True, **params):
-        super(MediaRouter, self).__init__('%s/<path:path>' % rule, **params)
+        super().__init__('%s/<path:path>' % rule, **params)
         self._default_suffix = default_suffix
         self._default_file = default_file
         self._show_indexes = show_indexes
@@ -619,7 +649,7 @@ class FileRouter(Router, MediaMixin):
     cache_control = CacheControl(maxage=86400)
 
     def __init__(self, route, file_path, status_code=None, raise_404=True):
-        super(FileRouter, self).__init__(route)
+        super().__init__(route)
         self._status_code = status_code
         self._file_path = file_path
         self._raise_404 = raise_404
