@@ -16,37 +16,37 @@ import sys
 import time
 import os
 import socket
+import io
+from asyncio import wait_for
 from wsgiref.handlers import format_date_time
+from urllib.parse import urlparse, unquote
 
 import pulsar
-from pulsar import (reraise, HttpException, ProtocolError, Future, task,
-                    From, isfuture, chain_future)
-from pulsar.utils.pep import is_string, native_str
-from pulsar.utils.httpurl import (Headers, unquote, has_empty_content,
-                                  host_and_port_default, http_parser,
-                                  urlparse, iri_to_uri, DEFAULT_CHARSET)
+from pulsar import (reraise, HttpException, ProtocolError, task, isfuture,
+                    BadRequest)
+from pulsar.utils.pep import native_str
+from pulsar.utils.httpurl import (Headers, has_empty_content, http_parser,
+                                  iri_to_uri, http_chunks)
 
-from pulsar.utils.internet import format_address, is_tls
+from pulsar.utils.internet import is_tls
 from pulsar.async.protocols import ProtocolConsumer
 
-from .utils import handle_wsgi_error, wsgi_request, HOP_HEADERS
+from .utils import (handle_wsgi_error, wsgi_request, HOP_HEADERS,
+                    log_wsgi_info, LOGGER)
+from .formdata import http_protocol, HttpBodyReader
+
+__all__ = ['HttpServerResponse', 'test_wsgi_environ', 'AbortWsgi']
 
 
-__all__ = ['HttpServerResponse', 'MAX_CHUNK_SIZE', 'test_wsgi_environ']
+MAX_TIME_IN_LOOP = 0.2
 
 
-MAX_CHUNK_SIZE = 65536
-MAX_TIME_IN_LOOP = 0.5
-
-
-class FakeConnection(object):
-
-    def __init__(self, loop=None):
-        self._loop = loop
+class AbortWsgi(Exception):
+    pass
 
 
 def test_wsgi_environ(path=None, method=None, headers=None, extra=None,
-                      secure=False, loop=None):
+                      secure=False, loop=None, body=None):
     '''An function to create a WSGI environment dictionary for testing.
 
     :param url: the resource in the ``PATH_INFO``.
@@ -64,7 +64,8 @@ def test_wsgi_environ(path=None, method=None, headers=None, extra=None,
     parsed = urlparse(path)
     if 'host' not in request_headers:
         if not parsed.netloc:
-            path = '%s%s' % ('https://:443' if secure else 'http://:80', path)
+            scheme = ('https' if secure else 'http')
+            path = '%s://127.0.0.1%s' % (scheme, path)
         else:
             request_headers['host'] = parsed.netloc
     #
@@ -72,97 +73,24 @@ def test_wsgi_environ(path=None, method=None, headers=None, extra=None,
     data = data.encode('latin1')
     parser.execute(data, len(data))
     #
-    headers = Headers()
-    stream = StreamReader(request_headers, parser)
-    extra = extra or {}
-    extra['pulsar.connection'] = FakeConnection(loop=loop)
-    return wsgi_environ(stream, ('127.0.0.1', 8060), '777.777.777.777:8080',
-                        headers, https=secure, extra=extra)
+    stream = io.BytesIO(body or b'')
+    return wsgi_environ(stream, parser, request_headers,
+                        ('127.0.0.1', 8060), '255.0.1.2:8080',
+                        Headers(), https=secure, extra=extra)
 
 
-class StreamReader:
-    _expect_sent = None
-    _waiting = None
+def wsgi_environ(stream, parser, request_headers, address, client_address,
+                 headers, server_software=None, https=False, extra=None):
+    '''Build the WSGI Environment dictionary
 
-    def __init__(self, headers, parser, transport=None):
-        self.headers = headers
-        self.parser = parser
-        self.transport = transport
-        self.buffer = b''
-        self.on_message_complete = Future()
-
-    def __repr__(self):
-        return repr(self.transport)
-    __str__ = __repr__
-
-    def done(self):
-        '''``True`` when the full HTTP message has been read.
-        '''
-        return self.on_message_complete.done()
-
-    def protocol(self):
-        version = self.parser.get_version()
-        return "HTTP/%s" % ".".join(('%s' % v for v in version))
-
-    def waiting_expect(self):
-        '''``True`` when the client is waiting for 100 Continue.
-        '''
-        if self._expect_sent is None:
-            if (not self.parser.is_message_complete() and
-                    self.headers.has('expect', '100-continue')):
-                return True
-            self._expect_sent = ''
-        return False
-
-    def recv(self):
-        '''Read bytes in the buffer.
-        '''
-        if self.waiting_expect():
-            if self.parser.get_version() < (1, 1):
-                raise HttpException(status=417)
-            else:
-                msg = '%s 100 Continue\r\n\r\n' % self.protocol()
-                self._expect_sent = msg
-                self.transport.write(msg.encode(DEFAULT_CHARSET))
-        return self.parser.recv_body()
-
-    def read(self, maxbuf=None):
-        '''Return bytes in the buffer.
-
-        If the stream is not yet ready, return a :class:`asyncio.Future`
-        which results in the bytes read.
-        '''
-        if not self._waiting:
-            body = self.recv()
-            if self.done():
-                return self._getvalue(body, maxbuf)
-            else:
-                self._waiting = chain_future(
-                    self.on_message_complete,
-                    lambda r: self._getvalue(body, maxbuf))
-                return self._waiting
-        else:
-            return self._waiting
-
-    def fail(self):
-        if self.waiting_expect():
-            raise HttpException(status=417)
-
-    #    INTERNALS
-    def _getvalue(self, body, maxbuf):
-        if self.buffer:
-            body = self.buffer + body
-        body = body + self.recv()
-        if maxbuf and len(body) > maxbuf:
-            body, self.buffer = body[:maxbuf], body[maxbuf:]
-        return body
-
-
-def wsgi_environ(stream, address, client_address, headers,
-                 server_software=None, https=False, extra=None):
-    protocol = stream.protocol()
-    parser = stream.parser
-    request_headers = stream.headers
+    :param stream: a wsgi stream object
+    :param parser: pulsar HTTP parser
+    :param request_headers: headers of request
+    :param address: server address
+    :param client_address: client address
+    :param headers: container for response headers
+    '''
+    protocol = http_protocol(parser)
     raw_uri = parser.get_url()
     request_uri = urlparse(raw_uri)
     #
@@ -215,7 +143,7 @@ def wsgi_environ(stream, address, client_address, headers,
     environ['wsgi.url_scheme'] = url_scheme
     if url_scheme == 'https':
         environ['HTTPS'] = 'on'
-    if is_string(forward):
+    if isinstance(forward, str):
         # we only took the last one
         # http://en.wikipedia.org/wiki/X-Forwarded-For
         if forward.find(",") >= 0:
@@ -227,12 +155,8 @@ def wsgi_environ(stream, address, client_address, headers,
         remote = forward
     environ['REMOTE_ADDR'] = remote[0]
     environ['REMOTE_PORT'] = str(remote[1])
-    if not host and protocol == 'HTTP/1.0':
-        host = format_address(address)
-    if host:
-        host = host_and_port_default(url_scheme, host)
-        environ['SERVER_NAME'] = socket.getfqdn(host[0])
-        environ['SERVER_PORT'] = host[1]
+    environ['SERVER_NAME'] = socket.getfqdn(address[0])
+    environ['SERVER_PORT'] = address[1]
     path_info = request_uri.path
     if path_info is not None:
         if script_name:
@@ -244,19 +168,7 @@ def wsgi_environ(stream, address, client_address, headers,
     return environ
 
 
-def chunk_encoding(chunk):
-    '''Write a chunk::
-
-    chunk-size(hex) CRLF
-    chunk-data CRLF
-
-If the size is 0, this is the last chunk, and an extra CRLF is appended.
-'''
-    head = ("%X\r\n" % len(chunk)).encode('utf-8')
-    return head + chunk + b'\r\n'
-
-
-def keep_alive(headers, version):
+def keep_alive(headers, version, method):
     """ return True if the connection should be kept alive"""
     conn = set((v.lower() for v in headers.get_all('connection', ())))
     if "close" in conn:
@@ -269,6 +181,8 @@ def keep_alive(headers, version):
     elif version == (1, 1):
         headers['connection'] = 'keep-alive'
         return True
+    elif method == 'CONNECT':
+        return True
     else:
         return False
 
@@ -289,13 +203,14 @@ class HttpServerResponse(ProtocolConsumer):
     '''
     _status = None
     _headers_sent = None
-    _stream = None
+    _body_reader = None
     _buffer = None
+    _logger = LOGGER
     SERVER_SOFTWARE = pulsar.SERVER_SOFTWARE
     ONE_TIME_EVENTS = ProtocolConsumer.ONE_TIME_EVENTS + ('on_headers',)
 
     def __init__(self, wsgi_callable, cfg, server_software=None, loop=None):
-        super(HttpServerResponse, self).__init__(loop=loop)
+        super().__init__(loop=loop)
         self.wsgi_callable = wsgi_callable
         self.cfg = cfg
         self.parser = http_parser(kind=0)
@@ -320,16 +235,21 @@ class HttpServerResponse(ProtocolConsumer):
         '''
         parser = self.parser
         processed = parser.execute(data, len(data))
-        if not self._stream and parser.is_headers_complete():
-            headers = Headers(parser.get_headers(), kind='client')
-            self._stream = StreamReader(headers, parser, self.transport)
-            self._response(self.wsgi_environ())
+        if parser.is_headers_complete():
+            if not self._body_reader:
+                headers = Headers(parser.get_headers(), kind='client')
+                self._body_reader = HttpBodyReader(headers,
+                                                   parser,
+                                                   self.transport,
+                                                   loop=self._loop)
+                self._response(self.wsgi_environ())
+            body = parser.recv_body()
+            if body:
+                self._body_reader.feed_data(body)
         #
         if parser.is_message_complete():
             #
-            # Stream has the whole body
-            if not self._stream.on_message_complete.done():
-                self._stream.on_message_complete.set_result(None)
+            self._body_reader.feed_eof()
 
             if processed < len(data):
                 if not self._buffer:
@@ -427,7 +347,7 @@ class HttpServerResponse(ProtocolConsumer):
         :param force: Optional flag used internally
         :return: a :class:`~asyncio.Future` or the number of bytes written
         '''
-        write = super(HttpServerResponse, self).write
+        write = super().write
         chunks = []
         if not self._headers_sent:
             tosend = self.get_headers()
@@ -436,15 +356,11 @@ class HttpServerResponse(ProtocolConsumer):
             chunks.append(self._headers_sent)
         if data:
             if self.chunked:
-                while len(data) >= MAX_CHUNK_SIZE:
-                    chunk, data = data[:MAX_CHUNK_SIZE], data[MAX_CHUNK_SIZE:]
-                    chunks.append(chunk_encoding(chunk))
-                if data:
-                    chunks.append(chunk_encoding(data))
+                chunks.extend(http_chunks(data))
             else:
                 chunks.append(data)
         elif force and self.chunked:
-            chunks.append(chunk_encoding(data))
+            chunks.extend(http_chunks(data, True))
         if chunks:
             return write(b''.join(chunks))
 
@@ -455,18 +371,21 @@ class HttpServerResponse(ProtocolConsumer):
         exc_info = None
         response = None
         done = False
+        alive = self.cfg.keep_alive or 15
         while not done:
             done = True
             try:
                 if exc_info is None:
-                    if 'SERVER_NAME' not in environ:
-                        raise HttpException(status=400)
+                    if (not environ.get('HTTP_HOST') and
+                            environ['SERVER_PROTOCOL'] != 'HTTP/1.0'):
+                        raise BadRequest
                     response = self.wsgi_callable(environ, self.start_response)
+                    if isfuture(response):
+                        response = yield from wait_for(response, alive)
                 else:
                     response = handle_wsgi_error(environ, exc_info)
-                #
-                if isfuture(response):
-                    response = yield From(response)
+                    if isfuture(response):
+                        response = yield from wait_for(response, alive)
                 #
                 if exc_info:
                     self.start_response(response.status,
@@ -477,11 +396,11 @@ class HttpServerResponse(ProtocolConsumer):
                 start = loop.time()
                 for chunk in response:
                     if isfuture(chunk):
-                        chunk = yield From(chunk)
+                        chunk = yield from wait_for(chunk, alive)
                         start = loop.time()
                     result = self.write(chunk)
                     if isfuture(result):
-                        yield From(result)
+                        yield from wait_for(result, alive)
                         start = loop.time()
                     else:
                         time_in_loop = loop.time() - start
@@ -489,26 +408,31 @@ class HttpServerResponse(ProtocolConsumer):
                             self.logger.debug(
                                 'Released the event loop after %.3f seconds',
                                 time_in_loop)
-                            yield From(None)
+                            yield None
                             start = loop.time()
                 #
                 # make sure we write headers and last chunk if needed
                 self.write(b'', True)
 
-            except IOError:     # client disconnected, end this connection
+            # client disconnected, end this connection
+            except (IOError, AbortWsgi):
                 self.finished()
             except Exception:
                 if wsgi_request(environ).cache.handle_wsgi_error:
                     self.keep_alive = False
+                    self._write_headers()
                     self.connection.close()
                     self.finished()
                 else:
                     done = False
                     exc_info = sys.exc_info()
             else:
-                if not self.keep_alive:
-                    self.connection.close()
+                log_wsgi_info(self.logger.info, environ, self.status)
                 self.finished()
+                if not self.keep_alive:
+                    self.logger.debug('No keep alive, closing connection %s',
+                                      self.connection)
+                    self.connection.close()
             finally:
                 if hasattr(response, 'close'):
                     try:
@@ -556,15 +480,19 @@ class HttpServerResponse(ProtocolConsumer):
         transport = self.transport
         https = True if is_tls(transport.get_extra_info('socket')) else False
         multiprocess = (self.cfg.concurrency == 'process')
-        environ = wsgi_environ(self._stream,
+        environ = wsgi_environ(self._body_reader,
+                               self.parser,
+                               self._body_reader.headers,
                                transport.get_extra_info('sockname'),
-                               self.address, self.headers,
+                               self.address,
+                               self.headers,
                                self.SERVER_SOFTWARE,
                                https=https,
                                extra={'pulsar.connection': self.connection,
                                       'pulsar.cfg': self.cfg,
                                       'wsgi.multiprocess': multiprocess})
-        self.keep_alive = keep_alive(self.headers, self.parser.get_version())
+        self.keep_alive = keep_alive(self.headers, self.parser.get_version(),
+                                     environ['REQUEST_METHOD'])
         self.headers.update([('Server', self.SERVER_SOFTWARE),
                              ('Date', format_date_time(time.time()))])
         return environ
@@ -572,3 +500,9 @@ class HttpServerResponse(ProtocolConsumer):
     def _new_request(self, _, exc=None):
         connection = self._connection
         connection.data_received(self._buffer)
+
+    def _write_headers(self):
+        if not self._headers_sent:
+            if self.content_length:
+                self.headers['Content-Length'] = '0'
+            self.write(b'')
