@@ -1,10 +1,9 @@
 import re
 import codecs
 
-from pulsar.utils.pep import ispy3k
 from pulsar.utils.structures import FrozenDict
 
-string_type = str if ispy3k else basestring
+
 _locale_delim_re = re.compile(r'[_-]')
 
 
@@ -85,7 +84,7 @@ class Accept(tuple):
            This used to raise :exc:`IndexError`, which was inconsistent
            with the list API.
         """
-        if isinstance(key, string_type):
+        if isinstance(key, str):
             for idx, (item, quality) in enumerate(self):
                 if self._value_matches(key, item):
                     return idx
@@ -103,10 +102,6 @@ class Accept(tuple):
             return -1
 
     def values(self):
-        """Return a list of the values, not the qualities."""
-        return list(self.itervalues())
-
-    def itervalues(self):
         """Iterate over all values."""
         for item in self:
             yield item[0]
@@ -134,8 +129,8 @@ class Accept(tuple):
         if matches:
             best_quality = -1
             result = default
-            for server_item in matches:
-                for client_item, quality in self:
+            for client_item, quality in self:
+                for server_item in matches:
                     if quality <= best_quality:
                         break
                     if self._value_matches(server_item, client_item):
