@@ -8,15 +8,15 @@ import textwrap
 import logging
 from datetime import datetime, timedelta
 from email.utils import formatdate
-
+from urllib.parse import parse_qsl
 
 from pulsar import format_traceback
 from pulsar.utils.system import json
 from pulsar.utils.structures import MultiValueDict
 from pulsar.utils.html import escape
 from pulsar.utils.pep import to_string
-from pulsar.utils.httpurl import (has_empty_content, REDIRECT_CODES, iteritems,
-                                  parse_qsl, HTTPError, parse_dict_header,
+from pulsar.utils.httpurl import (has_empty_content, REDIRECT_CODES,
+                                  HTTPError, parse_dict_header,
                                   JSON_CONTENT_TYPES)
 
 from .structures import Accept, RequestCacheControl
@@ -41,7 +41,7 @@ HOP_HEADERS = frozenset(('connection',
                          'upgrade')
                         )
 
-logger = logging.getLogger('pulsar.wsgi')
+LOGGER = logging.getLogger('pulsar.wsgi')
 error_css = '''
 .pulsar-error {
     width: 500px;
@@ -62,6 +62,17 @@ def set_wsgi_request_class(RequestClass):
     _RequestClass = RequestClass
 
 
+def log_wsgi_info(log, environ, status, exc=None):
+    if not environ.get('pulsar.logged'):
+        environ['pulsar.logged'] = True
+        msg = '' if not exc else ' - %s' % exc
+        log('%s %s %s - %s%s',
+            environ.get('REQUEST_METHOD'),
+            environ.get('RAW_URI'),
+            environ.get('SERVER_PROTOCOL'),
+            status, msg)
+
+
 def cookie_date(epoch_seconds=None):
     """Formats the time to ensure compatibility with Netscape's cookie
     standard.
@@ -84,7 +95,9 @@ def set_cookie(cookies, key, value='', max_age=None, expires=None, path='/',
     cookies[key] = value
     if expires is not None:
         if isinstance(expires, datetime):
-            delta = expires - expires.utcnow()
+            now = (expires.now(expires.tzinfo) if expires.tzinfo else
+                   expires.utcnow())
+            delta = expires - now
             # Add one second so the date matches exactly (a fraction of
             # time gets lost between converting to a timedelta and
             # then the date string).
@@ -193,7 +206,7 @@ class dump_environ(object):
 
     def __str__(self):
         def _():
-            for k, v in iteritems(self.environ):
+            for k, v in self.environ.items():
                 try:
                     v = str(v)
                 except Exception as e:
@@ -224,23 +237,21 @@ def handle_wsgi_error(environ, exc):
         response.headers.update(getattr(exc, 'headers', None) or ())
     path = '@ %s "%s"' % (request.method, request.path)
     status = response.status_code
-    if status == 500:
-        logger.critical('Unhandled exception during HTTP response %s.%s',
-                        path, dump_environ(environ), exc_info=exc_info)
+    if status >= 500:
+        LOGGER.critical('%s - %s.\n%s', exc, path,
+                        dump_environ(environ), exc_info=exc_info)
     else:
-        msg = str(exc)
-        msg = '' if not msg else ' - %s' % msg
-        logger.warning('HTTP %s %s%s', response.status, path, msg)
+        log_wsgi_info(LOGGER.warning, environ, response.status, exc)
     if has_empty_content(status, request.method) or status in REDIRECT_CODES:
         response.content_type = None
         response.content = None
     else:
         request.cache.pop('html_document', None)
-        renderer = environ.get('error.handler', render_error)
+        renderer = environ.get('error.handler') or render_error
         try:
             content = renderer(request, exc)
         except Exception:
-            logger.critical('Error while rendering error', exc_info=True)
+            LOGGER.critical('Error while rendering error', exc_info=True)
             response.content_type = 'text/plain'
             content = 'Critical server error'
         if content is not response:
@@ -254,8 +265,9 @@ def render_error(request, exc):
     debug = cfg.debug if cfg else False
     response = request.response
     if not response.content_type:
+        content_type = request.get('default.content_type')
         response.content_type = request.content_types.best_match(
-            DEFAULT_RESPONSE_CONTENT_TYPES)
+            content_type or DEFAULT_RESPONSE_CONTENT_TYPES)
     content_type = None
     if response.content_type:
         content_type = response.content_type.split(';')[0]
@@ -264,7 +276,7 @@ def render_error(request, exc):
     if debug:
         msg = render_error_debug(request, exc, is_html)
     else:
-        msg = error_messages.get(response.status_code) or str(exc)
+        msg = escape(error_messages.get(response.status_code) or exc)
         if is_html:
             msg = textwrap.dedent("""
                 <h1>{0[reason]}</h1>
@@ -275,7 +287,6 @@ def render_error(request, exc):
     #
     if content_type == 'text/html':
         doc = HtmlDocument(title=response.status)
-        doc.head.links.append('bootstrap_css')
         doc.head.embedded_css.append(error_css)
         doc.body.append(Html('div', msg, cn='pulsar-error'))
         return doc.render(request)
