@@ -40,30 +40,28 @@ Wsgi Response
 .. _AJAX: http://en.wikipedia.org/wiki/Ajax_(programming)
 .. _TLS: http://en.wikipedia.org/wiki/Transport_Layer_Security
 '''
-import re
 from functools import reduce, partial
-from io import BytesIO
+from http.client import responses
 
 from pulsar import Future, chain_future
-from pulsar.utils.system import json
-from pulsar.utils.multipart import parse_form_data, parse_options_header
 from pulsar.utils.structures import AttributeDictionary
-from pulsar.utils.httpurl import (Headers, SimpleCookie, responses,
-                                  has_empty_content, ispy3k, REDIRECT_CODES,
+from pulsar.utils.httpurl import (Headers, SimpleCookie,
+                                  has_empty_content, REDIRECT_CODES,
                                   ENCODE_URL_METHODS, JSON_CONTENT_TYPES,
-                                  remove_double_slash, iri_to_uri)
+                                  remove_double_slash, iri_to_uri,
+                                  is_absolute_uri, parse_options_header)
 
 from .content import HtmlDocument
 from .utils import (set_wsgi_request_class, set_cookie, query_dict,
                     parse_accept_header)
 from .structures import ContentAccept, CharsetAccept, LanguageAccept
+from .formdata import parse_form_data
 
 
 __all__ = ['EnvironMixin', 'WsgiResponse',
            'WsgiRequest', 'cached_property']
 
 MAX_BUFFER_SIZE = 2**16
-absolute_http_url_re = re.compile(r"^https?://", re.I)
 
 
 def redirect(path, code=None, permanent=False):
@@ -87,7 +85,7 @@ def cached_property(method):
 
 def wsgi_encoder(gen, encoding):
     for data in gen:
-        if not isinstance(data, bytes):
+        if isinstance(data, str):
             yield data.encode(encoding)
         else:
             yield data
@@ -133,6 +131,7 @@ class WsgiResponse(object):
         A python :class:`SimpleCookie` container of cookies included in the
         request as well as cookies set during the response.
     '''
+    _iterated = False
     _started = False
     DEFAULT_STATUS_CODE = 200
 
@@ -153,6 +152,10 @@ class WsgiResponse(object):
     def started(self):
         return self._started
 
+    @property
+    def iterated(self):
+        return self._iterated
+
     @property
     def path(self):
         if self.environ:
@@ -168,30 +171,20 @@ class WsgiResponse(object):
         if self.environ:
             return self.environ.get('pulsar.connection')
 
-    @property
-    def environ_cache(self):
-        if self.environ:
-            return self.environ.get('pulsar.cache')
-
     @property
     def content(self):
         return self._content
 
     @content.setter
     def content(self, content):
-        if not self._started:
+        if not self._iterated:
             if content is None:
                 content = ()
-            elif ispy3k:
+            else:
                 if isinstance(content, str):
                     if not self.encoding:   # use utf-8 if not set
                         self.encoding = 'utf-8'
                     content = content.encode(self.encoding)
-            else:   # pragma    nocover
-                if isinstance(content, unicode):
-                    if not self.encoding:  # use utf-8 if not set
-                        self.encoding = 'utf-8'
-                    content = content.encode(self.encoding)
 
             if isinstance(content, bytes):
                 content = (content,)
@@ -247,10 +240,16 @@ class WsgiResponse(object):
         if not self.is_streamed:
             return reduce(lambda x, y: x+len(y), self.content, 0)
 
+    def start(self, start_response):
+        assert not self._started
+        self._started = True
+        return start_response(self.status, self.get_headers())
+
     def __iter__(self):
-        if self._started:
+        if self._iterated:
             raise RuntimeError('WsgiResponse can be iterated once only')
         self._started = True
+        self._iterated = True
         if self.is_streamed:
             return wsgi_encoder(self.content, self.encoding or 'utf-8')
         else:
@@ -376,7 +375,7 @@ class EnvironMixin(object):
 class WsgiRequest(EnvironMixin):
     '''An :class:`EnvironMixin` for wsgi requests.'''
     def __init__(self, environ, app_handler=None, urlargs=None):
-        super(WsgiRequest, self).__init__(environ)
+        super().__init__(environ)
         self.cache.cfg = environ.get('pulsar.cfg', {})
         if app_handler:
             self.cache.app_handler = app_handler
@@ -460,12 +459,6 @@ class WsgiRequest(EnvironMixin):
         '''
         return self.cache.cfg
 
-    @property
-    def ipaddress(self):
-        '''internet protocol address of the client
-        '''
-        return self.environ.get('REMOTE_ADDR')
-
     @cached_property
     def response(self):
         '''The :class:`WsgiResponse` for this client request.
@@ -513,7 +506,7 @@ class WsgiRequest(EnvironMixin):
         else:
             return None, {}
 
-    def data_and_files(self, data=True, files=True):
+    def data_and_files(self, data=True, files=True, stream=None):
         '''Retrieve body data.
 
         Returns a two-elements tuple of a
@@ -525,9 +518,13 @@ class WsgiRequest(EnvironMixin):
 
         The result is cached.
         '''
-        value = self.cache.data_and_files
+        if self.method in ENCODE_URL_METHODS:
+            value = {}, None
+        else:
+            value = self.cache.data_and_files
+
         if not value:
-            return self._data_and_files(data, files)
+            return self._data_and_files(data, files, stream)
         elif data and files:
             return value
         elif data:
@@ -542,29 +539,18 @@ class WsgiRequest(EnvironMixin):
         '''
         return self.data_and_files(files=False)
 
-    def _data_and_files(self, data=True, files=True, future=None):
-        result = {}, None
-        chunk = None
+    def _data_and_files(self, data=True, files=True, stream=None, future=None):
         if future is None:
-            stream = self.environ.get('wsgi.input')
-            if self.method not in ENCODE_URL_METHODS and stream:
-                chunk = stream.read()
-                if isinstance(chunk, Future):
-                    return chain_future(
-                        chunk, partial(self._data_and_files, data, files))
+            data_files = parse_form_data(self.environ, stream=stream)
+            if isinstance(data_files, Future):
+                return chain_future(
+                    data_files,
+                    partial(self._data_and_files, data, files, stream))
         else:
-            chunk = future
-        if chunk is not None:
-            content_type, options = self.content_type_options
-            charset = options.get('charset', 'utf-8')
-            if content_type in JSON_CONTENT_TYPES:
-                result = json.loads(chunk.decode(charset)), None
-            else:
-                self.environ['wsgi.input'] = BytesIO(chunk)
-                result = parse_form_data(self.environ, charset)
-            self.environ['wsgi.input'] = BytesIO(chunk)
-        self.cache.data_and_files = result
-        return self.data_and_files(data, files)
+            data_files = future
+
+        self.cache.data_and_files = data_files
+        return self.data_and_files(data, files, stream)
 
     @cached_property
     def url_data(self):
@@ -624,7 +610,7 @@ class WsgiRequest(EnvironMixin):
         If no ``location`` is specified, the relative URI is built from
         :meth:`full_path`.
         '''
-        if not location or not absolute_http_url_re.match(location):
+        if not is_absolute_uri(location):
             location = self.full_path(location)
             if not scheme:
                 scheme = self.is_secure and 'https' or 'http'
