@@ -1,14 +1,15 @@
 import os
-import sys
 import threading
 import logging
-from functools import wraps
 from collections import OrderedDict
 from threading import current_thread
+import asyncio
+
+from asyncio import iscoroutine, coroutine
 
 from pulsar.utils.config import Global
-from pulsar.utils.pep import ispy3k
-from pulsar.utils.system import platform, current_process
+from pulsar.utils.system import current_process
+
 
 __all__ = ['get_event_loop',
            'new_event_loop',
@@ -21,70 +22,31 @@ __all__ = ['get_event_loop',
            'logger',
            'NOTHING',
            'SELECTORS',
-           'appengine',
            'Future',
-           'ConnectionRefusedError',
-           'ConnectionResetError',
            'reraise',
-           'From',
-           'async',
-           'sleep',
-           'iscoroutine',
-           'get_io_loop',
+           'coroutine',
+           'is_async',
            'CANCELLED_ERRORS']
 
-# Dance between different versions. So boring!
-appengine = False
 
-if platform.is_appengine:   # pragma    nocover
-    from . import appengine as asyncio
-    trollius = asyncio
-    _PENDING = 'PENDING'
-    _CANCELLED = 'CANCELLED'
-    _FINISHED = 'FINISHED'
-    _FUTURE_CLASSES = asyncio._FUTURE_CLASSES
-    _EVENT_LOOP_CLASSES = ()
-    BaseEventLoop = asyncio.BaseEventLoop
-    _StopError = asyncio._StopError
-    appengine = True
-    reraise = asyncio.reraise
-    CANCELLED_ERRORS = ()
+_EVENT_LOOP_CLASSES = (asyncio.AbstractEventLoop,)
+CANCELLED_ERRORS = (asyncio.CancelledError,)
 
-else:
 
-    # Set the debug flags before importing asyncio
-    if '--debug' in sys.argv:   # pragma    nocover
-        os.environ['PYTHONASYNCIODEBUG'] = 'debug'
-        os.environ['TROLLIUSDEBUG'] = 'debug'
+def reraise(tp, value, tb=None):
+    if value.__traceback__ is not tb:
+        raise value.with_traceback(tb)
+    raise value
 
-    import trollius
+Future = asyncio.Future
 
-    try:
-        import asyncio
-    except ImportError:     # pragma    nocover
-        asyncio = trollius
 
-    from trollius.futures import (_PENDING, _CANCELLED, _FINISHED,
-                                  _FUTURE_CLASSES)
-    from trollius.base_events import BaseEventLoop, _StopError
-    from trollius import selectors, events
-    from trollius.py33_exceptions import reraise
+def isfuture(x):
+    return isinstance(x, Future)
 
-    _EVENT_LOOP_CLASSES = (asyncio.AbstractEventLoop,
-                           trollius.AbstractEventLoop)
-    CANCELLED_ERRORS = (asyncio.CancelledError, trollius.CancelledError)
 
-Future = trollius.Future
-From = trollius.From
-Return = trollius.Return
-async = trollius.async
-sleep = trollius.sleep
-iscoroutine = trollius.iscoroutine
-iscoroutinefunction = trollius.iscoroutinefunction
-ConnectionRefusedError = trollius.ConnectionRefusedError
-ConnectionResetError = trollius.ConnectionResetError
-
-isfuture = lambda x: isinstance(x, _FUTURE_CLASSES)
+def is_async(c):
+    return isfuture(c) or iscoroutine(c)
 
 
 LOGGER = logging.getLogger('pulsar')
@@ -122,13 +84,6 @@ if default_selector:
 get_event_loop = asyncio.get_event_loop
 
 
-def get_io_loop(loop=None):
-    if not isinstance(loop, _EVENT_LOOP_CLASSES):
-        actor = get_actor()
-        return actor._loop if actor else new_event_loop()
-    return loop
-
-
 def new_event_loop(**kwargs):
     '''Obtain a new event loop.'''
     loop = asyncio.new_event_loop()
@@ -186,5 +141,9 @@ def thread_data(name, value=NOTHING, ct=None):
     return loc.get(name)
 
 
-get_actor = lambda: thread_data('actor')
-set_actor = lambda actor: thread_data('actor', actor)
+def get_actor():
+    return thread_data('actor')
+
+
+def set_actor(actor):
+    return thread_data('actor', actor)
