@@ -1,22 +1,18 @@
 import sys
 import os
-from time import time
+import pickle
 from itertools import chain
 from threading import current_thread
 
 from pulsar import HaltServer, CommandError, MonitorStarted, system
 from pulsar.utils.log import WritelnDecorator
-from pulsar.utils.pep import pickle
-from pulsar.utils.security import gen_unique_id
 
-from .futures import task
 from .events import EventHandler
-from .threads import get_executor
 from .proxy import ActorProxy, ActorProxyMonitor, actor_identity
 from .mailbox import command_in_context
-from .access import get_actor, set_actor
+from .access import get_actor
 from .cov import Coverage
-from .consts import *
+from .consts import *   # noqa
 
 
 __all__ = ['is_actor', 'send', 'spawn',
@@ -27,10 +23,6 @@ def is_actor(obj):
     return isinstance(obj, Actor)
 
 
-def create_aid():
-    return gen_unique_id()[:8]
-
-
 def get_stream(cfg):
     '''Obtain the python stream handler given a config dictionary.
     '''
@@ -69,7 +61,7 @@ def send(target, action, *args, **params):
 
 
 def spawn(**kwargs):
-    '''Spawn a new :class:`.Actor` and return an :class:`.ActorProxyFuture`.
+    '''Spawn a new :class:`.Actor` and return an :class:`~asyncio.Future`.
 
     **Parameter kwargs**
 
@@ -80,7 +72,7 @@ def spawn(**kwargs):
     * :ref:`actor hooks <actor-hooks>` such as ``start``, ``stopping``
       and ``periodic_task``
 
-    :return: an :class:`.ActorProxyFuture`.
+    :return: an :class:`~asyncio.Future`.
 
     A typical usage::
 
@@ -213,7 +205,7 @@ class Actor(EventHandler, Coverage):
         for name, value in impl.params.items():
             setattr(self, name, value)
         del impl.params
-        super(Actor, self).__init__(impl.setup_event_loop(self))
+        super().__init__(impl.setup_event_loop(self))
         for name, hook in hooks:
             self.bind_event(name, hook)
         try:
@@ -236,10 +228,6 @@ class Actor(EventHandler, Coverage):
     def aid(self):
         return self.__impl.aid
 
-    @property
-    def identity(self):
-        return self.__impl.identity
-
     @property
     def impl(self):
         return self.__impl
@@ -280,13 +268,6 @@ class Actor(EventHandler, Coverage):
         '''Dictionary of registered actors or None'''
         return self.__impl.registered
 
-    def executor(self):
-        '''An executor for this actor
-
-        Obtained from the :attr:`_loop` attribute
-        '''
-        return get_executor(self._loop)
-
     #######################################################################
     #    HIGH LEVEL API METHODS
     #######################################################################
@@ -303,11 +284,11 @@ class Actor(EventHandler, Coverage):
             self.state = ACTOR_STATES.STARTING
             self._run()
 
-    @task
     def send(self, target, action, *args, **kwargs):
         '''Send a message to ``target`` to perform ``action`` with given
         positional ``args`` and key-valued ``kwargs``.
-        Always return a :class:`~asyncio.Future`.'''
+        Returns a coroutine or a Future.
+        '''
         target = self.monitor if target == 'monitor' else target
         mailbox = self.mailbox
         if isinstance(target, ActorProxyMonitor):
@@ -427,7 +408,7 @@ class Actor(EventHandler, Coverage):
         actor = {'name': self.name,
                  'state': self.info_state,
                  'actor_id': self.aid,
-                 'uptime': time() - self._started,
+                 'uptime':  self._loop.time() - self._started,
                  'thread_id': self.tid,
                  'process_id': self.pid,
                  'is_process': isp,
