@@ -1,15 +1,20 @@
+import logging
 from functools import reduce
 
 from pulsar.utils.internet import is_socket_closed
 
-from .access import asyncio
-from .futures import coroutine_return, AsyncObject, future_timeout, task
+import asyncio
+
+from .futures import AsyncObject
 from .protocols import Producer
 
 
 __all__ = ['Pool', 'PoolConnection', 'AbstractClient', 'AbstractUdpClient']
 
 
+logger = logging.getLogger('pulsar.pool')
+
+
 class Pool(AsyncObject):
     '''An asynchronous pool of open connections.
 
@@ -19,12 +24,27 @@ class Pool(AsyncObject):
     This class is not thread safe.
     '''
     def __init__(self, creator, pool_size=10, loop=None, timeout=None, **kw):
+        '''
+        Construct an asynchronous Pool.
+
+        :param creator: a callable function that returns a connection object.
+
+        :param pool_size: The size of the pool to be maintained,
+          defaults to 10. This is the largest number of connections that
+          will be kept persistently in the pool. Note that the pool
+          begins with no connections; once this number of connections
+          is requested, that number of connections will remain.
+
+        :param timeout: The number of seconds to wait before giving up
+          on returning a connection. Defaults to 30.
+        '''
         self._creator = creator
         self._closed = False
         self._timeout = timeout
         self._queue = asyncio.Queue(maxsize=pool_size, loop=loop)
         self._connecting = 0
         self._loop = self._queue._loop
+        self._logger = logger
         self._in_use_connections = set()
 
     @property
@@ -57,7 +77,6 @@ class Pool(AsyncObject):
             return connection in self._queue._queue
         return True
 
-    @task
     def connect(self):
         '''Get a connection from the pool.
 
@@ -67,23 +86,24 @@ class Pool(AsyncObject):
         :return: a :class:`~asyncio.Future` resulting in the connection.
         '''
         assert not self._closed
-        connection = yield self._get()
-        coroutine_return(PoolConnection(self, connection))
+        connection = yield from self._get()
+        return PoolConnection(self, connection)
 
-    def close(self):
-        '''Close all :attr:`available` and :attr:`in_use` connections.
+    def close(self, async=True):
+        '''Close all :attr:`available` connections
         '''
-        self._closed = True
-        queue = self._queue
-        while queue.qsize():
-            connection = queue.get_nowait()
-            connection.close()
-        in_use = self._in_use_connections
-        self._in_use_connections = set()
-        for connection in in_use:
-            connection.close()
+        if not self._closed:
+            self._closed = True
+            queue = self._queue
+            while queue.qsize():
+                connection = queue.get_nowait()
+                if connection:
+                    connection.close()
+            in_use = self._in_use_connections
+            self._in_use_connections = set()
+            for connection in in_use:
+                connection.close()
 
-    @task
     def _get(self):
         queue = self._queue
         # grab the connection without waiting, important!
@@ -91,30 +111,35 @@ class Pool(AsyncObject):
             connection = queue.get_nowait()
         # wait for one to be available
         elif self.in_use + self._connecting >= queue._maxsize:
-            connection = yield future_timeout(queue.get(), self._timeout)
+            connection = yield from asyncio.wait_for(queue.get(),
+                                                     self._timeout,
+                                                     loop=self._loop)
         else:   # must create a new connection
             self._connecting += 1
             try:
-                connection = yield self._creator()
+                connection = yield from self._creator()
             finally:
                 self._connecting -= 1
         # None signal that a connection was removed form the queue
         # Go again
         if connection is None:
-            connection = yield self._get()
+            connection = yield from self._get()
         else:
             if self.is_connection_closed(connection):
-                connection = yield self._get()
+                connection = yield from self._get()
             else:
                 self._in_use_connections.add(connection)
-        coroutine_return(connection)
+        return connection
 
     def _put(self, conn, discard=False):
         if not self._closed:
             try:
+                # None signal that a connection was removed form the queue
                 self._queue.put_nowait(None if discard else conn)
             except asyncio.QueueFull:
-                conn.close()
+                # The queue of available connection is already full
+                if conn:
+                    conn.close()
         self._in_use_connections.discard(conn)
 
     def is_connection_closed(self, connection):
@@ -123,14 +148,10 @@ class Pool(AsyncObject):
             return True
         return False
 
-    def info(self, message=None, level=None):   # pragma    nocover
-        if self._queue._maxsize != 2:
-            return
-        message = '%s: ' % message if message else ''
-        self.logger.log(level or 10,
-                        '%smax size %s, in_use %s, available %s',
-                        message, self._queue._maxsize, self.in_use,
-                        self.available)
+    def status(self, message=None, level=None):
+        return ('Pool size: %d  Connections in pool: %d '
+                'Current Checked out connections: %d' %
+                (self._queue._maxsize, self.available, self.in_use))
 
     def _count_connections(self, x, y):
         return x + int(y is not None)
@@ -160,13 +181,14 @@ class PoolConnection(object):
         if self.pool is not None:
             self.pool._put(self.connection, discard)
             self.pool = None
-            self.connection = None
+            conn, self.connection = self.connection, None
+            return conn
 
     def detach(self):
         '''Remove the underlying :attr:`connection` from the connection
         :attr:`pool`.
         '''
-        self.close(True)
+        return self.close(True)
 
     def __enter__(self):
         return self
@@ -201,19 +223,18 @@ class AbstractClient(Producer):
         return self.fire_event('finish')
     abort = close
 
-    @task
     def create_connection(self, address, protocol_factory=None, **kw):
         '''Helper method for creating a connection to an ``address``.
         '''
         protocol_factory = protocol_factory or self.create_protocol
         if isinstance(address, tuple):
             host, port = address
-            _, protocol = yield self._loop.create_connection(
+            _, protocol = yield from self._loop.create_connection(
                 protocol_factory, host, port, **kw)
         else:
             raise NotImplementedError('Could not connect to %s' %
                                       str(address))
-        coroutine_return(protocol)
+        return protocol
 
 
 class AbstractUdpClient(Producer):
@@ -236,12 +257,11 @@ class AbstractUdpClient(Producer):
         return self.fire_event('finish')
     abort = close
 
-    @task
     def create_datagram_endpoint(self, protocol_factory=None, **kw):
         '''Helper method for creating a connection to an ``address``.
         '''
         protocol_factory = protocol_factory or self.create_protocol
-        _, protocol = yield self._loop.create_datagram_endpoint(
+        _, protocol = yield from self._loop.create_datagram_endpoint(
             protocol_factory, **kw)
-        yield protocol.event('connection_made')
-        coroutine_return(protocol)
+        yield from protocol.event('connection_made')
+        return protocol
