@@ -3,23 +3,24 @@ import sys
 from time import time
 from collections import OrderedDict
 from multiprocessing import Process, current_process
+from concurrent.futures import ThreadPoolExecutor
+
+import asyncio
 
 import pulsar
 from pulsar import system, MonitorStarted, HaltServer, Config
-from pulsar.utils.pep import itervalues, iteritems, range
 from pulsar.utils.log import logger_fds
 from pulsar.utils import autoreload
 from pulsar.utils.tools import Pidfile
 
 from .proxy import ActorProxyMonitor, get_proxy, actor_proxy_future
-from .access import get_actor, set_actor, logger, _StopError, SELECTORS
+from .access import get_actor, set_actor, logger, SELECTORS
 from .threads import Thread
-from .mailbox import MailboxClient, MailboxProtocol, ProxyMailbox
-from .futures import async, add_errback, chain_future, Future, From
-from .eventloop import EventLoop
+from .mailbox import MailboxClient, MailboxProtocol, ProxyMailbox, create_aid
+from .futures import async, add_errback, chain_future, Future
 from .protocols import TcpServer
-from .actor import Actor, create_aid
-from .consts import *
+from .actor import Actor
+from .consts import *   # noqa
 
 
 if sys.platform == 'win32':     # pragma    nocover
@@ -77,9 +78,11 @@ class Concurrency(object):
         self.kind = kind
         self.cfg = cfg
         self.params = kw
-        self.identity = aid
         return self.create_actor()
 
+    def identity(self, actor):
+        return actor.aid
+
     @property
     def unique_name(self):
         return '%s(%s)' % (self.name, self.aid)
@@ -125,7 +128,8 @@ class Concurrency(object):
         '''Start running the ``actor``.
         '''
         set_actor(actor)
-        actor.mailbox.start_serving()
+        if not actor.mailbox.address:
+            actor.mailbox.start_serving()
         actor._loop.run_forever()
 
     def add_monitor(self, actor, monitor_name, **params):
@@ -134,9 +138,14 @@ class Concurrency(object):
     def setup_event_loop(self, actor):
         '''Set up the event loop for ``actor``.
         '''
-        actor._logger = self.cfg.configured_logger(actor.name)
-        loop = EventLoop(self.selector(), logger=actor._logger,
-                         iothreadloop=True, cfg=actor.cfg)
+        actor._logger = self.cfg.configured_logger('pulsar.%s' % actor.name)
+        loop = asyncio.SelectorEventLoop(self.selector())
+        if self.cfg.debug:
+            loop.set_debug(True)
+        executor = ThreadPoolExecutor(self.cfg.thread_workers)
+        loop.set_default_executor(executor)
+        loop.logger = actor._logger
+        asyncio.set_event_loop(loop)
         actor.mailbox = self.create_mailbox(actor, loop)
         return loop
 
@@ -170,8 +179,6 @@ class Concurrency(object):
     def create_actor(self):
         self.daemon = False
         self.params['monitor'] = get_proxy(self.params['monitor'])
-        # make sure these parameters are picklable
-        # pickle.dumps(self.params)
         return ActorProxyMonitor(self)
 
     def create_mailbox(self, actor, loop):
@@ -205,7 +212,7 @@ class Concurrency(object):
             min(next, MAX_NOTIFY), self.periodic_task, actor)
         return ack
 
-    def stop(self, actor, exc=None, exit_code=0):
+    def stop(self, actor, exc=None, exit_code=None):
         '''Gracefully stop the ``actor``.
         '''
         if actor.state <= ACTOR_STATES.RUN:
@@ -225,13 +232,18 @@ class Concurrency(object):
                     actor.logger.error(str(exc))
                 elif exit_code:
                     actor.stream.writeln(str(exc))
+            else:
+                if not exit_code:
+                    exit_code = getattr(actor._loop, 'exit_code', 0)
             #
             actor.exit_code = exit_code
             stopping = actor.fire_event('stopping')
-            self.close_executor(actor)
             if not stopping.done() and actor._loop.is_running():
                 actor.logger.debug('asynchronous stopping')
-                cbk = lambda _: self._stop_actor(actor)
+
+                def cbk(_):
+                    return self._stop_actor(actor)
+
                 return chain_future(stopping, callback=cbk, errback=cbk)
             else:
                 if actor.logger:
@@ -240,16 +252,11 @@ class Concurrency(object):
         elif actor.stopped():
             return self._stop_actor(actor, True)
 
-    def close_executor(self, actor):
-        '''Close the :meth:`executor`'''
-        executor = actor._loop._default_executor
-        if executor:
-            actor.logger.debug('Waiting for executor shutdown')
-            executor.shutdown()
-            actor._loop._default_executor = None
-
     def _install_signals(self, actor):
-        proc_name = "%s-%s" % (actor.cfg.proc_name, actor.name)
+        proc_name = actor.cfg.proc_name
+        if not self.is_arbiter():
+            name = actor.name.split('.')[0]
+            proc_name = "%s-%s" % (proc_name, name)
         if system.set_proctitle(proc_name):
             actor.logger.debug('Set process title to %s',
                                system.get_proctitle())
@@ -263,12 +270,26 @@ class Concurrency(object):
                 except ValueError:
                     pass
 
+    def _remove_signals(self, actor):
+        if signal and actor.is_process():
+            actor.logger.debug('Remove signal handlers')
+            for sig in system.EXIT_SIGNALS:
+                try:
+                    actor._loop.remove_signal_handler(sig)
+                except Exception:
+                    pass
+        if actor._loop.is_running():  # pragma nocover
+            actor.logger.critical('Event loop still running when stopping')
+            actor._loop.stop()
+        else:
+            actor.logger.debug('Close event loop')
+            actor._loop.close()
+
     def _stop_actor(self, actor, finished=False):
-        '''Exit from the :class:`.Actor` domain.'''
+        # Stop the actor if finished is True
+        # otherwise starts the stopping process
         if finished:
-            if actor._loop.is_running():
-                actor.logger.critical('Event loop still running when stopping')
-                actor._loop.stop()
+            self._remove_signals(actor)
             return True
         #
         actor.state = ACTOR_STATES.CLOSE
@@ -308,12 +329,14 @@ class ProcessMixin(object):
 
     def handle_exit_signal(self, actor, sig):
         actor.logger.warning("Got %s. Stopping.", system.SIG_NAMES.get(sig))
-        actor._loop.exit_signal = sig
-        raise _StopError
+        self.stop(actor, exit_code=int(sig))
 
 
 class MonitorMixin(object):
 
+    def identity(self, actor):
+        return actor.name
+
     def create_actor(self):
         self.managed_actors = {}
         self.terminated_actors = []
@@ -336,10 +359,6 @@ class MonitorMixin(object):
         elif actor.monitor and check_monitor:
             return actor.monitor.get_actor(aid)
 
-    @property
-    def pid(self):
-        return current_process().pid
-
     def spawn(self, monitor, kind=None, **params):
         '''Spawn a new :class:`Actor` and return its
         :class:`.ActorProxyMonitor`.
@@ -365,7 +384,7 @@ class MonitorMixin(object):
         '''
         alive = 0
         if self.managed_actors:
-            for aid, actor in list(iteritems(self.managed_actors)):
+            for aid, actor in list(self.managed_actors.items()):
                 alive += self.manage_actor(monitor, actor, stop)
         return alive
 
@@ -429,7 +448,7 @@ class MonitorMixin(object):
             num_to_kill = len(self.managed_actors) - monitor.cfg.workers
             for i in range(num_to_kill, 0, -1):
                 w, kage = 0, sys.maxsize
-                for worker in itervalues(self.managed_actors):
+                for worker in self.managed_actors.values():
                     age = worker.impl.age
                     if age < kage:
                         w, kage = w, age
@@ -463,7 +482,7 @@ class MonitorMixin(object):
         if actor.started():
             info['actor'].update({'concurrency': actor.cfg.concurrency,
                                   'workers': len(self.managed_actors)})
-            info['workers'] = [a.info for a in itervalues(self.managed_actors)
+            info['workers'] = [a.info for a in self.managed_actors.values()
                                if a.info]
         return info
 
@@ -483,7 +502,7 @@ class MonitorConcurrency(MonitorMixin, Concurrency):
         return True
 
     def setup_event_loop(self, actor):
-        actor._logger = self.cfg.configured_logger(actor.name)
+        actor._logger = self.cfg.configured_logger('pulsar.%s' % actor.name)
         actor.mailbox = ProxyMailbox(actor)
         loop = actor.mailbox._loop
         loop.call_soon(actor.start)
@@ -496,9 +515,6 @@ class MonitorConcurrency(MonitorMixin, Concurrency):
     def create_mailbox(self, actor, loop):
         raise NotImplementedError
 
-    def close_executor(self, actor):
-        pass
-
     def periodic_task(self, monitor, **kw):
         '''Override the :meth:`.Concurrency.periodic_task` to implement
         the :class:`.Monitor` :ref:`periodic task <actor-periodic-task>`.'''
@@ -521,6 +537,7 @@ class MonitorConcurrency(MonitorMixin, Concurrency):
                 interval, self.periodic_task, monitor)
 
     def _stop_actor(self, actor, finished=False):
+        # remove all workers from this monitor
         if finished:
             return
 
@@ -545,20 +562,21 @@ class ArbiterConcurrency(MonitorMixin, ProcessMixin, Concurrency):
         return True
 
     def create_actor(self):
-        if self.cfg.daemon:
+        if self.cfg.daemon:     # pragma    nocover
+            # Daemonize the system
             if not self.cfg.pidfile:
                 self.cfg.set('pidfile', 'pulsar.pid')
             system.daemonize(keep_fds=logger_fds())
-        self.identity = self.name
-        actor = super(ArbiterConcurrency, self).create_actor()
+        self.aid = self.name
+        actor = super().create_actor()
         self.monitors = OrderedDict()
-        self.registered = {'arbiter': actor}
+        self.registered = {self.identity(actor): actor}
         actor.bind_event('start', self._start_arbiter)
         return actor
 
     def get_actor(self, actor, aid, check_monitor=True):
         '''Given an actor unique id return the actor proxy.'''
-        a = super(ArbiterConcurrency, self).get_actor(actor, aid)
+        a = super().get_actor(actor, aid)
         if a is None:
             if aid in self.monitors:  # Check in monitors aid
                 return self.monitors[aid]
@@ -567,7 +585,7 @@ class ArbiterConcurrency(MonitorMixin, ProcessMixin, Concurrency):
             elif aid in self.registered:
                 return self.registered[aid]
             else:  # Finally check in workers in monitors
-                for m in itervalues(self.monitors):
+                for m in self.monitors.values():
                     a = m.get_actor(aid, check_monitor=False)
                     if a is not None:
                         return a
@@ -622,7 +640,7 @@ class ArbiterConcurrency(MonitorMixin, ProcessMixin, Concurrency):
         if actor.started():
             # managed actors job
             self.manage_actors(actor)
-            for m in list(itervalues(self.monitors)):
+            for m in list(self.monitors.values()):
                 if m.closed():
                     actor._remove_actor(m)
 
@@ -640,8 +658,7 @@ class ArbiterConcurrency(MonitorMixin, ProcessMixin, Concurrency):
             actor.stop(exit_code=autoreload.EXIT_CODE)
 
     def _stop_actor(self, actor, finished=False):
-        '''Stop the pools the message queue and remaining actors
-        '''
+        # remove all actors and monitors
         if finished:
             self._stop_arbiter(actor)
         elif actor._loop.is_running():
@@ -669,24 +686,26 @@ class ArbiterConcurrency(MonitorMixin, ProcessMixin, Concurrency):
     def _close_all(self, actor):
         # Close al monitors at once
         try:
-            for m in itervalues(self.monitors):
-                yield From(m.stop())
-            yield From(self._close_actors(actor))
+            for m in self.monitors.values():
+                stop = m.stop()
+                if stop:
+                    yield from stop
+            yield from self._close_actors(actor)
         except Exception:
             actor.logger.exception('Exception while closing arbiter')
         self._exit_arbiter(actor, True)
 
     def _remove_actor(self, arbiter, actor, log=True):
-        a = super(ArbiterConcurrency, self)._remove_actor(arbiter, actor,
-                                                          False)
-        b = self.registered.pop(actor.name, None)
-        c = self.monitors.pop(actor.aid, None)
+        a = super()._remove_actor(arbiter, actor, False)
+        b = self.registered.pop(self.identity(actor), None)
+        c = self.monitors.pop(self.identity(actor), None)
         removed = a or b or c
         if removed and log:
             arbiter.logger.warning('Removed %s', actor)
         return removed
 
     def _stop_arbiter(self, actor):     # pragma    nocover
+        self._remove_signals(actor)
         p = self.pidfile
         if p is not None:
             actor.logger.debug('Removing %s' % p.fname)
@@ -697,8 +716,9 @@ class ArbiterConcurrency(MonitorMixin, ProcessMixin, Concurrency):
         actor.collect_coverage()
         exit_code = actor.exit_code or 0
         if exit_code == autoreload.EXIT_CODE:
-            actor.stream.writeln("\nCode changed, reloading server")
+            actor.logger.info("Code changed, reloading server")
         else:
+            # actor.logger.info("Bye (exit code = %s)", exit_code)
             actor.stream.writeln("\nBye (exit code = %s)" % exit_code)
         try:
             actor.cfg.when_exit(actor)
@@ -725,7 +745,7 @@ class ArbiterConcurrency(MonitorMixin, ProcessMixin, Concurrency):
     def _info_monitor(self, actor, info=None):
         data = info
         monitors = {}
-        for m in itervalues(self.monitors):
+        for m in self.monitors.values():
             info = m.info()
             if info:
                 actor = info['actor']
@@ -740,13 +760,14 @@ class ArbiterConcurrency(MonitorMixin, ProcessMixin, Concurrency):
         server.pop('actor_id', None)
         server.pop('age', None)
         data['server'] = server
-        data['workers'] = [a.info for a in itervalues(self.managed_actors)]
+        data['workers'] = [a.info for a in self.managed_actors.values()]
         data['monitors'] = monitors
         return data
 
     def _register(self, actor):
-        self.registered[actor.name] = actor
-        self.monitors[actor.aid] = actor
+        aid = self.identity(actor)
+        self.registered[aid] = actor
+        self.monitors[aid] = actor
 
 
 def run_actor(self):
@@ -825,7 +846,7 @@ def _spawn_actor(kind, monitor, cfg=None, name=None, aid=None, **kw):
         if not cfg.exc_id:
             cfg.set('exc_id', aid)
     #
-    for key, value in iteritems(kw):
+    for key, value in kw.items():
         if key in cfg.settings:
             cfg.set(key, value)
         else:
