@@ -1,94 +1,31 @@
 from collections import Mapping
-from inspect import isgeneratorfunction, isgenerator
+from inspect import isgeneratorfunction
 from functools import wraps, partial
 
-from pulsar.utils.pep import iteritems, range
-
+from asyncio import Future, CancelledError, TimeoutError, async, sleep
 from .consts import MAX_ASYNC_WHILE
-from .access import (trollius, get_event_loop, async, Return, Future, sleep,
-                     From, isfuture, LOGGER, _PENDING, _CANCELLED, _FINISHED,
-                     _EVENT_LOOP_CLASSES)
-
-coroutine = trollius.coroutine
-iscoroutine = trollius.iscoroutine
-CancelledError = trollius.CancelledError
-TimeoutError = trollius.TimeoutError
-InvalidStateError = trollius.InvalidStateError
+from .access import get_event_loop, LOGGER, isfuture, is_async
 
 
-__all__ = ['CancelledError',
-           'TimeoutError',
-           'InvalidStateError',
-           'coroutine_return',
-           'maybe_async',
+__all__ = ['maybe_async',
            'run_in_loop',
-           'async',
            'add_errback',
            'add_callback',
-           'future_timeout',
            'task_callback',
            'multi_async',
-           'async_while',
+           'as_coroutine',
            'task',
+           'async',
+           'async_while',
            'chain_future',
            'future_result_exc',
-           'AsyncObject',
-           'yield_from']
+           'AsyncObject']
 
 
-def coroutine_return(*value):
-    raise Return(*value)
+return_false = lambda: False
 
 
-def yield_from(coro, timeout=None, loop=None):
-    '''Wraps a ``coroutine`` by yielding values wrapped with
-    the ``From`` function.
-    '''
-    value = None
-    exc = None
-    while True:
-        if exc is not None:
-            result = coro.throw(exc)
-        elif value is not None:
-            result = coro.send(value)
-        else:
-            result = next(coro)
-        exc = None
-        try:
-            if result is not None:
-                result = async(result, loop=loop)
-                future_timeout(result, timeout)
-        except TypeError:
-            value = result
-        else:
-            try:
-                value = yield From(result)
-            except Exception as e:
-                exc = e
-
-
-def future_timeout(future, timeout=None, exc_class=None):
-    '''Add a ``timeout`` to ``future`` in a thread-safe way.
-
-    :return: the ``future``
-    '''
-    exc_class = exc_class or TimeoutError
-
-    def _check_timeout():
-        if not future.done():
-            future.set_exception(exc_class())
-
-    if timeout and timeout > 0 and not future.done():
-        if future._loop == get_event_loop():
-            future._loop.call_later(timeout, _check_timeout)
-        else:
-            future._loop.call_soon_threadsafe(future._loop.call_later,
-                                              timeout, _check_timeout)
-
-    return future
-
-
-def chain_future(future, callback=None, errback=None, next=None, timeout=None):
+def chain_future(future, callback=None, errback=None, next=None):
     '''Chain a :class:`~asyncio.Future` to an existing ``future``.
 
     This function `chain` the ``next`` future to an existing ``future``.
@@ -101,16 +38,12 @@ def chain_future(future, callback=None, errback=None, next=None, timeout=None):
     :param errback: optional callback to execute on the exception of ``future``
     :param next: optional :class:`~asyncio.Future` to chain.
         If not provided a new future is created
-    :param timeout: optional timeout to set on ``next``
     :return: the future ``next``
     '''
-    future = async(future)
+    loop = next._loop if next else None
+    future = async(future, loop=loop)
     if next is None:
         next = Future(loop=future._loop)
-        if timeout and timeout > 0:
-            future_timeout(next, timeout)
-    else:
-        assert timeout is None, 'cannot set timeout'
 
     def _callback(fut):
         try:
@@ -128,7 +61,7 @@ def chain_future(future, callback=None, errback=None, next=None, timeout=None):
         except Exception as exc:
             next.set_exception(exc)
         else:
-            if isfuture(result):
+            if isinstance(result, Future):
                 chain_future(result, next=next)
             else:
                 next.set_result(result)
@@ -137,10 +70,10 @@ def chain_future(future, callback=None, errback=None, next=None, timeout=None):
     return next
 
 
-def as_exception(fut):
-    if fut._exception:
-        return fut.exception()
-    elif getattr(fut, '_state', None) == _CANCELLED:
+def as_exception(future):
+    if future._exception:
+        return future.exception()
+    elif future.cancelled():
         return CancelledError()
 
 
@@ -175,7 +108,7 @@ def future_result_exc(future):
 
     The :class:`.Future` must be ``done``
     '''
-    if future._state == _CANCELLED:
+    if future.cancelled():
         return None, CancelledError()
     elif future._exception:
         return None, future.exception()
@@ -209,6 +142,12 @@ def maybe_async(value, loop=None):
         return value
 
 
+def as_coroutine(value):
+    if is_async(value):
+        value = yield from value
+    return value
+
+
 def task(function):
     '''Thread-safe decorator to run a ``function`` in an event loop.
 
@@ -217,31 +156,21 @@ def task(function):
         an :ref:`async object <async-object>`, in which case the loop
         is given by the object ``_loop`` attribute.
     :return: a :class:`~asyncio.Future`
-
-    The coroutine is wrapped with the :func:`yield_from` function.
     '''
     if isgeneratorfunction(function):
         wrapper = function
     else:
         def wrapper(*args, **kw):
-            res = yield function(*args, **kw)
-            raise Return(res)
+            res = function(*args, **kw)
+            if is_async(res):
+                res = yield from res
+            return res
 
     @wraps(function)
     def _(*args, **kwargs):
-        event_loop = get_event_loop()
+        loop = getattr(args[0], '_loop', None) if args else None
         coro = wrapper(*args, **kwargs)
-        if args:
-            loop = getattr(args[0], '_loop', event_loop)
-            # Not in the current event loop
-            if loop != event_loop and isinstance(loop, _EVENT_LOOP_CLASSES):
-                future = run_in_loop(loop, yield_from, coro, loop=loop)
-                if (not getattr(loop, '_iothreadloop', True)
-                        and not loop.is_running()):
-                    return loop.run_until_complete(future)
-                else:
-                    return future
-        return async(yield_from(coro))
+        return async(coro, loop=loop)
 
     return _
 
@@ -284,24 +213,22 @@ def async_while(timeout, while_clause, *args):
     :return: A :class:`.Future`.
     '''
     loop = get_event_loop()
+    start = loop.time()
+    di = 0.1
+    interval = 0
+    result = while_clause(*args)
 
-    def _():
-        start = loop.time()
-        di = 0.1
-        interval = 0
+    while result:
+        interval = min(interval+di, MAX_ASYNC_WHILE)
+        try:
+            yield from sleep(interval, loop=loop)
+        except TimeoutError:
+            pass
+        if timeout and loop.time() - start >= timeout:
+            break
         result = while_clause(*args)
-        while result:
-            interval = min(interval+di, MAX_ASYNC_WHILE)
-            try:
-                yield From(sleep(interval, loop=loop))
-            except TimeoutError:
-                pass
-            if timeout and loop.time() - start >= timeout:
-                break
-            result = while_clause(*args)
-        coroutine_return(result)
 
-    return async(_(), loop)
+    return result
 
 
 # ############################################################## Bench
@@ -361,6 +288,12 @@ class AsyncObject(object):
         '''
         return self._logger or getattr(self._loop, 'logger', LOGGER)
 
+    @property
+    def debug(self):
+        '''True when in debug mode
+        '''
+        return getattr(self._loop, 'get_debug', return_false)()
+
     def timeit(self, method, times, *args, **kwargs):
         '''Useful utility for benchmarking an asynchronous ``method``.
 
@@ -379,36 +312,19 @@ class AsyncObject(object):
         return bench(getattr(self, method), *args, **kwargs)
 
 
-# ############################################################## Task
-class Task(trollius.Task):
-
-    def _wakeup(self, future, inthread=False):
-        if inthread or future._loop is self._loop:
-            try:
-                value = future.result()
-            except Exception as exc:
-                # This may also be a cancellation.
-                self._step(None, exc)
-            else:
-                self._step(value, None)
-        else:
-            self._loop.call_soon_threadsafe(self._wakeup, future, True)
-        self = None
-
-
 # ############################################################## MultiFuture
 class MultiFuture(Future):
     '''Handle several futures at once. Thread safe.
     '''
     def __init__(self, data=None, loop=None, type=None, raise_on_error=True):
-        super(MultiFuture, self).__init__(loop=loop)
+        super().__init__(loop=loop)
         self._futures = {}
         self._failures = []
         self._raise_on_error = raise_on_error
         if data is not None:
             type = type or data.__class__
             if issubclass(type, Mapping):
-                data = iteritems(data)
+                data = data.items()
             else:
                 type = list
                 data = enumerate(data)
@@ -439,7 +355,7 @@ class MultiFuture(Future):
         # thread safe
         if inthread or future._loop is self._loop:
             self._futures.pop(key, None)
-            if self._state == _PENDING:
+            if not self.done():
                 self._get_set_item(key, future)
                 self._check()
         else:
