@@ -15,7 +15,7 @@ The implementation details are outlined below:
       from pulsar import send
 
       def example():
-          result = yield send('abc', 'ping')
+          result = yield from send('abc', 'ping')
 
 * The :class:`.Arbiter` :attr:`~pulsar.Actor.mailbox` is a :class:`.TcpServer`
   accepting connections from remote actors.
@@ -51,16 +51,16 @@ Client
 
 '''
 import socket
+import pickle
 from collections import namedtuple
 
 from pulsar import ProtocolError, CommandError
 from pulsar.utils.internet import nice_address
 from pulsar.utils.websocket import frame_parser
-from pulsar.utils.security import gen_unique_id
-from pulsar.utils.pep import pickle
+from pulsar.utils.string import gen_unique_id
 
-from .access import get_actor
-from .futures import Future, coroutine_return, task
+from .access import get_actor, is_async
+from .futures import Future, task
 from .proxy import actor_identity, get_proxy, get_command, ActorProxy
 from .protocols import Protocol
 from .clients import AbstractClient
@@ -69,12 +69,19 @@ from .clients import AbstractClient
 CommandRequest = namedtuple('CommandRequest', 'actor caller connection')
 
 
-def command_in_context(command, caller, actor, args, kwargs):
+def create_aid():
+    return gen_unique_id()[:8]
+
+
+def command_in_context(command, caller, target, args, kwargs, connection=None):
     cmnd = get_command(command)
     if not cmnd:
         raise CommandError('unknown %s' % command)
-    request = CommandRequest(actor, caller, None)
-    return cmnd(request, args, kwargs)
+    request = CommandRequest(target, caller, connection)
+    result = cmnd(request, args, kwargs)
+    if is_async(result):
+        result = yield from result
+    return result
 
 
 class ProxyMailbox(object):
@@ -107,9 +114,9 @@ class ProxyMailbox(object):
 class Message(object):
     '''A message which travels from actor to actor.
     '''
-    def __init__(self, data, future=None):
+    def __init__(self, data, waiter=None):
         self.data = data
-        self.future = future
+        self.waiter = waiter
 
     def __repr__(self):
         return self.data.get('command', 'unknown')
@@ -123,12 +130,12 @@ class Message(object):
                 'target': actor_identity(target),
                 'args': args if args is not None else (),
                 'kwargs': kwargs if kwargs is not None else {}}
+        waiter = Future()
         if command.ack:
-            future = Future()
-            data['ack'] = gen_unique_id()[:8]
+            data['ack'] = create_aid()
         else:
-            future = None
-        return cls(data, future)
+            waiter.set_result(None)
+        return cls(data, waiter)
 
     @classmethod
     def callback(cls, result, ack):
@@ -142,7 +149,7 @@ class MailboxProtocol(Protocol):
     Encoding and decoding uses the unmasked websocket protocol.
     '''
     def __init__(self, **kw):
-        super(MailboxProtocol, self).__init__(**kw)
+        super().__init__(**kw)
         self._pending_responses = {}
         self._parser = frame_parser(kind=2, pyparser=True)
         actor = get_actor()
@@ -153,7 +160,7 @@ class MailboxProtocol(Protocol):
         '''Used by the server to send messages to the client.'''
         req = Message.command(command, sender, target, args, kwargs)
         self._start(req)
-        return req.future
+        return req.waiter
 
     def data_received(self, data):
         # Feed data into the parser
@@ -169,12 +176,12 @@ class MailboxProtocol(Protocol):
     ########################################################################
     #    INTERNALS
     def _start(self, req):
-        if req.future and 'ack' in req.data:
-            self._pending_responses[req.data['ack']] = req.future
+        if req.waiter and 'ack' in req.data:
+            self._pending_responses[req.data['ack']] = req.waiter
             try:
                 self._write(req)
             except Exception as exc:
-                req.future.set_exception(exc)
+                req.waiter.set_exception(exc)
         else:
             self._write(req)
 
@@ -212,17 +219,15 @@ class MailboxProtocol(Protocol):
                         raise CommandError(
                             "'%s' got message from unknown '%s'"
                             % (actor, message['sender']))
-                    result = yield actor.send(target, command,
-                                              *message['args'],
-                                              **message['kwargs'])
+                    result = yield from actor.send(target, command,
+                                                   *message['args'],
+                                                   **message['kwargs'])
                 else:
-                    actor = target
-                    cmd = get_command(command)
-                    req = CommandRequest(target, caller, self)
-                    if actor.cfg.debug:
-                        actor.logger.debug('Executing command %s from %s',
-                                           command, caller or 'monitor')
-                    result = yield cmd(req, message['args'], message['kwargs'])
+                    result = yield from command_in_context(command, caller,
+                                                           target,
+                                                           message['args'],
+                                                           message['kwargs'],
+                                                           self)
             except CommandError as exc:
                 self.logger.warning('Command error: %s' % exc)
                 result = None
@@ -253,13 +258,13 @@ class MailboxClient(AbstractClient):
     protocol_factory = MailboxProtocol
 
     def __init__(self, address, actor, loop):
-        super(MailboxClient, self).__init__(loop)
+        super().__init__(loop)
         self.address = address
         self.name = 'Mailbox for %s' % actor
         self._connection = None
 
     def response(self, request):
-        resp = super(MailboxClient, self).response
+        resp = super().response
         self._consumer = resp(request, self._consumer, False)
         return self._consumer
 
@@ -273,12 +278,12 @@ class MailboxClient(AbstractClient):
     def request(self, command, sender, target, args, kwargs):
         # the request method
         if self._connection is None:
-            self._connection = yield self.connect()
+            self._connection = yield from self.connect()
             self._connection.bind_event('connection_lost', self._lost)
         req = Message.command(command, sender, target, args, kwargs)
         self._connection._start(req)
-        response = yield req.future
-        coroutine_return(response)
+        response = yield from req.waiter
+        return response
 
     def start_serving(self):
         pass
