@@ -1,12 +1,12 @@
 import sys
+import asyncio
 
 import pulsar
 from pulsar.utils.internet import nice_address, format_address
 
 from .futures import multi_async, task, Future
-from .events import EventHandler
+from .events import EventHandler, AbortEvent
 from .mixins import FlowControl, Timeout
-from .access import asyncio, get_io_loop, From, ConnectionResetError
 
 
 __all__ = ['ProtocolConsumer',
@@ -15,7 +15,12 @@ __all__ = ['ProtocolConsumer',
            'Connection',
            'Producer',
            'TcpServer',
-           'DatagramServer']
+           'DatagramServer',
+           'AbortRequest']
+
+
+class AbortRequest(AbortEvent):
+    pass
 
 
 class ProtocolConsumer(EventHandler):
@@ -93,7 +98,8 @@ class ProtocolConsumer(EventHandler):
 
     @property
     def on_finished(self):
-        '''The ``post_request`` one time event.
+        '''Event fired once a full response to a request is received. It is
+        the ``post_request`` one time event.
         '''
         return self.event('post_request')
 
@@ -140,7 +146,8 @@ class ProtocolConsumer(EventHandler):
         For server side consumer, this method simply fires the ``pre_request``
         event.'''
         if hasattr(self, '_request'):
-            raise RuntimeError('Consumer already started')
+            raise RuntimeError('%s already requested %s' %
+                               (self, self._request))
         conn = self._connection
         if not conn:
             raise RuntimeError('Cannot start new request. No connection.')
@@ -152,12 +159,30 @@ class ProtocolConsumer(EventHandler):
             conn._producer._requests_processed = p + 1
         self.bind_event('post_request', self._finished)
         self._request = request
-        self.fire_event('pre_request')
-        if self._request is not None:
-            try:
-                self.start_request()
-            except Exception as exc:
-                self.finished(exc=exc)
+        return asyncio.async(self._start(), loop=self._loop)
+
+    def abort_request(self):
+        '''Abort the request.
+
+        This method can be called during the pre-request stage
+        '''
+        future = self.events['pre_request']
+        if future.done():
+            raise RuntimeError('Request already sent')
+        future.add_done_callback(self._abort_request)
+        raise AbortRequest
+
+    def _start(self):
+        try:
+            yield from self.fire_event('pre_request')
+        except AbortEvent:
+            self.logger.debug('Abort request %s', self.request)
+        else:
+            if self._request is not None:
+                try:
+                    self.start_request()
+                except Exception as exc:
+                    self.finished(exc=exc)
 
     def connection_lost(self, exc):
         '''Called by the :attr:`connection` when the transport is closed.
@@ -204,6 +229,16 @@ class ProtocolConsumer(EventHandler):
         if c and c._current_consumer is self:
             c._current_consumer = None
 
+    @task
+    def _abort_request(self, fut):
+        exc = fut.exception()
+        for event in self.ONE_TIME_EVENTS:
+            if not self.event(event).fired():
+                try:
+                    yield from self.fire_event(event, exc=exc)
+                except AbortRequest:
+                    pass
+
 
 class PulsarProtocol(EventHandler, FlowControl):
     '''A mixin class for both :class:`.Protocol` and
@@ -221,19 +256,18 @@ class PulsarProtocol(EventHandler, FlowControl):
 
     _transport = None
     _address = None
-    _type = 'server'
 
-    def __init__(self, loop=None, session=1, producer=None, **kw):
-        super(PulsarProtocol, self).__init__(loop)
+    def __init__(self, loop, session=1, producer=None, logger=None, **kw):
+        super().__init__(loop)
         FlowControl.__init__(self, **kw)
+        self._logger = logger
         self._session = session
         self._producer = producer
 
     def __repr__(self):
         address = self._address
         if address:
-            return '%s %s session %s' % (self._type, nice_address(address),
-                                         self._session)
+            return '%s session %s' % (nice_address(address), self._session)
         else:
             return '<pending> session %s' % self._session
     __str__ = __repr__
@@ -278,12 +312,26 @@ class PulsarProtocol(EventHandler, FlowControl):
     @property
     def closed(self):
         '''``True`` if the :attr:`transport` is closed.'''
-        return self._transport._closing if self._transport else True
+        if self._transport:
+            if not getattr(self._transport, '_closing', False):
+                return False
+            return True
+        return True
 
     def close(self):
         '''Close by closing the :attr:`transport`.'''
         if self._transport:
-            self._transport.close()
+            if self.debug:
+                self.logger.debug('Closing connection %s', self)
+            if self._transport.can_write_eof():
+                try:
+                    self._transport.write_eof()
+                except Exception:
+                    pass
+            try:
+                self._transport.close()
+            except Exception:
+                pass
 
     def abort(self):
         '''Abort by aborting the :attr:`transport`.'''
@@ -297,7 +345,6 @@ class PulsarProtocol(EventHandler, FlowControl):
         self._transport = transport
         addr = self._transport.get_extra_info('peername')
         if not addr:
-            self._type = 'client'
             addr = self._transport.get_extra_info('sockname')
         self._address = addr
         # let everyone know we have a connection with endpoint
@@ -306,10 +353,13 @@ class PulsarProtocol(EventHandler, FlowControl):
     def connection_lost(self, exc=None):
         '''Fires the ``connection_lost`` event.
         '''
+        if self.debug and not exc:
+            self.logger.debug('Lost connection %s', self)
         self.fire_event('connection_lost')
 
     def eof_received(self):
-        '''The socket was closed from the remote end'''
+        '''The socket was closed from the remote end
+        '''
 
     def info(self):
         info = {'connection': {'session': self._session}}
@@ -331,8 +381,6 @@ class Protocol(PulsarProtocol, asyncio.Protocol):
         '''
         t = self._transport
         if t:
-            if t._closing:  # Uses private variable.
-                raise ConnectionResetError('Connection lost')
             if self._paused:
                 # # Uses private variable once again!
                 # This occurs when the protocol is paused from writing
@@ -372,30 +420,30 @@ class Connection(Protocol, Timeout):
     '''
     def __init__(self, consumer_factory=None, timeout=None,
                  low_limit=None, high_limit=None, **kw):
-        super(Connection, self).__init__(**kw)
+        super().__init__(**kw)
         self.bind_event('connection_lost', self._connection_lost)
         self._processed = 0
         self._current_consumer = None
         self._consumer_factory = consumer_factory
         self.timeout = timeout
 
+    @property
+    def requests_processed(self):
+        return self._processed
+
     def current_consumer(self):
         '''The :class:`ProtocolConsumer` currently handling incoming data.
 
         This instance will receive data when this connection get data
-        from the :attr:`~Protocol.transport` via the :meth:`data_received`
-        method.
+        from the :attr:`~PulsarProtocol.transport` via the
+        :meth:`data_received` method.
+
+        If no consumer is available, build a new one and return it.
         '''
         if self._current_consumer is None:
             self._build_consumer(None)
         return self._current_consumer
 
-    def set_consumer(self, consumer):
-        assert self._current_consumer is None, 'Consumer is not None'
-        self._current_consumer = consumer
-        consumer._connection = self
-        consumer.connection_made(self)
-
     def data_received(self, data):
         '''Delegates handling of data to the :meth:`current_consumer`.
 
@@ -404,12 +452,13 @@ class Connection(Protocol, Timeout):
         '''
         self._data_received_count = self._data_received_count + 1
         self.fire_event('data_received', data=data)
-        while data:
+        toprocess = data
+        while toprocess:
             consumer = self.current_consumer()
-            data = consumer._data_received(data)
-            if isinstance(data, Future):
+            toprocess = consumer._data_received(toprocess)
+            if isinstance(toprocess, Future):
                 break
-        self.fire_event('data_received', data=data)
+        self.fire_event('data_processed', data=data)
 
     def upgrade(self, consumer_factory):
         '''Upgrade the :func:`_consumer_factory` callable.
@@ -434,7 +483,7 @@ class Connection(Protocol, Timeout):
             self._build_consumer(None)
 
     def info(self):
-        info = super(Connection, self).info()
+        info = super().info()
         c = info['connection']
         c['request_processed'] = self._processed
         c['data_processed_count'] = self._data_received_count
@@ -442,9 +491,12 @@ class Connection(Protocol, Timeout):
         return info
 
     def _build_consumer(self, _, exc=None):
-        if not exc:
+        if not exc or isinstance(exc, AbortEvent):
             consumer = self._producer.build_consumer(self._consumer_factory)
-            self.set_consumer(consumer)
+            assert self._current_consumer is None, 'Consumer is not None'
+            self._current_consumer = consumer
+            consumer._connection = self
+            consumer.connection_made(self)
 
     def _connection_lost(self, _, exc=None):
         '''It performs these actions in the following order:
@@ -471,9 +523,9 @@ class Producer(EventHandler):
         protocol_factory(session, producer, **params)
     '''
 
-    def __init__(self, loop, protocol_factory=None, name=None,
+    def __init__(self, loop=None, protocol_factory=None, name=None,
                  max_requests=None, logger=None):
-        super(Producer, self).__init__(get_io_loop(loop))
+        super().__init__(loop or asyncio.get_event_loop())
         self.protocol_factory = protocol_factory or self.protocol_factory
         self._name = name or self.__class__.__name__
         self._requests_processed = 0
@@ -503,6 +555,7 @@ class Producer(EventHandler):
         kw['session'] = self.sessions
         kw['producer'] = self
         kw['loop'] = self._loop
+        kw['logger'] = self._logger
         return self.protocol_factory(**kw)
 
     def build_consumer(self, consumer_factory):
@@ -514,6 +567,7 @@ class Producer(EventHandler):
         :param consumer_factory: consumer factory to use.
         '''
         consumer = consumer_factory(loop=self._loop)
+        consumer._logger = self._logger
         consumer.copy_many_times_events(self)
         return consumer
 
@@ -536,9 +590,8 @@ class TcpServer(Producer):
     def __init__(self, protocol_factory, loop, address=None,
                  name=None, sockets=None, max_requests=None,
                  keep_alive=None, logger=None):
-        super(TcpServer, self).__init__(loop, protocol_factory, name=name,
-                                        max_requests=max_requests,
-                                        logger=logger)
+        super().__init__(loop, protocol_factory, name=name,
+                         max_requests=max_requests, logger=logger)
         self._params = {'address': address, 'sockets': sockets}
         self._keep_alive = max(keep_alive or 0, 0)
         self._concurrent_connections = set()
@@ -568,6 +621,7 @@ class TcpServer(Producer):
         :param sslcontext: optional SSLContext object.
         :return: a :class:`.Future` called back when the server is
             serving the socket.'''
+        assert not self._server
         if hasattr(self, '_params'):
             address = self._params['address']
             sockets = self._params['sockets']
@@ -577,21 +631,21 @@ class TcpServer(Producer):
                 if sockets:
                     server = None
                     for sock in sockets:
-                        srv = yield From(create_server(self.create_protocol,
+                        srv = yield from create_server(self.create_protocol,
                                                        sock=sock,
                                                        backlog=backlog,
-                                                       ssl=sslcontext))
+                                                       ssl=sslcontext)
                         if server:
                             server.sockets.extend(srv.sockets)
                         else:
                             server = srv
                 else:
                     if isinstance(address, tuple):
-                        server = yield From(create_server(self.create_protocol,
+                        server = yield from create_server(self.create_protocol,
                                                           host=address[0],
                                                           port=address[1],
                                                           backlog=backlog,
-                                                          ssl=sslcontext))
+                                                          ssl=sslcontext)
                     else:
                         raise NotImplementedError
                 self._server = server
@@ -600,29 +654,18 @@ class TcpServer(Producer):
                     address = sock.getsockname()
                     self.logger.info('%s serving on %s', self._name,
                                      format_address(address))
-                self.fire_event('start')
+                self._loop.call_soon(self.fire_event, 'start')
             except Exception as exc:
                 self.fire_event('start', exc=exc)
 
-    def stop_serving(self):
+    def close(self):
         '''Stop serving the :attr:`.Server.sockets`.
         '''
         if self._server:
             server, self._server = self._server, None
-            server.close()
-
-    @task
-    def close(self):
-        '''Stop serving the :attr:`.Server.sockets` and close all
-        concurrent connections.
-        '''
-        if not self.fired_event('stop'):
-            if self._server:
-                server, self._server = self._server, None
-                server.close()
-                yield From(None)
-                yield From(self._close_connections())
-            self.fire_event('stop')
+            return self._close(server)
+        elif not self.fired_event('stop'):
+            return self.fire_event('stop')
 
     def info(self):
         sockets = []
@@ -646,10 +689,10 @@ class TcpServer(Producer):
     def create_protocol(self):
         '''Override :meth:`Producer.create_protocol`.
         '''
-        protocol = super(TcpServer, self).create_protocol(
-            timeout=self._keep_alive)
+        protocol = super().create_protocol(timeout=self._keep_alive)
         protocol.bind_event('connection_made', self._connection_made)
         protocol.bind_event('connection_lost', self._connection_lost)
+        protocol.copy_many_times_events(self)
         if (self._server and self._max_requests and
                 self._sessions >= self._max_requests):
             self.logger.info('Reached maximum number of connections %s. '
@@ -674,17 +717,28 @@ class TcpServer(Producer):
         all = []
         if connection:
             all.append(connection.event('connection_lost'))
-            connection.transport.close()
+            connection.close()
         else:
             connections = list(self._concurrent_connections)
             self._concurrent_connections = set()
             for connection in connections:
                 all.append(connection.event('connection_lost'))
-                connection.transport.close()
+                connection.close()
         if all:
             self.logger.info('%s closing %d connections', self, len(all))
             return multi_async(all)
 
+    @task
+    def _close(self, server):
+        '''Stop serving the :attr:`.Server.sockets` and close all
+        concurrent connections.
+        '''
+        server.close()
+        coro = self._close_connections()
+        if coro:
+            yield from coro
+        self.fire_event('stop')
+
 
 class DatagramServer(Producer):
     '''An :class:`.Producer` for serving UDP sockets.
@@ -704,9 +758,8 @@ class DatagramServer(Producer):
     def __init__(self, protocol_factory, loop=None, address=None,
                  name=None, sockets=None, max_requests=None,
                  logger=None):
-        super(DatagramServer, self).__init__(loop, protocol_factory, name=name,
-                                             max_requests=max_requests,
-                                             logger=logger)
+        super().__init__(loop, protocol_factory, name=name,
+                         max_requests=max_requests, logger=logger)
         self._params = {'address': address, 'sockets': sockets}
 
     @task
@@ -727,9 +780,9 @@ class DatagramServer(Producer):
                         proto = self.create_protocol()
                         transports.append(transport(self._loop, proto))
                 else:
-                    transport, _ = yield From(
-                        self._loop.create_datagram_endpoint(
-                            self.protocol_factory, local_addr=address))
+                    loop = self._loop
+                    transport, _ = yield from loop.create_datagram_endpoint(
+                        self.protocol_factory, local_addr=address)
                     transports.append(transport)
                 self._transports = transports
                 self._started = self._loop.time()
