@@ -24,14 +24,15 @@ import argparse
 import os
 import textwrap
 import logging
+import pickle
+import types
 
 from pulsar import __version__, SERVER_NAME
 from . import system
 from .internet import parse_address
 from .importer import import_system_file
-from .httpurl import HttpParser as PyHttpParser
-from .log import configured_logger, get_level
-from .pep import to_bytes, iteritems, native_str, pickle
+from .log import configured_logger
+from .pep import to_bytes
 
 
 __all__ = ['Config',
@@ -78,12 +79,17 @@ def ordered_settings():
         yield KNOWN_SETTINGS[name]
 
 
+simple_values = (list, tuple, float, int, dict, str, types.FunctionType)
+
+
 def valid_config_value(val):
-    try:
-        pickle.loads(pickle.dumps(val))
-        return True
-    except Exception:
-        return False
+    if isinstance(val, simple_values):
+        try:
+            pickle.loads(pickle.dumps(val))
+            return True
+        except Exception:
+            pass
+    return False
 
 
 class Config(object):
@@ -101,9 +107,9 @@ class Config(object):
         as in the :class:`argparse.ArgumentParser` class.
     :param apps: list of application namespaces to include in the
         :attr:`settings` dictionary. For example if ``apps`` is set to
-        ``['socket', 'tasks']``, the
+        ``['socket', 'test']``, the
         :ref:`socket server <setting-section-socket-servers>` and
-        :ref:`task queue <setting-section-task-consumer>` settings are
+        :ref:`task queue <setting-section-test>` settings are
         loaded in addition to the standard
         :ref:`global settings <setting-section-global-server-settings>`,
         :ref:`worker settings <setting-section-worker-processes>` and
@@ -154,7 +160,7 @@ class Config(object):
         return name in self.settings
 
     def items(self):
-        for k, setting in iteritems(self.settings):
+        for k, setting in self.settings.items():
             yield k, setting.value
 
     def __getstate__(self):
@@ -177,7 +183,7 @@ class Config(object):
     def __setattr__(self, name, value):
         if name != "settings" and name in self.settings:
             raise AttributeError("Invalid access!")
-        super(Config, self).__setattr__(name, value)
+        super().__setattr__(name, value)
 
     def update(self, data, default=False):
         '''Update this :attr:`Config` with ``data``.
@@ -241,11 +247,8 @@ class Config(object):
 
         :rtype: an instance of :class:`ArgumentParser`.
         '''
-        kwargs = {
-            "description": self.description,
-            "epilog": self.epilog
-        }
-        parser = argparse.ArgumentParser(**kwargs)
+        parser = argparse.ArgumentParser(description=self.description,
+                                         epilog=self.epilog)
         parser.add_argument('--version',
                             action='version',
                             version=self.version)
@@ -255,7 +258,10 @@ class Config(object):
         '''Add this container :attr:`settings` to an existing ``parser``.
         '''
         setts = self.settings
-        sorter = lambda x: (setts[x].section, setts[x].order)
+
+        def sorter(x):
+            return (setts[x].section, setts[x].order)
+
         for k in sorted(setts, key=sorter):
             setts[k].add_argument(parser)
         return parser
@@ -281,9 +287,28 @@ class Config(object):
                     if valid_config_value(val):
                         unknowns.append((k, val))
                 else:
-                    self.set(kl, val)
+                    self.set(kl, val, True)
         return unknowns
 
+    def parse_command_line(self, argv=None):
+        '''Parse the command line
+        '''
+        if self.config:
+            parser = argparse.ArgumentParser(add_help=False)
+            self.settings['config'].add_argument(parser)
+            opts, _ = parser.parse_known_args(argv)
+            if opts.config is not None:
+                self.set('config', opts.config)
+
+            self.params.update(self.import_from_module())
+
+        parser = self.parser()
+        opts = parser.parse_args(argv)
+        for k, v in opts.__dict__.items():
+            if v is None:
+                continue
+            self.set(k.lower(), v)
+
     def on_start(self):
         '''Invoked by a :class:`.Application` just before starting.
         '''
@@ -296,7 +321,10 @@ class Config(object):
 
     @property
     def workers(self):
-        return self.settings['workers'].get()
+        '''Number of workers
+        '''
+        workers = self.settings.get('workers')
+        return workers.get() if workers else 0
 
     @property
     def address(self):
@@ -355,45 +383,28 @@ class Config(object):
     def clone(self):
         return pickle.loads(pickle.dumps(self))
 
-    def configured_logger(self, name=None, replace=False):
+    def configured_logger(self, name=None):
         '''Configured logger.
         '''
-        loggers = {}
-        internal_level = get_level('warning')
         loghandlers = self.loghandlers
-        # base name is always pulsar
-        basename = 'pulsar'
-        # the namespace which uses the default log level
-        # (the one without namespace prefix)
-        defname = self.log_name or basename
-        # the namespace name for this config
-        name = name or self.name
-        if name:
-            name = '%s.%s' % (basename, name)
+        # logname
+        if not name:
+            # base name is always pulsar
+            basename = 'pulsar'
+            # the namespace name for this config
+            name = self.name
+            if name and name != basename:
+                name = '%s.%s' % (basename, name)
+            else:
+                name = basename
         #
         namespaces = {}
 
         for loglevel in self.loglevel or ():
             bits = loglevel.split('.')
-            namespaces['.'.join(bits[:-1]) or defname] = bits[-1]
-
-        if defname not in namespaces:
-            namespaces[defname] = self.settings['loglevel'].default[0]
-
-        deflevel = get_level(namespaces[defname])
-        internal_level = max(deflevel, internal_level) if deflevel else 0
-        for namespace in (basename, 'asyncio'):
-            if namespace not in namespaces:
-                namespaces[namespace] = internal_level
-        # Trollius same as asyncio
-        namespaces['trollius'] = namespaces['asyncio']
-
-        if name and name not in namespaces:
-            namespaces[name] = namespaces[basename]
+            namespaces['.'.join(bits[:-1]) or ''] = bits[-1]
 
         for namespace in sorted(namespaces):
-            if namespace in loggers:
-                continue
             if self.daemon:     # pragma    nocover
                 handlers = []
                 for hnd in loghandlers:
@@ -402,14 +413,11 @@ class Config(object):
                 if not handlers:
                     handlers.append('file')
                 loghandlers = handlers
-            logger = configured_logger(namespace,
-                                       config=self.logconfig,
-                                       level=namespaces[namespace],
-                                       handlers=loghandlers,
-                                       rootlevel=internal_level,
-                                       replace=replace)
-            loggers[namespace] = logger
-        return loggers.get(name)
+            configured_logger(namespace,
+                              config=self.logconfig,
+                              level=namespaces[namespace],
+                              handlers=loghandlers)
+        return logging.getLogger(name)
 
     def __copy__(self):
         return self.copy()
@@ -474,8 +482,7 @@ class SettingMeta(type):
         setattr(cls, "short", '' if not lines else lines[0])
 
 
-# This works for Python 2 and Python 3
-class Setting(SettingMeta('BaseSettings', (object,), {'virtual': True})):
+class Setting(metaclass=SettingMeta):
     '''Class for creating :ref:`pulsar settings <settings>`.
 
     Most parameters can be specified on the command line,
@@ -648,7 +655,6 @@ class TestOption(Setting):
 def validate_bool(val):
     if isinstance(val, bool):
         return val
-    val = native_str(val)
     if not isinstance(val, str):
         raise TypeError("Invalid type for casting: %s" % val)
     if val.lower().strip() == "true":
@@ -678,12 +684,11 @@ def validate_pos_float(val):
 
 
 def validate_string(val):
-    va = native_str(val)
-    if va is None:
+    if val is None:
         return None
-    if not isinstance(va, str):
+    if not isinstance(val, str):
         raise TypeError("Not a string: %s" % val)
-    return va.strip()
+    return val.strip()
 
 
 def validate_list(val):
@@ -779,28 +784,12 @@ class HttpProxyServer(Global):
             os.environ['wss_proxy'] = self.value
 
 
-class HttpParser(Global):
-    name = "http_py_parser"
-    flags = ["--http-py-parser"]
-    action = "store_true"
-    default = False
-    desc = '''\
-        Set the python parser as default HTTP parser
-    '''
-
-    def on_start(self):
-        if self.value:  # pragma    nocover
-            from pulsar.utils.httpurl import setDefaultHttpParser
-            setDefaultHttpParser(PyHttpParser)
-
-
 class Debug(Global):
     name = "debug"
     flags = ["--debug"]
-    nargs = '?'
-    type = int
-    default = 0
-    const = 1
+    validator = validate_bool
+    action = "store_true"
+    default = False
     desc = """\
         Turn on debugging in the server.
 
@@ -1016,20 +1005,6 @@ class UseGreenlet(Global):
     Use greenlet whenever possible.
     '''
 
-
-class PyCares(Global):
-    name = 'no_pycares'
-    flags = ['--no-pycares']
-    validator = validate_bool
-    action = "store_true"
-    default = False
-    desc = '''\
-    Switch off pycares DNS lookup.
-
-    By default, pulsar uses pycares for its DNS lookups (if it is available).
-    Use this flag to revert to the standard library dns resolver.
-    '''
-
 ############################################################################
 #    Worker Processes
 section_docs['Worker Processes'] = '''
@@ -1104,9 +1079,9 @@ class ThreadWorkers(Setting):
     flags = ["--thread-workers"]
     validator = validate_pos_int
     type = int
-    default = 1
+    default = 5
     desc = """\
-        The number of threads used by the actor event loop executor.
+        Maximum number of threads used by the actor event loop executor.
 
         The executor is a thread pool used by the event loop to perform CPU
         intensive operations or when it needs to execute blocking calls.
