@@ -1,6 +1,31 @@
 '''
 A list of all Exception specific to pulsar library.
 '''
+import traceback
+
+from .httpurl import Headers
+
+
+__all__ = ['PulsarException',
+           'MonitorStarted',
+           'ImproperlyConfigured',
+           'CommandError',
+           'CommandNotFound',
+           'ProtocolError',
+           'EventAlreadyRegistered',
+           'InvalidOperation',
+           'HaltServer',
+           'HTTPError',
+           'SSLError',
+           # HTTP Exceptions
+           'HttpException',
+           'HttpRedirect',
+           'BadRequest',
+           'PermissionDenied',
+           'MethodNotAllowed',
+           'Http404',
+           #
+           'format_traceback']
 
 
 class PulsarException(Exception):
@@ -30,8 +55,7 @@ class CommandError(PulsarException):
 class CommandNotFound(CommandError):
 
     def __init__(self, name):
-        super(CommandNotFound, self).__init__('Command "%s" not available' %
-                                              name)
+        super().__init__('Command "%s" not available' % name)
 
 
 class ProtocolError(PulsarException):
@@ -43,7 +67,7 @@ class ProtocolError(PulsarException):
     status_code = None
 
     def ProtocolError(self, msg=None, status_code=None):
-        super(ProtocolError, self).__init__(msg)
+        super().__init__(msg)
         self.status_code = status_code
 
 
@@ -62,7 +86,7 @@ class HaltServer(BaseException):
     When ``exit_code`` is greater than 1, it is considered an expected
     failure and therefore the full stack trace is not logged.'''
     def __init__(self, reason='Exiting server.', exit_code=3):
-        super(HaltServer, self).__init__(reason)
+        super().__init__(reason)
         self.exit_code = exit_code
 
 
@@ -100,13 +124,13 @@ class HttpException(HTTPError):
         self.code = self.status  # for compatibility with HTTPError
         self.handler = handler
         self.strict = strict
-        self.headers = headers
         self.content_type = content_type
-        super(HttpException, self).__init__(msg)
+        self._headers = Headers.make(headers)
+        super().__init__(msg)
 
     @property
-    def hdrs(self):
-        return self.headers
+    def headers(self):
+        return list(self._headers)
 
 
 class HttpRedirect(HttpException):
@@ -117,18 +141,15 @@ class HttpRedirect(HttpException):
     status = 302
 
     def __init__(self, location, status=None, headers=None, **kw):
-        headers = [] if headers is None else headers
-        headers.append(('location', location))
-        super(HttpRedirect, self).__init__(status=status or self.status,
-                                           headers=headers, **kw)
+        headers = Headers.make(headers)
+        headers['location'] = location
+        super().__init__(status=status or self.status, headers=headers, **kw)
 
     @property
     def location(self):
         '''The value in the ``Location`` header entry.
-
-        Equivalent to ``self.headers['location']``.
         '''
-        return self.headers['location']
+        return self._headers['location']
 
 
 class BadRequest(HttpException):
@@ -148,3 +169,7 @@ class Http404(HttpException):
 class MethodNotAllowed(HttpException):
     '''An :class:`HttpException` with default ``405`` status code.'''
     status = 405
+
+
+def format_traceback(exc):
+    return traceback.format_exception(exc.__class__, exc, exc.__traceback__)
