@@ -42,19 +42,21 @@ import os
 import sys
 import re
 import string
-import time
 import mimetypes
-import platform
-import socket
 from hashlib import sha1, md5
 from uuid import uuid4
 from email.utils import formatdate
 from io import BytesIO
 import zlib
-from collections import deque
+from collections import deque, OrderedDict
+from urllib import request as urllibr
+from http import client as httpclient
+from urllib.parse import quote, urlsplit, splitport
+from http.cookiejar import CookieJar, Cookie
+from http.cookies import SimpleCookie
 
-from .structures import mapping_iterator, OrderedDict
-from .pep import ispy3k, iteritems, itervalues, to_bytes, native_str
+from .structures import mapping_iterator
+from .string import to_bytes, to_string
 from .html import capfirst
 
 # try:
@@ -69,15 +71,6 @@ from .html import capfirst
 hasextensions = False
 _Http_Parser = None
 
-try:
-    from select import poll, POLLIN
-except ImportError:   # pragma    nocover
-    poll = False
-    try:
-        from select import select
-    except ImportError:  # pragma    nocover
-        select = False
-
 
 def setDefaultHttpParser(parser):   # pragma    nocover
     global _Http_Parser
@@ -89,8 +82,6 @@ def http_parser(**kwargs):
     return _Http_Parser(**kwargs)
 
 
-create_connection = socket.create_connection
-
 try:    # Compiled with SSL?
     BaseSSLError = None
     ssl = None
@@ -99,81 +90,9 @@ try:    # Compiled with SSL?
 except (ImportError, AttributeError):   # pragma : no cover
     pass
 
-if ispy3k:  # Python 3
-    from urllib import request as urllibr
-    from http import client as httpclient
-    from urllib.parse import (quote, unquote, urlencode, urlparse, urlsplit,
-                              parse_qs, parse_qsl, splitport, urlunparse,
-                              urljoin)
-    from http.client import responses
-    from http.cookiejar import CookieJar, Cookie
-    from http.cookies import SimpleCookie
-
-    string_type = str
-    getproxies_environment = urllibr.getproxies_environment
-    ascii_letters = string.ascii_letters
-    chr = chr
-    is_string = lambda s: isinstance(s, str)
-
-    def force_native_str(s, encoding=None):
-        if isinstance(s, bytes):
-            return s.decode(encoding or 'utf-8')
-        elif not isinstance(s, str):
-            return str(s)
-        else:
-            return s
-
-else:   # pragma : no cover
-    import urllib2 as urllibr
-    import httplib as httpclient
-    from urllib import (quote, unquote, urlencode, getproxies_environment,
-                        splitport)
-    from urlparse import (urlparse, urlsplit, parse_qs, urlunparse, urljoin,
-                          parse_qsl)
-    from httplib import responses
-    from cookielib import CookieJar, Cookie
-    from Cookie import SimpleCookie
-
-    string_type = unicode
-    ascii_letters = string.letters
-    chr = unichr
-    is_string = lambda s: isinstance(s, unicode)
-
-    if sys.version_info < (2, 7):
-        #
-        def create_connection(address, timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
-                              source_address=None):
-            """Form Python 2.7"""
-            host, port = address
-            err = None
-            for res in socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM):
-                af, socktype, proto, canonname, sa = res
-                sock = None
-                try:
-                    sock = socket.socket(af, socktype, proto)
-                    if timeout is not socket._GLOBAL_DEFAULT_TIMEOUT:
-                        sock.settimeout(timeout)
-                    if source_address:
-                        sock.bind(source_address)
-                    sock.connect(sa)
-                    return sock
-                except Exception as _:
-                    err = _
-                    if sock is not None:
-                        sock.close()
-            if err is not None:
-                raise err
-            else:
-                raise Exception("getaddrinfo returns an empty list")
-
-    def force_native_str(s, encoding=None):
-        if isinstance(s, unicode):
-            return s.encode(encoding or 'utf-8')
-        elif not isinstance(s, str):
-            return str(s)
-        else:
-            return s
 
+getproxies_environment = urllibr.getproxies_environment
+ascii_letters = string.ascii_letters
 HTTPError = urllibr.HTTPError
 URLError = urllibr.URLError
 request_host = urllibr.request_host
@@ -199,14 +118,20 @@ URI_UNRESERVED_SET = frozenset(ascii_letters + string.digits + '-._~')
 URI_SAFE_CHARS = URI_RESERVED_CHARS + '%~'
 HEADER_TOKEN_CHARS = frozenset("!#$%&'*+-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                                '^_`abcdefghijklmnopqrstuvwxyz|~')
+MAX_CHUNK_SIZE = 65536
 
-escape = lambda s: quote(s, safe='~')
-urlquote = lambda iri: quote(iri, safe=URI_RESERVED_CHARS)
+
+def escape(s):
+    return quote(s, safe='~')
+
+
+def urlquote(iri):
+    return quote(iri, safe=URI_RESERVED_CHARS)
 
 
 def _gen_unquote(uri):
     unreserved_set = URI_UNRESERVED_SET
-    for n, part in enumerate(force_native_str(uri, 'latin1').split('%')):
+    for n, part in enumerate(to_string(uri, 'latin1').split('%')):
         if not n:
             yield part
         else:
@@ -248,8 +173,8 @@ def iri_to_uri(iri, kwargs=None):
     if iri is None:
         return iri
     if kwargs:
-        iri = '%s?%s' % (force_native_str(iri, 'latin1'),
-                         '&'.join(('%s=%s' % kv for kv in iteritems(kwargs))))
+        iri = '%s?%s' % (to_string(iri, 'latin1'),
+                         '&'.join(('%s=%s' % kv for kv in kwargs.items())))
     return urlquote(unquote_unreserved(iri))
 
 
@@ -444,7 +369,6 @@ header_parsers = {'Connection': split_comma,
 
 
 def header_values(header, value):
-    value = native_str(value)
     assert isinstance(value, str)
     if header in header_parsers:
         return header_parsers[header](value)
@@ -460,7 +384,7 @@ def quote_header_value(value, extra_chars='', allow_token=True):
     :param allow_token: if this is enabled token values are returned
         unchanged.
     """
-    value = force_native_str(value)
+    value = to_string(value)
     if allow_token:
         token_chars = HEADER_TOKEN_CHARS | set(extra_chars)
         if set(value).issubset(token_chars):
@@ -525,7 +449,36 @@ def parse_dict_header(value):
     return result
 
 
-class Headers(object):
+_special = re.escape('()<>@,;:\\"/[]?={} \t')
+_re_special = re.compile('[%s]' % _special)
+_qstr = '"(?:\\\\.|[^"])*"'  # Quoted string
+_value = '(?:[^%s]+|%s)' % (_special, _qstr)  # Save or quoted string
+_option = '(?:;|^)\s*([^%s]+)\s*=\s*(%s)' % (_special, _value)
+_re_option = re.compile(_option)  # key=value part of an Content-Type header
+
+
+def header_unquote(val, filename=False):
+    if val[0] == val[-1] == '"':
+        val = val[1:-1]
+        if val[1:3] == ':\\' or val[:2] == '\\\\':
+            val = val.split('\\')[-1]  # fix ie6 bug: full path --> filename
+        return val.replace('\\\\', '\\').replace('\\"', '"')
+    return val
+
+
+def parse_options_header(header, options=None):
+    if ';' not in header:
+        return header.lower().strip(), {}
+    ctype, tail = header.split(';', 1)
+    options = options or {}
+    for match in _re_option.finditer(tail):
+        key = match.group(1).lower()
+        value = header_unquote(match.group(2), key == 'filename')
+        options[key] = value
+    return ctype, options
+
+
+class Headers:
     '''Utility for managing HTTP headers for both clients and servers.
 
     It has a dictionary like interface with few extra functions to facilitate
@@ -559,6 +512,12 @@ class Headers(object):
     The strict parameter is rarely used and it forces the omission on
     non-standard header fields.
     '''
+    @classmethod
+    def make(cls, headers):
+        if not isinstance(headers, cls):
+            headers = cls(headers=headers)
+        return headers
+
     def __init__(self, headers=None, kind='server', strict=False):
         if isinstance(kind, int):
             kind = header_type.get(kind, 'both')
@@ -744,7 +703,7 @@ class Headers(object):
 
     def __iter__(self):
         dj = ', '
-        for k, values in iteritems(self._headers):
+        for k, values in self._headers.items():
             joiner = HEADER_FIELDS_JOINER.get(k, dj)
             if joiner:
                 yield k, joiner.join(values)
@@ -840,7 +799,9 @@ class HttpParser(object):
         self.__on_headers_complete = False
         self.__on_message_begin = False
         self.__on_message_complete = False
+        # decompress
         self.__decompress_obj = None
+        self.__decompress_first_try = True
 
     @property
     def kind(self):
@@ -915,8 +876,8 @@ class HttpParser(object):
                 else:
                     self.__on_firstline = True
                     self._buf.append(data[:idx])
-                    first_line = native_str(b''.join(self._buf),
-                                            DEFAULT_CHARSET)
+                    first_line = to_string(b''.join(self._buf),
+                                           DEFAULT_CHARSET)
                     rest = data[idx+2:]
                     data = b''
                     if self._parse_firstline(first_line):
@@ -1021,7 +982,7 @@ class HttpParser(object):
         idx = data.find(b'\r\n\r\n')
         if idx < 0:  # we don't have all headers
             return False
-        chunk = native_str(data[:idx], DEFAULT_CHARSET)
+        chunk = to_string(data[:idx], DEFAULT_CHARSET)
         # Split lines on \r\n keeping the \r\n on each line
         lines = deque(('%s\r\n' % line for line in chunk.split('\r\n')))
         # Parse headers into key/value pairs paying attention
@@ -1077,8 +1038,10 @@ class HttpParser(object):
             encoding = self._headers['Content-Encoding'][0]
             if encoding == "gzip":
                 self.__decompress_obj = zlib.decompressobj(16+zlib.MAX_WBITS)
+                self.__decompress_first_try = False
             elif encoding == "deflate":
                 self.__decompress_obj = zlib.decompressobj()
+
         rest = data[idx+4:]
         self._buf = [rest]
         self.__on_headers_complete = True
@@ -1096,9 +1059,10 @@ class HttpParser(object):
             else:
                 if self._clen_rest is not None:
                     self._clen_rest -= len(data)
+
                 # maybe decompress
-                if self.__decompress_obj is not None:
-                    data = self.__decompress_obj.decompress(data)
+                data = self._decompress(data)
+
                 self._partial_body = True
                 if data:
                     self._body.append(data)
@@ -1118,8 +1082,9 @@ class HttpParser(object):
             if size is None or len(rest) < size + 2:
                 return None
             body_part, rest = rest[:size], rest[size:]
-            if self.__decompress_obj is not None:
-                body_part = self.__decompress_obj.decompress(body_part)
+
+            # maybe decompress
+            body_part = self._decompress(body_part)
             self._partial_body = True
             self._body.append(body_part)
             rest = rest[2:]
@@ -1146,11 +1111,36 @@ class HttpParser(object):
         if data[:2] == b'\r\n':
             self._trailers = self._parse_headers(data[:idx])
 
+    def _decompress(self, data):
+        deco = self.__decompress_obj
+        if deco is not None:
+            if not self.__decompress_first_try:
+                data = deco.decompress(data)
+            else:
+                try:
+                    data = deco.decompress(data)
+                except zlib.error:
+                    self.__decompress_obj = zlib.decompressobj(-zlib.MAX_WBITS)
+                    deco = self.__decompress_obj
+                    data = deco.decompress(data)
+                self.__decompress_first_try = False
+        return data
+
+
 if not hasextensions:   # pragma    nocover
     setDefaultHttpParser(HttpParser)
 
 
 # ############################################    UTILITIES, ENCODERS, PARSERS
+absolute_http_url_re = re.compile(r"^https?://", re.I)
+
+
+def is_absolute_uri(location):
+    '''Check if a ``location`` is absolute, i.e. it includes the scheme
+    '''
+    return location and absolute_http_url_re.match(location)
+
+
 def get_environ_proxies():
     """Return a dict of environment proxies. From requests_."""
 
@@ -1165,7 +1155,9 @@ def get_environ_proxies():
         'no'
     ]
 
-    get_proxy = lambda k: os.environ.get(k) or os.environ.get(k.upper())
+    def get_proxy(k):
+        return os.environ.get(k) or os.environ.get(k.upper())
+
     proxies = [(key, get_proxy(key + '_proxy')) for key in proxy_keys]
     return dict([(key, val) for (key, val) in proxies if val])
 
@@ -1248,7 +1240,7 @@ def http_date(epoch_seconds=None):
     return formatdate(epoch_seconds, usegmt=True)
 
 
-# ################################################################# COOKIE
+# ################################################################# COOKIES
 def create_cookie(name, value, **kwargs):
     """Make a cookie from underspecified parameters.
 
@@ -1286,7 +1278,6 @@ def cookiejar_from_dict(*cookie_dicts):
 
     :param cookie_dict: Dict of key/values to insert into CookieJar.
     """
-    jars = []
     cookie_dicts = tuple((d for d in cookie_dicts if d))
     if len(cookie_dicts) == 1 and isinstance(cookie_dicts[0], CookieJar):
         return cookie_dicts[0]
@@ -1301,18 +1292,19 @@ def cookiejar_from_dict(*cookie_dicts):
     return cookiejar
 
 
+# ################################################################# VARY HEADER
 cc_delim_re = re.compile(r'\s*,\s*')
 
 
 def patch_vary_headers(response, newheaders):
-    """\
-Adds (or updates) the "Vary" header in the given HttpResponse object.
-newheaders is a list of header names that should be in "Vary". Existing
-headers in "Vary" aren't removed.
+    """Adds (or updates) the "Vary" header in the given HttpResponse object.
 
-For information on the Vary header, see:
+    newheaders is a list of header names that should be in "Vary". Existing
+    headers in "Vary" aren't removed.
 
-    http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.44
+    For information on the Vary header, see:
+
+        http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.44
     """
     # Note that we need to keep the original order intact, because cache
     # implementations may rely on the order of the Vary contents in, say,
@@ -1373,3 +1365,25 @@ class CacheControl(object):
                 headers.add_header('cache-control', 'proxy-revalidate')
         else:
             headers['cache-control'] = 'no-cache'
+
+
+def chunk_encoding(chunk):
+    '''Write a chunk::
+
+        chunk-size(hex) CRLF
+        chunk-data CRLF
+
+    If the size is 0, this is the last chunk, and an extra CRLF is appended.
+    '''
+    head = ("%X\r\n" % len(chunk)).encode('utf-8')
+    return head + chunk + b'\r\n'
+
+
+def http_chunks(data, finish=False):
+    while len(data) >= MAX_CHUNK_SIZE:
+        chunk, data = data[:MAX_CHUNK_SIZE], data[MAX_CHUNK_SIZE:]
+        yield chunk_encoding(chunk)
+    if data:
+        yield chunk_encoding(data)
+    if finish:
+        yield chunk_encoding(data)
