@@ -1,7 +1,7 @@
 import os
 import sys
 import glob
-from importlib import *
+from importlib import *     # noqa
 
 
 def expand_star(mod_name):
@@ -16,7 +16,7 @@ def expand_star(mod_name):
     return expanded
 
 
-def import_modules(modules):
+def import_modules(modules, safe=True):
     '''Safely import a list of *modules*
     '''
     all = []
@@ -28,8 +28,9 @@ def import_modules(modules):
         for module in to_load:
             try:
                 all.append(import_module(module))
-            except ImportError as e:
-                pass
+            except ImportError:
+                if not safe:
+                    raise
     return all
 
 
@@ -44,7 +45,7 @@ def module_attribute(dotpath, default=None, safe=False):
         try:
             module = import_module('.'.join(bits[:-1]))
             return getattr(module, bits[-1], default)
-        except Exception as e:
+        except Exception:
             if not safe:
                 raise
             return default
@@ -88,3 +89,8 @@ def import_system_file(mod, add_to_path=True):
         else:
             mod_name = '.'.join(names)
         return import_module(mod_name)
+    else:
+        try:
+            return import_module(mod)
+        except ImportError:
+            pass
