@@ -3,30 +3,17 @@ Module containing utilities and mixins for logging and serialisation.
 '''
 import sys
 import logging
+from logging.config import dictConfig
 from copy import deepcopy, copy
-from time import time
 from threading import Lock
 from functools import wraps
 
-from .system import current_process, platform
-
-
-from .pep import force_native_str
+from .system import current_process
+from .string import to_string
+from .structures import AttributeDictionary
 
 win32 = sys.platform == "win32"
 
-if sys.version_info < (2, 7):    # pragma    nocover
-    from .fallbacks._dictconfig import dictConfig
-
-    class NullHandler(logging.Handler):
-        def emit(self, record):
-            pass
-else:
-    from logging.config import dictConfig
-    from logging import NullHandler
-
-from .structures import AttributeDictionary
-
 
 LOGGING_CONFIG = {
     'version': 1,
@@ -68,19 +55,22 @@ LOGGING_CONFIG = {
         }
     },
     'filters ': {},
-    'loggers': {},
+    'loggers': {
+        'asyncio': {
+            'level': 'WARNING'
+        }
+    },
     'root': {}
 }
 
 
-if not platform.is_appengine:
+def file_handler(**kw):
+    return logging.FileHandler('pulsar.log', **kw)
 
-    def file_handler(**kw):
-        return logging.FileHandler('pulsar.log', **kw)
 
-    LOGGING_CONFIG['handlers']['file'] = {
-        '()': file_handler,
-        'formatter': 'verbose'}
+LOGGING_CONFIG['handlers']['file'] = {
+    '()': file_handler,
+    'formatter': 'verbose'}
 
 
 def update_config(config, c):
@@ -133,7 +123,7 @@ class LazyString:
 
     def __str__(self):
         if self.value is None:
-            self.value = force_native_str(self.f(*self.args, **self.kwargs))
+            self.value = to_string(self.f(*self.args, **self.kwargs))
         return self.value
     __repr__ = __str__
 
@@ -224,48 +214,58 @@ class Silence(logging.Handler):
         pass
 
 
-def configured_logger(name, config=None, level=None, handlers=None,
-                      rootlevel=None, replace=False):
+def clear_logger():
+    process_global('_config_logging', None, True)
+
+
+def configured_logger(name=None, config=None, level=None, handlers=None):
     '''Configured logger.
     '''
+    name = name or ''
     with process_global('lock'):
-        logconfig = original = process_global('_config_logging')
+        logconfig = process_global('_config_logging')
         # if the logger was not configured, do so.
-        if not logconfig or replace:
-            logconfig = deepcopy(LOGGING_CONFIG)
-            if config:
-                update_config(logconfig, config)
-            original = logconfig
+        if not logconfig:
+            logconfig = deepcopy(config or LOGGING_CONFIG)
+            logconfig['configured'] = set()
             process_global('_config_logging', logconfig, True)
         else:
-            loggers = logconfig.get('loggers')
-            if loggers and name in loggers:
+            configured = logconfig.get('configured')
+            if name in configured:
                 return logging.getLogger(name)
-            logconfig = deepcopy(logconfig)
-            logconfig['disable_existing_loggers'] = False
-            logconfig.pop('loggers', None)
-            logconfig.pop('root', None)
 
         level = get_level(level)
-        rootlevel = get_level(rootlevel or level)
         # No loggers configured. This means no logconfig setting
         # parameter was used. Set up the root logger with default
         # loggers
         if level == logging.NOTSET:
             handlers = ['silent']
-        else:
-            handlers = handlers or ['console']
+
         level = logging.getLevelName(level)
-        if 'loggers' not in logconfig:
-            logconfig['loggers'] = {}
-        l = {'level': level, 'handlers': handlers, 'propagate': False}
-        original['loggers'][name] = l
-        logconfig['loggers'][name] = l
+        cfg = {'level': level, 'propagate': False}
+        if handlers:
+            cfg['handlers'] = handlers
+
+        config = copy(logconfig)
+        configured = config.pop('configured')
+        configured.add(name)
+
+        if name:
+            config.pop('root', None)
+            loggers = config.pop('logger', {})
+            if name in loggers:
+                loggers[name].update(cfg)
+                cfg = loggers[name]
+            config['loggers'] = {name: cfg}
+        else:
+            if 'root' in config:
+                config['root'].update(cfg)
+                cfg = config['root']
+            if 'handlers' not in cfg:
+                cfg['handlers'] = ['console']
+            config['root'] = cfg
         #
-        if not original.get('root'):
-            logconfig['root'] = {'handlers': handlers, 'level': rootlevel}
-        if logconfig:
-            dictConfig(logconfig)
+        dictConfig(config)
         return logging.getLogger(name)
 
 
