@@ -1,15 +1,13 @@
 '''Stand alone compact module for managing python paths.'''
 import os
 import sys
-
-from .importer import import_module
-from .pep import range, string_type
+from importlib import import_module
 
 
 __all__ = ['Path']
 
 
-class Path(string_type):
+class Path(str):
     '''A lightweight utility for the filesystem and python modules.'''
     def __new__(cls, path=None):
         path = path or ''
@@ -21,10 +19,17 @@ class Path(string_type):
         '''Return the current working directory as a path object.'''
         return cls(os.getcwd())
 
-    isfile = lambda self: os.path.isfile(self)
-    isdir = lambda self: os.path.isdir(self)
-    exists = lambda self: os.path.exists(self)
-    realpath = lambda self: self.__class__(os.path.realpath(string_type(self)))
+    def isfile(self):
+        return os.path.isfile(self)
+
+    def isdir(self):
+        return os.path.isdir(self)
+
+    def exists(self):
+        return os.path.exists(self)
+
+    def realpath(self):
+        return self.__class__(os.path.realpath(str(self)))
 
     def join(self, *path):
         return self.__class__(os.path.join(self, *path))
@@ -75,15 +80,17 @@ class Path(string_type):
                    must_exist=True):
         '''Add a directory to the python path.
 
-:parameter module: Optional module name to try to import once we have found
-    the directory
-:parameter up: number of level to go up the directory three from
-    :attr:`local_path`.
-:parameter down: Optional tuple of directory names to travel down once we have
-    gone *up* levels.
-:parameter front: Boolean indicating if we want to insert the new path at the
-    front of ``sys.path`` using ``sys.path.insert(0,path)``.
-:parameter must_exist: Boolean indicating if the module must exists.'''
+        :parameter module: Optional module name to try to import once we
+            have found the directory
+        :parameter up: number of level to go up the directory three from
+            :attr:`local_path`.
+        :parameter down: Optional tuple of directory names to travel down
+            once we have gone *up* levels.
+        :parameter front: Boolean indicating if we want to insert the new
+            path at the front of ``sys.path`` using
+            ``sys.path.insert(0,path)``.
+        :parameter must_exist: Boolean indicating if the module must exists.
+        '''
         if module:
             try:
                 return import_module(module)
