@@ -46,11 +46,10 @@ Zset
    :members:
    :member-order: bysource
 '''
-from collections import *
+from collections import *       # noqa
 
-from .skiplist import Skiplist
-from .zset import Zset
-from .misc import (MultiValueDict, AttributeDictionary, FrozenDict,
-                   Dict, Deque, merge_prefix, recursive_update,
-                   mapping_iterator, inverse_mapping, isgenerator,
-                   aslist)
+from .skiplist import Skiplist  # noqa
+from .zset import Zset          # noqa
+from .misc import (MultiValueDict, AttributeDictionary, FrozenDict,  # noqa
+                   Dict, Deque, merge_prefix, recursive_update,  # noqa
+                   mapping_iterator, inverse_mapping, aslist)    # noqa
