@@ -36,30 +36,33 @@ parse_close
 import os
 from struct import pack, unpack
 from array import array
+from base64 import b64encode
 
-from .pep import ispy3k, range, to_bytes
+from .pep import to_bytes
 from .exceptions import ProtocolError
+from .httpurl import DEFAULT_CHARSET
 
 try:
-    from .lib import FrameParser as CFrameParser, CLOSE_CODES
+    from .lib import FrameParser as CFrameParser
 except:     # pragma    nocover
     CFrameParser = None
 
-    CLOSE_CODES = {
-        1000: "OK",
-        1001: "going away",
-        1002: "protocol error",
-        1003: "unsupported type",
-        # 1004: - (reserved)
-        # 1005: no status code (internal)
-        # 1006: connection closed abnormally (internal)
-        1007: "invalid data",
-        1008: "policy violation",
-        1009: "message too big",
-        1010: "extension required",
-        1011: "unexpected error",
-        # 1015: TLS failure (internal)
-    }
+
+CLOSE_CODES = {
+    1000: "OK",
+    1001: "going away",
+    1002: "protocol error",
+    1003: "unsupported type",
+    # 1004: - (reserved)
+    # 1005: no status code (internal)
+    # 1006: connection closed abnormally (internal)
+    1007: "invalid data",
+    1008: "policy violation",
+    1009: "message too big",
+    1010: "extension required",
+    1011: "unexpected error",
+    # 1015: TLS failure (internal)
+}
 
 
 DEFAULT_VERSION = 13
@@ -67,12 +70,6 @@ SUPPORTED_VERSIONS = (DEFAULT_VERSION,)
 WS_EXTENSIONS = {}
 WS_PROTOCOLS = {}
 
-if ispy3k:
-    string_type = str
-
-else:  # pragma    nocover
-    string_type = basestring
-
 
 def get_version(version):
     try:
@@ -84,6 +81,10 @@ def get_version(version):
     return version
 
 
+def websocket_key():
+    return b64encode(os.urandom(16)).decode(DEFAULT_CHARSET)
+
+
 class Extension(object):
 
     def receive(self, data):
@@ -109,7 +110,7 @@ def frame_parser(version=None, kind=0, extensions=None, protocols=None,
     version = get_version(version)
     Parser = FrameParser if pyparser else CFrameParser
     # extensions, protocols
-    return Parser(version, kind, ProtocolError, None, None)
+    return Parser(version, kind, ProtocolError, close_codes=CLOSE_CODES)
 
 
 def websocket_mask(data, masking_key):
@@ -118,7 +119,7 @@ def websocket_mask(data, masking_key):
     data = array('B', data)
     for i in range(len(data)):
         data[i] ^= key[i % mask_size]
-    return data.tobytes() if ispy3k else data.tostring()
+    return data.tobytes()
 
 
 class Frame:
@@ -182,7 +183,8 @@ class FrameParser(object):
           by the client)
         * 2 Assumes always unmasked data
     '''
-    def __init__(self, version, kind, ProtocolError, extensions, protocols):
+    def __init__(self, version, kind, ProtocolError, extensions=None,
+                 protocols=None, close_codes=None):
         self.version = version
         self.kind = kind
         self.frame = None
@@ -200,6 +202,7 @@ class FrameParser(object):
         self._max_payload = 1 << 63
         self._extensions = extensions
         self._protocols = protocols
+        self._close_codes = close_codes or CLOSE_CODES
 
     @property
     def max_payload(self):
@@ -233,7 +236,8 @@ class FrameParser(object):
         '''return a `close` :class:`Frame`.
         '''
         code = code or 1000
-        body = pack('!H', code) + CLOSE_CODES.get(code, '').encode('utf-8')
+        body = pack('!H', code)
+        body += self._close_codes.get(code, '').encode('utf-8')
         return self.encode(body, opcode=0x8)
 
     def continuation(self, body=None, final=True):
@@ -252,7 +256,7 @@ class FrameParser(object):
         return self._encode(data, opcode, masking_key, fin,
                             rsv1, rsv2, rsv3)
 
-    def multi_encode(self, message, masking_key=None, opcode=-1,
+    def multi_encode(self, message, masking_key=None, opcode=None,
                      rsv1=0, rsv2=0, rsv3=0, max_payload=0):
         '''Encode a ``message`` into several frames depending on size.
 
@@ -282,9 +286,9 @@ class FrameParser(object):
             chunk = self._chunk(2)
             first_byte, second_byte = unpack("BB", chunk)
             fin = (first_byte >> 7) & 1
-            rsv1 = (first_byte >> 6) & 1
-            rsv2 = (first_byte >> 5) & 1
-            rsv3 = (first_byte >> 4) & 1
+            # rsv1 = (first_byte >> 6) & 1
+            # rsv2 = (first_byte >> 5) & 1
+            # rsv3 = (first_byte >> 4) & 1
             opcode = first_byte & 0xf
             if fin not in (0, 1):
                 raise ProtocolError('FIN must be 0 or 1')
@@ -371,7 +375,7 @@ class FrameParser(object):
         else:
             masking_key = b''
         if opcode is None:
-            opcode = 1 if isinstance(message, string_type) else 2
+            opcode = 1 if isinstance(message, str) else 2
         data = to_bytes(message or b'', 'utf-8')
         if opcode not in self._opcodes:
             raise ProtocolError('WEBSOCKET opcode a reserved value')
